/******************************************************************************
 * A x e l                                                                   *
 ******************************************************************************
 * ObjectManager                                                             *
 * 2007-04-13                                                                *
 * Julien Wintz                                                              *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                   *
 ******************************************************************************
 * Comments :                                                                *
 *****************************************************************************/

# ifndef OBJECTMANAGER_H
# define OBJECTMANAGER_H

# include <QObject>
# include <QList>
# include <QAxel/QAxelObject.h>

namespace QAxel {

/****************************************************************************/
/*!
  \class ObjectManager ObjectManager.h
  \brief This class manages all Items in Axel.

  This class is a \e singleton that manages all \link QAxelObject objetcs \endlink in
  Axel. An \c Item is a toplevel graphical item that can be viewed in a
  \link Viewer rendering-area \endlink. They are usually results of some
  operation by the user and consist of a number of smaller components made
  up of the lower-level primitives. This class manages all these items in a
  uniform manner and provides for easy methods for adding, removing and
  finding items.

  To use the class you must access it using the method 
  \link instance() instance() \endlink. Such as:
  \code
  // get pointer to instance
  ObjectManager * G = ObjectManager::instance ();

  // do something
  G->find (....);
  \endcode
*/
/****************************************************************************/
class ObjectManager : public QObject
{
    Q_OBJECT

public:
    ObjectManager ();
    ~ObjectManager();
 
    static ObjectManager * instance     ();

    void addObject(QAxelObject * object) ;
    void removeObject(QAxelObject * object) ;

    QAxelObject * findByName (const QString & name);
    QAxelObject * findByType (const QString & type);

    QList<QAxelObject *> objects(void) ;

public: signals:
    void objectAdded(QAxelObject *) ;
    void objectRemoved(QAxelObject *) ;

private:
    static ObjectManager * m_instance ;
    QList<QAxelObject *> m_items ;
} ;

} ; // namespace QAxel ;

# endif  // OBJECTMANAGER_H
 
