/*****************************************************************************
 * Axel
 *****************************************************************************
 * Curves in Axel
 * 2006-04-18
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QCURVE_H
# define QCURVE_H

# include <QAxel/QAxelObject.h>

namespace QAxel {

class QDisplayList ;
class QBoundingBox ;

/****************************************************************************/
/*!
  \class QCurve QCurve.h
  \brief The class QCurve is an abstract base class for curves

  More ...
*/
/****************************************************************************/
class QCurve : public QAxelObject 
{
    Q_OBJECT
	
public:
     QCurve(void) ;
     QCurve(QBoundingBox * boundingbox) ;
     QCurve(double xmin, double xmax, 
	    double ymin, double ymax,
	    double zmin, double zmax) ;
    ~QCurve(void) ;
    
    virtual void draw(void) = 0 ;
    virtual void drawWithNames(void) = 0 ;	

    bool bbDrawn(void) { 
	return bbdrawn_ ; 
    }
    
    void setBBDrawn(bool b) { 
	bbdrawn_ = b ; 
    }
    
    QBoundingBox * boundingbox(void) { 
	return boundingbox_ ; 
    }

    void setBoundingBox(QBoundingBox * box) {
	boundingbox_ = box ;
    }
    
public slots:
    QAxelObject * toObject(void) { 
	return (QAxelObject *)this ; 
    }
	
protected:
    QBoundingBox * boundingbox_ ;
    bool bbdrawn_ ;

protected:
    bool           c_dirty ;
    QDisplayList * c_dlist ;
} ;

} ; // namespace QAxel

# endif
