/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPiecewiseLinearCurve
 * 2007-05-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QDisplayList.h>
# include <QAxel/QPiecewiseLinearCurve.h>

namespace QAxel {

QPiecewiseLinearCurve::QPiecewiseLinearCurve(void) : QCurve()
{
    QString str ;
    setProperty("type", "PiecewiseLinearCurve") ;
    setProperty("name", "plcurve_"+str.setNum(id_)) ;
}

QPiecewiseLinearCurve::QPiecewiseLinearCurve(QBoundingBox * bb) : QCurve(bb)
{
    QString str ;
    setProperty("type", "PiecewiseLinearCurve") ;
    setProperty("name", "plcurve_"+str.setNum(id_)) ;
}

QPiecewiseLinearCurve::QPiecewiseLinearCurve(double xmin, double xmax, 
					     double ymin, double ymax,
					     double zmin, double zmax) : QCurve(xmin, xmax,
										ymin, ymax,
										zmin, zmax)
{
    QString str ;
    setProperty("type", "PiecewiseLinearCurve") ;
    setProperty("name", "plcurve_"+str.setNum(id_)) ;
}

QPiecewiseLinearCurve::~QPiecewiseLinearCurve(void)
{

}

void QPiecewiseLinearCurve::draw(void)
{
    if(c_dirty || !c_dlist->isValid()) {
	c_dlist->begin() ;
	
	if(mode() == QAxelObject::Edit || mode() == QAxelObject::Extrude)
	    foreach(QVertex * v, c_vertices)
		v->draw() ;

	foreach(QEdge * e, c_edges)
	    e->draw() ;

	c_dlist->end() ;
    }
    c_dlist->draw() ;
}

void QPiecewiseLinearCurve::drawWithNames(void)
{
    if(mode() == QAxelObject::Edit || mode() == QAxelObject::Extrude) {
	foreach(QVertex * v, c_vertices)
	    v->drawWithNames() ;
    } else {
	glPushName((GLuint)this) ;
	c_dlist->draw() ;
	glPopName() ;
    }
}

bool QPiecewiseLinearCurve::closed(void)
{
    return false ;
}

bool QPiecewiseLinearCurve::opened(void)
{
    return true ;
}

void QPiecewiseLinearCurve::sort(void)
{
    
}

void QPiecewiseLinearCurve::push(QVertex * vertex)
{
    vertex->setParent(this) ;
    c_vertices << vertex ;
}

void QPiecewiseLinearCurve::push(QEdge * edge)
{
    edge->setParent(this) ;
    c_edges << edge ;
}

void QPiecewiseLinearCurve::pop(QVertex * vertex)
{
    c_vertices.removeAll(vertex) ;
}

void QPiecewiseLinearCurve::pop(QEdge * edge)
{
    c_edges.removeAll(edge) ;
}

void QPiecewiseLinearCurve::clear(void) 
{
    c_vertices.clear() ;
    c_edges.clear() ;
}

int QPiecewiseLinearCurve::id_ = 0 ;

} ; // namespace QAxel
