# include <QAxel/QPlane.h>

namespace QAxel {

int QPlane::id_ = 0 ;

QPlane::QPlane(float a, float b, float c, float d, float size) 
{
	this->a_ = a ;
	this->b_ = b ;
	this->c_ = c ;
	this->d_ = d ;
	this->size_ = size ;
	
	computePosition();
	computeOrientation();
	
	// Selection matter
	QString str;
	setProperty("type", "Plane");
	setProperty("name", "plane_"+str.setNum(id_)) ;
	id_++ ;
	
	setObjectColor(new QColor(0,0,255,128)) ;
	// setSelectionColor(new QColor(192,128,128,128)) ;
}

float QPlane::size(void) { return size_ ; }
void QPlane::setSize(float size) { size_ = size ; }

float QPlane::a()
{
	GLdouble m[4][4];
	frame_->getMatrix(m);
	a_=m[1][0];
	return a_;
}

float QPlane::b()
{
	GLdouble m[4][4];
	frame_->getMatrix(m);
	b_=m[1][1];
	return b_;
}

float QPlane::c()
{
	GLdouble m[4][4];
	frame_->getMatrix(m);
	c_=m[1][2];
	return c_;
}

float QPlane::d()
{
	Vec v1 = frame_->position();
	GLdouble m[4][4];
	frame_->getMatrix(m);
	Vec v2(m[1][0],m[1][1],m[1][2]);
	v2.normalize();
	d_=-(v1*v2);
	return d_;
}

void QPlane::seta(float a)
{
	a_=a;
	computePosition();
	computeOrientation();
}

void QPlane::setb(float b)
{
	b_=b;
	computePosition();
	computeOrientation();
}

void QPlane::setc(float c)
{
	c_=c;
	computePosition();
	computeOrientation();
}

void QPlane::setd(float d)
{
	d_=d;
	computePosition();
	computeOrientation();
}

void QPlane::computePosition(void)
{
	Vec v(a_,b_,c_);
	double norm = v.normalize();
	if (norm)
		frame_->setPosition(v*(-d_/norm));
	else
		frame_->setPosition(v*(-d_));
}

void QPlane::computeOrientation(void)
{
	qglviewer::Vec from(0.0,1.0,0.0);
	qglviewer::Vec to(a_,b_,c_);
	qglviewer::Quaternion q(from, to);
	frame_->setOrientation(q);
}

void QPlane::draw(void)
{
	if(hidden()) return ;
	glPushMatrix() ;
	glMultMatrixd(frame_->matrix()) ; // Multiply matrix to get in the frame coordinate system.
	
	glDisable(GL_LIGHTING) ;

	glColorAlpha() ;
	
	glBegin(GL_POLYGON);
		glNormal3f(a_,b_,c_);
		glVertex3f(-size_, 0, -size_) ; 
		glVertex3f(size_, 0, -size_) ;
		glVertex3f(size_, 0, size_) ;
		glVertex3f(-size_, 0, size_) ;
	glEnd() ;
		
	glColor3ub(200, 200, 200) ; // the plane's frame color
	glBegin(GL_LINE_LOOP) ;
		glVertex3f(-size_, 0, -size_) ; 
		glVertex3f(size_, 0, -size_) ;
		glVertex3f(size_, 0, size_) ;
		glVertex3f(-size_, 0, size_) ;
	glEnd() ;
	
	glPopMatrix() ;
	
	glEnable(GL_LIGHTING) ;
}

void QPlane::drawWithNames(void)
{
	glPushName((GLuint)this);
	draw();
	glPopName();
}

QString QPlane::equation(void)
{
    QString a ; if(fabs(a_)<1e-5) a = "0.00001" ; else a.setNum(a_, 'f', 3) ;
    QString b ; if(fabs(b_)<1e-5) b = "0.00001" ; else b.setNum(b_, 'f', 3) ;
    QString c ; if(fabs(c_)<1e-5) c = "0.00001" ; else c.setNum(c_, 'f', 3) ;
    QString d ; if(fabs(d_)<1e-5) d = "0.00001" ; else d.setNum(d_, 'f', 3) ;
    QString equation ;
    equation = a + "x +  " + b + "y + " + c + "z + " + d ;
    return equation ;
}

} ; // namespace QAxel
