/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPoint4f
 * 2007-04-18
 * Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QAxel/QPoint4f.h>

namespace QAxel {

int QPoint4f::id_ = 0 ;
QPoint4f::QPoint4f(float s, float t, float u, float v)
{
	p_s = s;
	p_t = t;
	p_u = u;
	p_v = v;
	QString str ;
	setProperty("type", "Point4f") ;
	setProperty("name", "point_"+str.setNum(id_)) ;
	id_++ ;
	setObjectColor(new QColor(0,255,0)) ;
	p_size = 0.03 ;
}

/* Constructor with shuffled variables. The integer i represents the variable corresponding to the first given coordinate (var0). 
More precisely :
if i = 0, then var0 = s, var1 = t, var2 = u, var3 = v 
if i = 1, then var0 = t, var1 = u, var2 = v, var3 = s
if i = 2, then var0 = u, var1 = v, var2 = s, var3 = t
if i = 3, then var0 = v, var1 = s, var2 = t, var3 = u */
QPoint4f::QPoint4f(int i, float var0, float var1, float var2, float var3)
{
	switch (i)
	{
		case 0:
			p_s = var0;
			p_t = var1;
			p_u = var2;
			p_v = var3;
		break;
		case 1:
			p_s = var3;
			p_t = var0;
			p_u = var1;
			p_v = var2;
		break;
		case 2:
			p_s = var2;
			p_t = var3;
			p_u = var0;
			p_v = var1;
		break;
		case 3:
			p_s = var1;
			p_t = var2;
			p_u = var3;
			p_v = var0;
		break;
		default: qCritical("Problem in the shuffle constructor of QPoint4f") ;
		break;
	}
	QString str ;
	setProperty("type", "Point4f") ;
	setProperty("name", "point_"+str.setNum(id_)) ;
	id_++ ;
	setObjectColor(new QColor(0,255,0)) ;
	p_size = 0.03 ;
}

QPoint4f::~QPoint4f(void) {}

float QPoint4f::size(void) { return p_size ; }
void QPoint4f::setSize(float size) { p_size = size ; }

void QPoint4f::draw(void)
{
	// if(hidden()) return ;
	// static GLUquadric* quad = gluNewQuadric() ;
	// glPushMatrix() ;
	// 	glMultMatrixd(frame_->matrix()) ;
	// 	glColor() ;
	// 	gluSphere(quad, p_size, 30, 15) ;
	// glPopMatrix() ;
}

void QPoint4f::drawWithNames(void)
{
	// glPushName((GLuint)this) ;
	// draw();
	// glPopName() ;
}

QString QPoint4f::toString(void)
{
    QString s ; s.setNum(p_s) ;
    QString t ; t.setNum(p_t) ;
    QString u ; u.setNum(p_u) ;
    QString v ; v.setNum(p_v) ;
    QString r = "(" + s + ", " + t + ", " + u + ", " + v + ")" ;
    return r ;
}

bool QPoint4f::equals(QPoint4f * p) 
{
	bool r = true ;
	if(p->s() != s()) r = false ;
	if(p->t() != t()) r = false ;
	if(p->u() != u()) r = false ;
	if(p->v() != v()) r = false ;
	return r ;
}

bool QPoint4f::equals(QPoint4f * p, double tolerance)
{
	bool r = true ;
	if(fabs(p->s()-s()) > tolerance) r = false ;
	if(fabs(p->t()-t()) > tolerance) r = false ;
	if(fabs(p->u()-u()) > tolerance) r = false ;
	if(fabs(p->v()-v()) > tolerance) r = false ;
	return r ;
}

std::ostream& operator << (std::ostream& stream, QPoint4f & p)
{
    return stream << p.toString().toLatin1().data() ;
}

std::ostream& operator << (std::ostream& stream, QPoint4f * p)
{
    return stream << p->toString().toLatin1().data() ;
}

} ; // namespace QAxel
