/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QCorePlugin
 * 2006-11-06
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# ifndef QCOREPLUGIN_H
# define QCOREPLUGIN_H

# include <QAxel/QAxelObject.h>
# include <QAxel/QShape.h>

# include <QCore/QCoreInterface.h>

# include <synaps/shape/interfaces/IShapeFactory.h>

using namespace QAxel ;

/****************************************************************************/
/*!
  \class QCorePlugin QCorePlugin.h
  \brief The class QCorePlugin implements the \link QCoreInterface \endlink
  as a glue to the Synaps library.

  More ...
*/
/****************************************************************************/
class QCorePlugin : public QObject, public QCoreInterface
{
    Q_OBJECT
    Q_INTERFACES(QCoreInterface QAxel::QPluginInterface QAxel::QInitializablePluginInterface)
	
public:
    QString name() const ;
    QStringList features() const ;
	
    void init(MainWindow * window) ;

    void create(QImplicitCurve * c) ;
    void create(QImplicitSurface * s) ;
    void create(QRationalCurve * c) ;
    void create(QRationalSurface * s) ;
    void create(QBSplineCurve * c) ;
    void create(QBSplineSurface * s) ;
	
    QPoint3f * eval(QAxel::QRationalCurve * c, float u) ;
    QPoint3f * eval(QAxel::QRationalSurface * s, float u, float v) ;
    QPoint3f * eval(QAxel::QBSplineCurve * c, float u) ;
    QPoint3f * eval(QAxel::QBSplineSurface * s, float u, float v) ;

    QShape * intersect(QAxelObject * s1, QAxelObject * s2) ;
    QShape * autointersect(QAxelObject * s) ;
    
    void draw(QAxelObject * o) ;
    
    void getGlData(QBSplineSurface * s, int * nbi, int * indices, int * nbv, double * vertices) ;
	
private:
    QMap<QAxelObject *, IShape *> map ;
    IShapeFactory * factory;
} ;

# endif
