/*****************************************************************************
 * Axel
 *****************************************************************************
 * QCell4D
 * 2007-05-02
 * Stephane CHAU
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 * 
 * 
 * 
 ****************************************************************************/

# ifndef QCell4D4D_H
# define QCell4D4D_H

# include <QData/QBoundingBox4D.h>
# include <QAxel/QPoint4f.h>

using namespace QAxel ;

namespace QData {

/****************************************************************************/
/*!
  \class QCell4D QCell4D.h
  \brief A cell in a subdivision process.

  To apply dynamic programming using cells, they also contain relevant 
  informations as its intersection points with the objects it contains,
  intersections of objects its contains as well as singularities of these
  objects.
*/
/****************************************************************************/
class QCell4D : public QBoundingBox4D
{
    Q_OBJECT

public:
    QCell4D(void) ;
    QCell4D(QBoundingBox4D * bb) ;
    QCell4D(float smin, float smax, float tmin=0, float tmax=0, float umin=0, float umax=0, float vmin=0, float vmax=0) ;
    
    QList<QPoint4f *> singularPoints(void) { return c_singularities ; }
    void pushSingularPoint(QPoint4f * p) { c_singularities << p ; }
    void setSingularPoints(QList<QPoint4f *> l) { c_singularities = l ; }

    QList<QPoint4f *> sMinIntersections(void) { return smin_intersections ; }
    QList<QPoint4f *> sMaxIntersections(void) { return smax_intersections ; }
    QList<QPoint4f *> tMinIntersections(void) { return tmin_intersections ; }
    QList<QPoint4f *> tMaxIntersections(void) { return tmax_intersections ; }
    QList<QPoint4f *> uMinIntersections(void) { return umin_intersections ; }
    QList<QPoint4f *> uMaxIntersections(void) { return umax_intersections ; }
    QList<QPoint4f *> vMinIntersections(void) { return vmin_intersections ; }
    QList<QPoint4f *> vMaxIntersections(void) { return vmax_intersections ; }
	QList<QPoint4f *> allIntersections(void);
    
    void pushSMinIntersection(QPoint4f * p) { smin_intersections << p ; }
    void pushSMaxIntersection(QPoint4f * p) { smax_intersections << p ; }
    void pushTMinIntersection(QPoint4f * p) { tmin_intersections << p ; }
    void pushTMaxIntersection(QPoint4f * p) { tmax_intersections << p ; }
    void pushUMinIntersection(QPoint4f * p) { umin_intersections << p ; }
    void pushUMaxIntersection(QPoint4f * p) { umax_intersections << p ; }
    void pushVMinIntersection(QPoint4f * p) { vmin_intersections << p ; }
    void pushVMaxIntersection(QPoint4f * p) { vmax_intersections << p ; }
    
    void setSMinIntersections(QList<QPoint4f *> l) { smin_intersections = l ; }
    void setSMaxIntersections(QList<QPoint4f *> l) { smax_intersections = l ; }
    void setTMinIntersections(QList<QPoint4f *> l) { tmin_intersections = l ; }
    void setTMaxIntersections(QList<QPoint4f *> l) { tmax_intersections = l ; }
    void setUMinIntersections(QList<QPoint4f *> l) { umin_intersections = l ; }
    void setUMaxIntersections(QList<QPoint4f *> l) { umax_intersections = l ; }
    void setVMinIntersections(QList<QPoint4f *> l) { vmin_intersections = l ; }
    void setVMaxIntersections(QList<QPoint4f *> l) { vmax_intersections = l ; }

    

    static float abs(float a) ;
    static QCell4D * intersection(QCell4D * c1, QCell4D * c2) ;
    QList<QCell4D *> subdivide4D(QPoint4f * center) ;
    
private:
    QList<QPoint4f *> c_singularities ; /*! Singularities in the cell */
	QList<QPoint4f *> smin_intersections;
	QList<QPoint4f *> smax_intersections;
	QList<QPoint4f *> tmin_intersections;
	QList<QPoint4f *> tmax_intersections;
	QList<QPoint4f *> umin_intersections;
	QList<QPoint4f *> umax_intersections;
	QList<QPoint4f *> vmin_intersections;
	QList<QPoint4f *> vmax_intersections;    
private:
    static int id_ ;
} ;

} ; // namespace QData

# endif // QCell4D_H
