/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QGraphNode
 * 2006-10-04
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/
 
# ifndef QGRAPHNODE_H
# define QGRAPHNODE_H

# include <QColor>
# include <QString>
# include <QList>

namespace QData {

class QCell ;	
class QRegion3f ;
class QGraphEdge ;

/****************************************************************************/
/*!
  \class QGraphNode QGraphNode.h
  \brief The class QGraphNode provides ...

  More ...
*/
/****************************************************************************/
class QGraphNode 
{	
public:
    QGraphNode(QString label, QRegion3f * region = NULL) ;
    QGraphNode(QString label, QCell * cell) ;

    enum Type { Graph, Tree } ;
    Type n_type ;
    
    QString label(void) ;
    
    bool isLeaf(void) ;
    
    void addEdge(QGraphEdge * edge);
    void takeEdge(QGraphEdge * edge) ;
    
    QList<QGraphNode *> parents(void) ;
    QList<QGraphNode *> children(void) ;
    QList<QGraphNode *> inNodesList(void) ;
    QList<QGraphNode *> outNodesList(void) ;
    QList<QGraphEdge *> edges() const;
    QList<QGraphEdge *> adjacentEdges() const;
    
    QColor lightColor(void) ;
    QColor darkColor(void) ;
    void setColors(QColor lightColor, QColor darkColor) ;
    void setLightColor(QColor lightColor) ;
    void setDarkColor(QColor darkColor) ;
    void resetColors(void) ;
    void setAlpha(int alpha) ;
    
    void setPosition(float x, float y) ;
    
    QRegion3f * region(void) ;
    QCell * cell(void) ;
    
private:
    float xpos, ypos ;
    QString label_ ;
    
    QRegion3f * region_ ;
    QCell * cell_ ;
    QColor lightColor_, darkColor_ ;
    QList<QGraphEdge *> edgeList;
} ;

} ; // namespace QData

# endif // QGRAPHNODE_H
