/******************************************************************************
 * A x e l                                                                    *
 ******************************************************************************
 * QParametricCurveSegmentation                                               *
 * 2007-06-21                                                                 *
 * Julien Wintz                                                               *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                    *
 ******************************************************************************
 * Comments :                                                                 *
 ******************************************************************************/
 
# ifndef QPARAMETRICCURVESEGMENTATION_H
# define QPARAMETRICCURVESEGMENTATION_H

# include <QAxel/QAxelObject.h>
# include <QAxel/QDisplayList.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QParametricCurve.h>
# include <QData/QBinaryTree.h>

using namespace QAxel ;

namespace QData {
    
typedef QTreeNode<QParametricCurve *, QBoundingBox *> QSegmentationNode_t ;
typedef QPair<QSegmentationNode_t *, QSegmentationNode_t *> QSegmentationQuery_t ;

class QParametricCurveSegmentation : public QAxelObject
{
    Q_OBJECT

public:
    QParametricCurveSegmentation(QParametricCurve * curve) ;

    void draw(void) ;
    void drawWithNames(void) ;

    bool isValid(void) ;

public:
    void build(void) ;
    QSegmentationNode_t * root(void) ;
    QList<QSegmentationNode_t *> leaves(void) ;

public:
    static QList<QBoundingBox *>   intersect(QParametricCurveSegmentation * s1, QParametricCurveSegmentation * s2) ;
    static QList<QBoundingBox *>   intersect(QSegmentationNode_t * n1, QSegmentationNode_t * n2) ;
    static QList<QSegmentationQuery_t> query(QParametricCurveSegmentation * r1, QParametricCurveSegmentation * r2) ;
    static QList<QSegmentationQuery_t> query(QSegmentationNode_t * n1, QSegmentationNode_t * n2) ;
    static QList<QSegmentationQuery_t> refine(QSegmentationQuery_t q) ;

protected:
    QSegmentationNode_t * alloc(unsigned size) ;
    QSegmentationNode_t * make(void) ;

private:
    QSegmentationNode_t * s_tree ;
    QSegmentationNode_t * s_root ;

private:
    bool s_isValid ;
    QParametricCurve * s_curve ;

private:
    QDisplayList * s_dlist ;

private:
    static int id_ ;
} ;

} ; // namespace QData

# endif // QPARAMETRICCURVESEGMENTATION_H
