/******************************************************************************
 * A x e l                                                                    *
 ******************************************************************************
 * QRegion                                                                    *
 * 2006-10-14                                                                 *
 * Julien Wintz                                                               *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                    *
 ******************************************************************************
 * Comments :                                                                 *
 ******************************************************************************/
 
# ifndef QREGION3F_H
# define QREGION3F_H

# include <QList>

# include <QAxel/QAxelObject.h>
# include <QAxel/QDisplayList.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QCurve.h>

using namespace QAxel ;

class Viewer ;

namespace QData {

class QCell ;
class QRegionVertex ;
class QRegionEdge ;
class QRegionSegmentation ;

/****************************************************************************/
/*!
  \class QRegion3f QRegion3f.h
  \brief The class QRegion3f provides ...

  More ...
*/
/****************************************************************************/
class QRegion3f : public QAxelObject
{	
    Q_OBJECT
    Q_ENUMS(Operation)
	
public:
    QRegion3f(void) ;
    ~QRegion3f(void) ;
    
    void draw(void) ;
    void drawWithNames(void) ;

    enum Operation { Union, Intersection } ;
    
public slots:	
    QList<QRegionVertex *> vertices(void) ;
    QList<QRegionEdge   *> edges(void) ;
    QList<QAxelObject   *> determinors(void) ;
    
    QRegionVertex * topMostVertex(void) ;
    QRegionVertex * leftMostVertex(void) ;
    QRegionVertex * rightMostVertex(void) ;
    QRegionVertex * bottomMostVertex(void) ;

    void addVertex(QRegionVertex * v) ;
    void addVertices(QList<QRegionVertex *> l) ;
    void addEdge(QRegionVertex * v1, QRegionVertex * v2, QAxelObject * o) ;
    void addEdge(QRegionEdge * e) ;
    void addDeterminor(QAxelObject * o) ;
    void addConflictor(QAxelObject * o) ;
    void removeVertex(QRegionVertex * v) ;
    void mergeWith(QRegion3f * other) ;
    void sort(void) ;

    int firstIndexInLoopFrom(int index) ;
    int lastIndexInLoopFrom(int index) ;

    bool containsHoles(void) ;
    bool opened(void) ;
    bool closed(void) ;
    bool mergesWith(QRegion3f * other, QString type) ;
    bool intersects(QRegion3f * other) ;
    bool unites(QRegion3f * other) ;
    bool contains(QPoint3f * p) ;

    QList<QRegion3f *> intersect(QRegion3f * other) ;
    QList<QRegion3f *> unite(QRegion3f * other) ;
    
    QList<QRegionVertex *> commonVerticesOf(QRegion3f * r) ;
    QList<QRegionVertex *> uncommonVerticesOf(QRegion3f * r) ;
    QList<QRegionVertex *> commonVerticesWith(QRegion3f * r) ;
    QList<QRegionVertex *> fusionEdges(void) ;

    QList<QRegionEdge *> commonEdgesOf(QRegion3f * r) ;
    QList<QRegionEdge *> commonEdgesWith(QRegion3f * r) ;
    QList<QRegionEdge *> uncommonEdgesOf(QRegion3f * r) ;
    QList<QRegionEdge *> edgesWith(QRegionVertex * v) ;
    
    QString toString(void) ;
    friend std::ostream& operator <<(std::ostream& stream, QRegion3f & r) ;
    friend std::ostream& operator <<(std::ostream& stream, QRegion3f * r) ;
    
    QCell * cell(void) ;
    void setCell(QCell * cell) ;

    QRegionSegmentation * segmentation(void) { return r_segmentation ; }
    
protected:	
    void deleteVerticesIn(QList<QRegionVertex *> l) ;
    void deleteEdgesIn(QList<QRegionVertex *> l) ;
    void deleteEdgesIn(QList<QRegionEdge *> l) ;
    void applyMap(QMap<QRegionVertex *, QRegionVertex *> map) ;
    float consecutivePointPosition(QPoint3f * intersection, QRegionEdge * e, QRegion3f * r) ;
    QRegionEdge * nextRegionFlag(QList<QPoint3f *> intersectionPoints1, QList<QPoint3f *> intersectionPoints2, QRegionEdge * e1, QRegionEdge * e2, QBoundingBox * bb, Operation op) ;
    QList<QPoint3f *> ccwRepartition(QList<QPoint3f *> & points, QBoundingBox * bb) ;
    QPoint3f * xsmallest(QList<QPoint3f *>& l) ;
    QPoint3f * xgreatest(QList<QPoint3f *>& l) ;
    QPoint3f * ysmallest(QList<QPoint3f *>& l) ;
    QPoint3f * ygreatest(QList<QPoint3f *>& l) ;

private:
    QList<QRegionVertex *> r_vertices ;
    QList<QRegionEdge *>   r_edges ;
    
    QList<QAxelObject *> r_determinors ;
    QList<QAxelObject *> r_conflictors ;
    
    QCell * r_cell ;

    QRegionSegmentation * r_segmentation ;
	
    // QList<QConditions *> r_conditions ;

private:
    bool           r_dirty ;
    QDisplayList * r_dlist ;

private:
    static int id_ ;
} ;
    
} ; // namespace QData

# endif // QREGION3F_H
