/******************************************************************************
 * A x e l                                                                    *
 ******************************************************************************
 * QRegionSegmentation                                                        *
 * 2007-04-06                                                                 *
 * Julien Wintz                                                               *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                    *
 ******************************************************************************
 * Comments :                                                                 *
 ******************************************************************************/
 
# ifndef QREGIONSEGMENTATION_H
# define QREGIONSEGMENTATION_H

# include <QAxel/QAxelObject.h>
# include <QAxel/QDisplayList.h>
# include <QAxel/QBoundingBox.h>
# include <QData/QBinaryTree.h>

using namespace QAxel ;

namespace QData {
    
typedef QTreeNode<QRegionEdge *, QBoundingBox *> QSegmentationNode_t ;
typedef QPair<QSegmentationNode_t *, QSegmentationNode_t *> QSegmentationQuery_t ;

class QRegionSegmentation : public QAxelObject
{
    Q_OBJECT

public:
    QRegionSegmentation(QRegion3f * region) ;

    void draw(void) ;
    void drawWithNames(void) ;

    bool isValid(void) ;

public:
    void build(void) ;
    QSegmentationNode_t * root(void) ;
    QList<QSegmentationNode_t *> leaves(void) ;

public:
    static QList<QBoundingBox *>   intersect(QRegionSegmentation * s1, QRegionSegmentation * s2) ;
    static QList<QBoundingBox *>   intersect(QSegmentationNode_t * n1, QSegmentationNode_t * n2) ;
    static QList<QSegmentationQuery_t> query(QRegionSegmentation * r1, QRegionSegmentation * r2) ;
    static QList<QSegmentationQuery_t> query(QSegmentationNode_t * n1, QSegmentationNode_t * n2) ;
    static QList<QSegmentationQuery_t> refine(QSegmentationQuery_t q) ;

protected:
    QSegmentationNode_t * alloc(unsigned size) ;
    QSegmentationNode_t * make(void) ;

private:
    QSegmentationNode_t * s_tree ;
    QSegmentationNode_t * s_root ;

private:
    bool s_isValid ;
    QRegion3f * s_region ;

private:
    QDisplayList * s_dlist ;

private:
    static int id_ ;
} ;

} ; // namespace QData

# endif
