/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ArrangementDock
 * 2006-11-17
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <iostream>

# include <QtGui/QButtonGroup>
# include <QtGui/QSplitter>
# include <QtGui/QToolBox>
# include <QtGui/QHBoxLayout>
# include <QtGui/QWidget>
# include <QtGui/QGroupBox>
# include <QtGui/QLabel>

# include <QGui/MainWindow.h>
# include <QGui/Viewer.h>
# include <QGui/QGraphWidget.h>
# include <QGui/ArrangementDock.h>
# include <QGui/QAlgorithmWidget.h>

# include <QData/QGraph.h>

# include <QArrangement/QArrangement.h>

# include <QGLViewer/qglviewer.h>

ArrangementDock::ArrangementDock(const QString & title, QWidget * parent) : QDockWidget(title, parent) 
{
    mainwindow = dynamic_cast<MainWindow *>(parent) ;
    setObjectName("Arrangement") ;
    setupUi() ;
}

void ArrangementDock::play()
{

    mainwindow->algorithmWidget()->addElement("Arrangement", 
					      ":images/16x16/arrangementDeamon.png", 
					      "Arrangement deamon running") ;
	
    double multiFiltering   = acw->multiplicityFilteringRadius() ;
    double isolePrecision   = acw->isoleSolverPrecision() ;
    double sbdPrecision     = acw->sbdSolverPrecision() ;
    double cellPrecision    = acw->cellPrecision() ;
    double cellPrecisionMax = acw->cellPrecisionMax() ;
	
    algo = new QImplicitCurvesArrangement(mainwindow, isolePrecision, sbdPrecision, multiFiltering, cellPrecision, cellPrecisionMax) ;
    algo->setGraphicalCellsVerbosity(acw->cellVerbosity()) ;
    algo->setGraphicalFusionVerbosity(acw->fusionVerbosity()) ;
    graphWidget->setGraph(algo->graph()) ;

    connect(algo->graph(),        SIGNAL(changed()),                     graphWidget, SLOT(update())) ;
    connect(algo,                 SIGNAL(display(QGraph *)),             quad3Widget, SLOT(setGraph(QGraph *))) ;
    connect(mainwindow->viewer(), SIGNAL(objectSelected(QAxelObject *)), algo,        SLOT(setTree(QAxelObject *))) ;
    connect(mainwindow->viewer(), SIGNAL(noObjectSelected()),            quad3Widget, SLOT(clear())) ;
# ifdef Q_WS_MAC
    connect(mainwindow->viewer(), SIGNAL(objectAdded(QAxelObject *)),    qobject_cast<QArrangement::QArrangement *>(algo), SLOT(addObject(QAxelObject *))) ;
# else
    connect(mainwindow->viewer(), SIGNAL(objectAdded(QAxelObject *)),    algo,        SLOT(addObject(QAxelObject *))) ;
# endif

    // if(!algo->isRunning()) 	algo->start() ;
    // algo->play() ;
    algo->run() ;
}

void ArrangementDock::pause()
{
    mainwindow->algorithmWidget()->setPixmap("Arrangement", ":images/16x16/pausedDeamon.png") ;
    mainwindow->algorithmWidget()->setToolTip("Arrangement", "Arrangement deamon running") ;
	
    // algo->pause() ;
}

void ArrangementDock::stop()
{
    mainwindow->algorithmWidget()->removeElement("Arrangement") ;
    // algo->stop() ;
    // algo->wait() ;
}

void ArrangementDock::setupUi(void)
{
    splitter = new QSplitter(this) ;
    splitter->setOrientation(Qt::Horizontal) ;

    toolBox = new QToolBox(splitter) ;
    toolBox->setCurrentIndex(0) ;
    acw = new AlgorithmControlWidget(toolBox);
    toolBox->addItem(acw, "Algorithm Control");
    page2 = new QWidget();
    toolBox->addItem(page2, "Structure Visualization");
    aqw = new ArrangementQueriesControlWidget(toolBox);
    toolBox->addItem(aqw, "Queries");    
    toolBox->setMaximumSize(QSize(300, 16777215));
    toolBox->setMinimumSize(QSize(300, 100)) ;
    
    graphWidget = new QGraphWidget() ; graphWidget->setViewer(mainwindow->viewer()) ;
    quad3Widget = new QGraphWidget() ; quad3Widget->setViewer(mainwindow->viewer()) ;
	
    splitter->addWidget(toolBox) ;
    splitter->addWidget(graphWidget) ;
    splitter->addWidget(quad3Widget) ;
    
    connect(acw, SIGNAL(play()), this, SLOT(play())) ;
    connect(acw, SIGNAL(pause()), this, SLOT(pause())) ;
    connect(acw, SIGNAL(stop()), this, SLOT(stop())) ;
    connect(acw, SIGNAL(showSubdivisionCells()), this, SLOT(showSubdivisionCells())) ;
    connect(acw, SIGNAL(hideSubdivisionCells()), this, SLOT(hideSubdivisionCells())) ;
    
    connect(aqw, SIGNAL(enterClickLocationMode()), this, SLOT(enterClickLocationMode())) ;
    connect(aqw, SIGNAL(leaveClickLocationMode()), this, SLOT(leaveClickLocationMode())) ;
    connect(aqw, SIGNAL(clearLocation()), this, SLOT(clearLocation())) ;

    setWidget(splitter) ;
}

void ArrangementDock::enterClickLocationMode(void)
{
    connect(mainwindow->viewer(), SIGNAL(pointClicked(const QPoint &)), this, SLOT(clickLocation(const QPoint &))) ;
    mainwindow->viewer()->setOrthographicDisplay() ;
}

void ArrangementDock::leaveClickLocationMode(void)
{
    disconnect(mainwindow->viewer(), SIGNAL(pointClicked(const QPoint &)), this, SLOT(clickLocation(const QPoint &))) ;
    mainwindow->viewer()->setPerspectiveDisplay() ;
}

void ArrangementDock::clickLocation(const QPoint & p)
{
    using qglviewer::Camera ;
    using qglviewer::Vec ;

    Viewer * viewer = mainwindow->viewer() ;
    Camera * camera = mainwindow->viewer()->camera() ;
    
    Vec orig, dir ;
    camera->convertClickToLine(p, orig, dir) ;

    float t = - orig.z/dir.z ;

    //Vec poi = orig + camera->position().norm() * dir ;
    Vec poi = orig + t*dir ;

    QPoint3f * point = new QPoint3f(poi.x, poi.y, 0.0) ;
    
    viewer->addObject(point) ; locatedPoints << point ;
    
    algo->locate(point) ;
}

void ArrangementDock::clearLocation(void)
{
    foreach(QPoint3f * p, locatedPoints)
	mainwindow->viewer()->removeObject(p) ;    
    locatedPoints.clear() ;
    algo->graph()->resetColors() ;
}

void ArrangementDock::showSubdivisionCells(void)
{
    QList<QAxelObject *> cells = mainwindow->viewer()->listObjectsFromType("Cell") ;
    foreach(QAxelObject * o, cells)
	o->setHidden(false) ;
    mainwindow->viewer()->update() ;
}

void ArrangementDock::hideSubdivisionCells(void)
{
    QList<QAxelObject *> cells = mainwindow->viewer()->listObjectsFromType("Cell") ;
    foreach(QAxelObject * o, cells)
	o->setHidden(true) ;
    mainwindow->viewer()->update() ;
}
