/*****************************************************************************
 * Axel
 *****************************************************************************
 * MainWindow
 * 2005-10-18
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006-2007 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QApplication>
# include <QActionGroup>
# include <QLayout>
# include <QMessageBox>
# include <QMenuBar>
# include <QMessageBox>
# include <QFile>
# include <QFileDialog>     
# include <QStyleFactory>    
# include <QDomDocument>
# include <QDomElement>
# include <QSettings>
# include <QUndoStack>
# include <QUndoView>
# include <QLabel>
# include <QToolBar>
# include <QStatusBar>
# include <QSplashScreen>
# include <QDialogButtonBox>
# include <QTextEdit>
# include <QFrame>
# include <QProcess>
# include <QStackedWidget>

# include <QAxel/PluginManager.h>

# include <QGui/MainWindow.h>
# include <QGui/Viewer.h>
# include <QGui/SampleSelector.h>
# include <QGui/ObjectToolDock.h>
# include <QGui/ObjectPropertiesDock.h>
# include <QGui/ImplicitCurvesArrangementDock.h>
# include <QGui/ParametricCurvesArrangementDock.h>
# include <QGui/HistoryDock.h>
# include <QGui/ToolPropertiesDock.h>
# include <QGui/SceneInspectorDock.h>
# include <QGui/ScriptingDock.h>
# include <QGui/PluginDialog.h>
# include <QGui/PreferencesDialog.h>
# include <QGui/ImageViewer.h>
# include <QGui/QCameraModeWidget.h>
# include <QGui/QSelectedObjectsWidget.h>
# include <QGui/QAlgorithmWidget.h>
# include <QGui/QDimensionModeWidget.h>

# include <QStream/IStream/IStreamAxl.h>
# include <QStream/IStream/IStreamVect.h>
# include <QStream/IStream/IStreamOff.h>
# include <QStream/IStream/IStreamG2.h>
# include <QStream/IStream/IStreamGcml.h>
# include <QStream/OStream/OStreamGcml.h>
# include <QStream/OStream/OStreamAxl.h>
# include <QStream/OStream/OStreamYafray.h>
# include <QStream/OStream/OStreamSurf.h>

# ifdef Q_WS_MAC
# include <QGui/CocoaUtil.h>
# endif

# include "ui_AboutDialog.h"

using namespace std ;
using namespace QAxel ;
using namespace QStream ;

MainWindow::MainWindow() 
{
    undoStack_ = new QUndoStack(this) ;

    setStatusBar() ;
    createViewer() ;
    createDocks() ;
    createActions() ;
    createMenu() ;
    createToolBars() ;
    loadPlugins() ;
    
    //centerArea_ = new QStackedWidget(this) ;
    //centerArea_->addWidget(viewer_) ;
    //centerArea_->addWidget(sampleSelector_) ;
    //setCentralWidget(centerArea_) ;
    
    setCentralWidget(viewer_) ;
    
    readSettings() ;
    
    setObjectName("Axel") ;
    setWindowTitle("Axel modeler") ;
    
# ifdef Q_WS_X11
    setWindowIcon(QIcon(":images/axel_icon.png")) ;
    QApplication::setStyle(QStyleFactory::create("cleanlooks")) ;
    QApplication::setPalette(QApplication::style()->standardPalette()) ;
# endif
    
    QFile file(":stylesheets/main.qss");
    file.open(QFile::ReadOnly);
    QString styleSheet = QLatin1String(file.readAll());
    qApp->setStyleSheet(styleSheet);
}    

/* Handling of close event whatever the way the application is closed 
 * If the user desires so, application settings are saved in $HOME/.conf/INRIA/Axel.conf
 */

void MainWindow::closeEvent(QCloseEvent *event)
{
    if(maybeSave()) {
	writeSettings() ;
	event->accept();
    } else {
	event->ignore();
    }
}

void MainWindow::writeSettings()
{
    QSettings settings("INRIA", "Axel");

    settings.beginGroup("MainWindow");
    settings.setValue("size", size());
    settings.setValue("pos", pos());
    settings.setValue("state", saveState()) ;
    settings.endGroup();

    settings.beginGroup("General");
    settings.endGroup() ;

    settings.beginGroup("Viewer");
    settings.endGroup() ;

    settings.beginGroup("Update");
    settings.endGroup() ;
	
    settings.beginGroup("Plugin");
    settings.endGroup() ;
}

void MainWindow::readSettings()
{
    QSettings settings("INRIA", "Axel");

    settings.beginGroup("MainWindow");
    resize(settings.value("size", QSize(400, 400)).toSize());
    move(settings.value("pos", QPoint(200, 200)).toPoint());
    restoreState(settings.value("state").toByteArray()) ;
    settings.endGroup();
	
    settings.beginGroup("General");
    settings.endGroup() ;

    settings.beginGroup("Viewer");
    settings.endGroup() ;

    settings.beginGroup("Update");
    settings.endGroup() ;
	
    settings.beginGroup("Plugin");
    settings.endGroup() ;
}

bool MainWindow::maybeSave()
{
    if (isWindowModified()) {
	int ret = QMessageBox::warning(this, "Axel",
				       "The document has been modified.\n"
				       "Do you want to save your changes?",
				       QMessageBox::Yes | QMessageBox::Default,
				       QMessageBox::No,
				       QMessageBox::Cancel | QMessageBox::Escape);
	if (ret == QMessageBox::Yes) { fileSave() ; return true ; }
	else if (ret == QMessageBox::Cancel) return false ;
    }
    return true;
}

void MainWindow::documentWasModified()
{
    setWindowModified(true);
}


void MainWindow::setCurrentFile(const QString &fileName)
{
    static int sequenceNumber = 1;

    isUntitled = fileName.isEmpty();
    if (isUntitled) {
        curFile_ = tr("untitled%1.axl").arg(sequenceNumber++);
    } else {
        curFile_ = QFileInfo(fileName).canonicalFilePath();
    }
        
    setWindowModified(false);

    setWindowTitle(tr("%1 - %2[*]").arg(tr("Axel Modeler"))
		   .arg(strippedName(curFile_)));
}

QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}

MainWindow *MainWindow::findMainWindow(const QString &fileName)
{
    QString canonicalFilePath = QFileInfo(fileName).canonicalFilePath();

    foreach (QWidget *widget, qApp->topLevelWidgets()) {
        MainWindow *mainWin = qobject_cast<MainWindow *>(widget);
        if (mainWin && mainWin->curFile_ == canonicalFilePath)
            return mainWin;
    }
    return 0;
}

void MainWindow::loadPlugins(void) 
{
    PluginManager::instance()->loadPlugins(qApp->applicationDirPath(), this);
}
      
void MainWindow::createViewer(void) 
{
    viewer_ = new Viewer(this) ;
    viewer_->camera()->setPosition(qglviewer::Vec(3,1,4)) ;
    viewer_->camera()->lookAt(qglviewer::Vec(0,0,0)) ;
    viewer_->setAxisIsDrawn(true) ;
    viewer_->setXYGridIsDrawn(false) ;
    viewer_->setXZGridIsDrawn(false) ;
    viewer_->setYZGridIsDrawn(false) ;

    connect(viewer_, SIGNAL(dimensionModeChanged(QString)), dimensionModeWidget_, SLOT(setDimensionMode(QString))) ;
    connect(viewer_, SIGNAL(selection(int)), selectedObjectsWidget_, SLOT(setCount(int))) ;
    connect(viewer_, SIGNAL(cameraModeChanged(QString)), cameraModeWidget_, SLOT(setText(QString)));
    
    viewer_->setPerspectiveDisplay() ;   

    //sampleSelector_ = new SampleSelector(this) ;
    //connect(sampleSelector_->interface(), SIGNAL(fileLoaded(QString)), this, SLOT(unloadSampleSelector(QString)));
} 

void MainWindow::createDocks(void) 
{	
    objectToolDock_ = new ObjectToolDock("Objects and Tools", this) ;
    objectToolDock_->setAllowedAreas(Qt::AllDockWidgetAreas) ;
    objectToolDock_->setObjectName("Objects and Tools") ;
    addDockWidget(Qt::LeftDockWidgetArea, objectToolDock_) ;
		
    historyDock_ = new HistoryDock("History Inspector", this) ;
    historyDock_->setAllowedAreas(Qt::AllDockWidgetAreas) ;
    historyDock_->setObjectName("History Dock") ;
    historyDock_->view()->setStack(undoStack_) ;
    addDockWidget(Qt::LeftDockWidgetArea, historyDock_) ;
	
    sceneInspectorDock_ = new SceneInspectorDock(viewer_, "Scene Inspector", this) ;
    sceneInspectorDock_->setAllowedAreas(Qt::AllDockWidgetAreas) ;
    sceneInspectorDock_->setObjectName("Scene Inspector Dock") ;
    addDockWidget(Qt::LeftDockWidgetArea, sceneInspectorDock_) ;
	
    objectPropertiesDock_ = new ObjectPropertiesDock(viewer_, "Object Properties", this) ;
    objectPropertiesDock_->setAllowedAreas(Qt::AllDockWidgetAreas) ;
    objectPropertiesDock_->setObjectName("Object Properties Dock") ;
    addDockWidget(Qt::RightDockWidgetArea, objectPropertiesDock_) ; 
	
    toolPropertiesDock_ = new ToolPropertiesDock(this, "Tool Properties", this) ;
    toolPropertiesDock_->setAllowedAreas(Qt::AllDockWidgetAreas) ;
    toolPropertiesDock_->setObjectName("Tool Properties Dock") ;
    addDockWidget(Qt::RightDockWidgetArea, toolPropertiesDock_) ;

    scriptingDock_ = new ScriptingDock(viewer_, "Script Editor", this) ;
    scriptingDock_->setAllowedAreas(Qt::AllDockWidgetAreas) ;
    scriptingDock_->setObjectName("Scripting Dock") ;
    addDockWidget(Qt::BottomDockWidgetArea, scriptingDock_) ;
	                                                              
    connect(objectToolDock_, SIGNAL(pointClicked()), toolPropertiesDock_, SLOT(createPointWidget())) ;
    connect(objectToolDock_, SIGNAL(planeClicked()), toolPropertiesDock_, SLOT(createPlaneWidget())) ;
    connect(objectToolDock_, SIGNAL(curveClicked()), toolPropertiesDock_, SLOT(createCurveWidget())) ;
    connect(objectToolDock_, SIGNAL(surfaceClicked()), toolPropertiesDock_, SLOT(createSurfaceWidget())) ;
    connect(objectToolDock_, SIGNAL(icIntersectionClicked()), toolPropertiesDock_, SLOT(createIcIntersectionWidget())) ;
    connect(objectToolDock_, SIGNAL(pcIntersectionClicked()), toolPropertiesDock_, SLOT(createPcIntersectionWidget())) ;
    connect(objectToolDock_, SIGNAL(isIntersectionClicked()), toolPropertiesDock_, SLOT(createIsIntersectionWidget())) ;
    connect(objectToolDock_, SIGNAL(psIntersectionClicked()), toolPropertiesDock_, SLOT(createPsIntersectionWidget())) ;
    connect(objectToolDock_, SIGNAL(bscInterpolationClicked()), toolPropertiesDock_, SLOT(createInterpolationBSplineCurveWidget())) ;
    connect(objectToolDock_, SIGNAL(bscApproximationClicked()), toolPropertiesDock_, SLOT(createApproximationBSplineCurveWidget())) ;
    connect(objectToolDock_, SIGNAL(bssApproximationClicked()), toolPropertiesDock_, SLOT(createApproximationBSplineSurfaceWidget())) ;
    connect(objectToolDock_, SIGNAL(bscJoinClicked()), toolPropertiesDock_, SLOT(createJoinBSplineCurveWidget())) ;
    connect(objectToolDock_, SIGNAL(bunionClicked()), toolPropertiesDock_, SLOT(createNoToolWidget())) ;
    connect(objectToolDock_, SIGNAL(bintersectionClicked()), toolPropertiesDock_, SLOT(createNoToolWidget())) ;
    connect(objectToolDock_, SIGNAL(bdifferenceClicked()), toolPropertiesDock_, SLOT(createNoToolWidget())) ;
    connect(objectToolDock_, SIGNAL(curvesArrangementClicked()), toolPropertiesDock_, SLOT(createCurvesArrangementWidget())) ;
    connect(objectToolDock_, SIGNAL(surfacesArrangementClicked()), toolPropertiesDock_, SLOT(createNoToolWidget())) ;
    connect(objectToolDock_, SIGNAL(criticalPointsClicked()), toolPropertiesDock_, SLOT(createNoToolWidget())) ;
    connect(objectToolDock_, SIGNAL(firstDerivativeClicked()), toolPropertiesDock_, SLOT(createFirstDerivativeWidget())) ;
    connect(objectToolDock_, SIGNAL(secondDerivativeClicked()), toolPropertiesDock_, SLOT(createSecondDerivativeWidget())) ;
    connect(objectToolDock_, SIGNAL(jacobiCurveClicked()), toolPropertiesDock_, SLOT(createJacobiCurveWidget())) ;
    connect(objectToolDock_, SIGNAL(hessianCurveClicked()), toolPropertiesDock_, SLOT(createHessianCurveWidget())) ;
    connect(objectToolDock_, SIGNAL(graphedSurfaceClicked()), toolPropertiesDock_, SLOT(createGraphedSurfaceWidget())) ;
	
    connect(objectToolDock_, SIGNAL(distanceConstraintClicked()), toolPropertiesDock_, SLOT(createDistanceConstraintWidget())) ;
    connect(objectToolDock_, SIGNAL(incidenceConstraintClicked()), toolPropertiesDock_, SLOT(createIncidenceConstraintWidget())) ;
    connect(objectToolDock_, SIGNAL(angleConstraintClicked()), toolPropertiesDock_, SLOT(createAngleConstraintWidget())) ;
	
    connect(objectPropertiesDock_, SIGNAL(propertyChanged()), sceneInspectorDock_, SLOT(update())) ;
    connect(objectPropertiesDock_, SIGNAL(updated()), viewer_, SLOT(update())) ;
    connect(objectPropertiesDock_, SIGNAL(addObject(QAxelObject *)), viewer_, SLOT(addObject(QAxelObject *))) ;
	
    connect(viewer_, SIGNAL(objectSelected(QAxelObject *)), objectPropertiesDock_, SLOT(setObject(QAxelObject *))) ;
    connect(viewer_, SIGNAL(manipulation(QAxelObject *)), objectPropertiesDock_, SLOT(setObject(QAxelObject *))) ;
    connect(viewer_, SIGNAL(noObjectSelected()), objectPropertiesDock_, SLOT(clear())) ;
    connect(viewer_, SIGNAL(objectRemoved()), objectPropertiesDock_, SLOT(clear())) ;
    connect(viewer_, SIGNAL(objectAdded(QAxelObject *)), sceneInspectorDock_, SLOT(update())) ;
    connect(viewer_, SIGNAL(objectAdded(QAxelObject *)), scriptingDock_, SLOT(addObject(QAxelObject *))) ;
    connect(viewer_, SIGNAL(objectRemoved()), sceneInspectorDock_, SLOT(update())) ;
    
    connect(viewer_, SIGNAL(sceneChanged()), this , SLOT(documentWasModified())) ;
    connect(viewer_, SIGNAL(sceneChanged()), this , SLOT(documentWasModified())) ;
} 

void MainWindow::createActions(void) 
{
    // Setting file actions                       	
    fileNewAction_ = new QAction("New", this) ;
    fileNewAction_->setShortcut(Qt::CTRL + Qt::Key_N) ;
    fileNewAction_->setIcon(QIcon(":images/32x32/filenew.png")) ;
    fileOpenAction_ = new QAction("Open...", this) ;
    fileOpenAction_->setShortcut(Qt::CTRL + Qt::Key_O) ;
    fileOpenAction_->setIcon(QIcon(":images/32x32/fileopen.png")) ;
    fileSaveAction_ = new QAction("Save", this) ;
    fileSaveAction_->setShortcut(Qt::CTRL + Qt::Key_S) ;
    fileSaveAction_->setIcon(QIcon(":images/32x32/filesave.png")) ;
    fileSaveAsAction_ = new QAction("Save As...", this) ;
    fileSaveAsAction_->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_S) ;
    fileSaveAsAction_->setIcon(QIcon(":images/32x32/filesaveas.png")) ;
    fileCloseAction_ = new QAction("Close", this) ;
    fileCloseAction_->setShortcut(Qt::CTRL + Qt::Key_W) ;
    fileCloseAction_->setIcon(QIcon(":images/32x32/fileclose.png")) ;
    importVectAction_ = new QAction("VECT (Polylines File Format)", this) ;
    importOffAction_ = new QAction("OFF (Object File Format)", this) ;
    importG2CurveAction_ = new QAction("G2 Curve (Sintef File Format)", this) ;
    importG2SurfaceAction_ = new QAction("G2 Surface (Sintef File Format)", this) ;
    importGcmlAction_ = new QAction("GCML (Gcml File Format)", this) ;
    exportVectAction_ = new QAction("VECT (Polylines File Format)", this) ;
    exportOffAction_ = new QAction("OFF (Object File Format)", this) ;
    exportGcmlAction_ = new QAction("GCML (Gcml File Format)", this) ;
    saveSnaphotAction_ = new QAction("Save snapshot", this) ;
    saveSnaphotAction_->setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_S) ;
    showPreferencesDialogAction_ = new QAction("Preferences", this) ; 
    showPreferencesDialogAction_->setIcon(QIcon(":images/48x48/preferences.png"));
    quitAction_ = new QAction("Quit", this) ;
    quitAction_->setShortcut(Qt::CTRL + Qt::Key_Q) ;
	
    connect(fileNewAction_, SIGNAL(triggered()), this, SLOT(fileNew())) ;
    connect(fileOpenAction_, SIGNAL(triggered()), this, SLOT(fileOpen())) ;
    connect(fileSaveAction_, SIGNAL(triggered()), this, SLOT(fileSave())) ;
    connect(fileSaveAsAction_, SIGNAL(triggered()), this, SLOT(fileSaveAs())) ;
    connect(fileCloseAction_, SIGNAL(triggered()), this, SLOT(fileClose())) ;
    connect(importVectAction_, SIGNAL(triggered()), this, SLOT(importVectSlot())) ;
    connect(importOffAction_, SIGNAL(triggered()), this, SLOT(importOffSlot())) ;
    connect(importG2CurveAction_, SIGNAL(triggered()), this, SLOT(importG2CurveSlot())) ;
    connect(importG2SurfaceAction_, SIGNAL(triggered()), this, SLOT(importG2SurfaceSlot())) ;
    connect(importGcmlAction_, SIGNAL(triggered()), this, SLOT(importGcmlSlot()));
    connect(exportVectAction_, SIGNAL(triggered()), this, SLOT(exportVectSlot())) ;
    connect(exportOffAction_, SIGNAL(triggered()), this, SLOT(exportOffSlot())) ;
    connect(exportGcmlAction_, SIGNAL(triggered()), this, SLOT(exportGcmlSlot())) ;
    connect(saveSnaphotAction_, SIGNAL(triggered()), this, SLOT(saveSnapshot())) ;
    connect(showPreferencesDialogAction_, SIGNAL(triggered()), this, SLOT(showPreferencesDialog())) ;
    connect(quitAction_, SIGNAL(triggered()), this, SLOT(close())) ;

    // Disable the actions that are connected to non-implemented slots, and
    // show that in their text.
    QList<QAction*> disabled_actions;
    disabled_actions << exportVectAction_
		     << exportOffAction_;

    foreach(QAction* action, disabled_actions) {
	action->setText(action->text()+ " (not implemented yet)");
	action->setEnabled(false);
    }
	
    // Setting Edit actions
    undoAction_ = undoStack_->createUndoAction(this) ;
    undoAction_->setIcon(QIcon(":images/22x22/edit-undo.png")) ;
    undoAction_->setShortcut(Qt::CTRL + Qt::Key_Z) ;
    redoAction_ = undoStack_->createRedoAction(this) ;
    redoAction_->setIcon(QIcon(":images/22x22/edit-redo.png")) ;
    redoAction_->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_Z) ;
	
    // Setting selection actions
    selectAllAction_ = new QAction("Select all", this) ;
    unselectAllAction_ = new QAction("Unselect all", this) ;
    deleteSelectionAction_ = new QAction("Delete selection", this) ;
    toggleVertexSelectionAction_ = new QAction("Select vertices", this) ; 
    toggleVertexSelectionAction_->setCheckable(true) ;                    
    toggleVertexSelectionAction_->setIcon(QIcon(":images/objects/select_vertex.png")) ;
    toggleEdgeSelectionAction_ = new QAction("Select edges", this) ; 
    toggleEdgeSelectionAction_->setCheckable(true) ;                      
    toggleEdgeSelectionAction_->setIcon(QIcon(":images/objects/select_edge.png")) ;
    toggleFaceSelectionAction_ = new QAction("Select faces", this) ; 
    toggleFaceSelectionAction_->setCheckable(true) ;                      
    toggleFaceSelectionAction_->setIcon(QIcon(":images/objects/select_face.png")) ;
    selectionActionGroup_ = new QActionGroup(this) ;
    selectionActionGroup_->addAction(toggleVertexSelectionAction_) ;
    selectionActionGroup_->addAction(toggleEdgeSelectionAction_) ;
    selectionActionGroup_->addAction(toggleFaceSelectionAction_) ;	
    toggleVertexSelectionAction_->setChecked(true) ;
	
    connect(selectAllAction_, SIGNAL(triggered()), viewer_, SLOT(selectAll())) ;
    connect(unselectAllAction_, SIGNAL(triggered()), viewer_, SLOT(unselectAll())) ;
    connect(deleteSelectionAction_, SIGNAL(triggered()), viewer_, SLOT(removeSelectedObjects())) ;
    connect(toggleVertexSelectionAction_, SIGNAL(triggered(bool)), this, SLOT(setVertexEditionMode(bool))) ;
    connect(toggleEdgeSelectionAction_, SIGNAL(triggered(bool)), this, SLOT(setEdgeEditionMode(bool))) ;
    connect(toggleFaceSelectionAction_, SIGNAL(triggered(bool)), this, SLOT(setFaceEditionMode(bool))) ;
	
    // Setting Display actions
    toggleAxisAction_ = new QAction("Toggle Axis", this) ; toggleAxisAction_->setCheckable(true) ;
    toggleAxisAction_->setShortcut(Qt::ALT + Qt::Key_A) ;
    toggleAxisAction_->setChecked(true) ;

    QObject::connect(dimensionModeWidget_ , SIGNAL(dimensionModeSet(bool)) , toggleAxisAction_ , SLOT (setChecked(bool))); 

    toggleXYGridAction_ = new QAction("Toggle XY Grid", this) ; toggleXYGridAction_->setCheckable(true) ;
    toggleXYGridAction_->setChecked(false) ; 
    toggleXYGridAction_->setShortcut(Qt::ALT + Qt::Key_Z) ;
    toggleXZGridAction_ = new QAction("Toggle XZ Grid", this) ; toggleXZGridAction_->setCheckable(true) ;
    toggleXZGridAction_->setChecked(false) ; 
    toggleXZGridAction_->setShortcut(Qt::ALT + Qt::Key_Y) ;
    toggleYZGridAction_ = new QAction("Toggle YZ Grid", this) ; toggleYZGridAction_->setCheckable(true) ;
    toggleYZGridAction_->setChecked(false) ;
    toggleYZGridAction_->setShortcut(Qt::ALT + Qt::Key_X) ;
	
    toggleEditionMode_ = new QAction("Toggle mesh edition mode", this) ;
    // toggleWireDisplayAction_ = new QAction("Wireframe", this) ;
    // toggleWireDisplayAction_->setCheckable(true) ;                    
    // toggleWireDisplayAction_->setIcon(QIcon(":images/objects/display_wire.png")) ;
    toggleLitWireDisplayAction_ = new QAction("Wireframe", this) ;
    toggleLitWireDisplayAction_->setCheckable(true) ;
    toggleLitWireDisplayAction_->setIcon(QIcon(":images/objects/display_wire_lit.png")) ;
    toggleLitWireDisplayAction_->setShortcut(Qt::ALT + Qt::Key_W) ;
    toggleFlatDisplayAction_ = new QAction("Flat", this) ;
    toggleFlatDisplayAction_->setCheckable(true) ;
    toggleFlatDisplayAction_->setIcon(QIcon(":images/objects/display_flat.png")) ;
    toggleFlatDisplayAction_->setShortcut(Qt::ALT + Qt::Key_F) ;
    toggleSmoothDisplayAction_ = new QAction("Smooth", this) ;
    toggleSmoothDisplayAction_->setCheckable(true) ;
    toggleSmoothDisplayAction_->setIcon(QIcon(":images/objects/display_smooth.png")) ;
    toggleSmoothDisplayAction_->setShortcut(Qt::ALT + Qt::Key_S) ;
    displayActionGroup_ = new QActionGroup(this) ;
    // displayActionGroup_->addAction(toggleWireDisplayAction_) ;
    displayActionGroup_->addAction(toggleLitWireDisplayAction_) ;
    displayActionGroup_->addAction(toggleFlatDisplayAction_) ;
    displayActionGroup_->addAction(toggleSmoothDisplayAction_) ;
    toggleSmoothDisplayAction_->setChecked(true) ;
    perspectiveCameraModeAction_ = new QAction("Perspective projection mode", this) ;
    perspectiveCameraModeAction_->setCheckable(true) ;
    perspectiveCameraModeAction_->setShortcut(Qt::ALT + Qt::Key_P) ;
    perspectiveCameraModeAction_->setIcon(QIcon(":images/objects/perspective.png")) ;
    orthographicCameraModeAction_ = new QAction("Orthographic projection mode", this) ;
    orthographicCameraModeAction_->setCheckable(true) ;
    orthographicCameraModeAction_->setShortcut(Qt::ALT + Qt::Key_O) ;
    orthographicCameraModeAction_->setIcon(QIcon(":images/objects/orthographic.png")) ;
    twoDimensionModeAction_ = new QAction("2D modeling mode", this) ;
    twoDimensionModeAction_->setCheckable(true) ;
    twoDimensionModeAction_->setShortcut(Qt::ALT + Qt::Key_2) ;
    twoDimensionModeAction_->setIcon(QIcon(":images/objects/orthographic.png")) ;
    
    threeDimensionModeAction_ = new QAction("3D modeling mode", this) ;
    threeDimensionModeAction_->setCheckable(true) ;
    threeDimensionModeAction_->setShortcut(Qt::ALT + Qt::Key_3) ;
    threeDimensionModeAction_->setIcon(QIcon(":images/objects/perspective.png")) ;
    cameraModeGroupAction_ = new QActionGroup(this) ;
    cameraModeGroupAction_->addAction(perspectiveCameraModeAction_) ;
    cameraModeGroupAction_->addAction(orthographicCameraModeAction_) ;
    dimensionModeGroupAction_ = new QActionGroup(this) ;
    dimensionModeGroupAction_->addAction(twoDimensionModeAction_);
    dimensionModeGroupAction_->addAction(threeDimensionModeAction_);

    perspectiveCameraModeAction_->setChecked(true) ;
    threeDimensionModeAction_->setChecked(true) ;
    
    connect(toggleAxisAction_, SIGNAL(triggered(bool)), viewer_, SLOT(setAxisIsDrawn(bool))) ;
    connect(toggleAxisAction_, SIGNAL(triggered(bool)), this, SLOT(displayAxisMessage(bool))) ;
    connect(toggleXYGridAction_, SIGNAL(triggered(bool)), viewer_, SLOT(setXYGridIsDrawn(bool))) ;
    connect(toggleXZGridAction_, SIGNAL(triggered(bool)), viewer_, SLOT(setXZGridIsDrawn(bool))) ;
    connect(toggleYZGridAction_, SIGNAL(triggered(bool)), viewer_, SLOT(setYZGridIsDrawn(bool))) ;
	
    connect(toggleEditionMode_, SIGNAL(triggered()), viewer_, SLOT(toggleMeshEditionMode())) ;
    // connect(toggleWireDisplayAction_, SIGNAL(triggered(bool)), this, SLOT(setWireDisplaySlot(bool))) ;
    connect(toggleLitWireDisplayAction_, SIGNAL(triggered(bool)), this, SLOT(setLitWireDisplaySlot(bool))) ;
    connect(toggleFlatDisplayAction_, SIGNAL(triggered(bool)), this, SLOT(setFlatDisplaySlot(bool))) ;
    connect(toggleSmoothDisplayAction_, SIGNAL(triggered(bool)), this, SLOT(setSmoothDisplaySlot(bool))) ;
	
    connect(perspectiveCameraModeAction_, SIGNAL(triggered()), viewer_ , SLOT(setPerspectiveDisplay())) ;
    connect(perspectiveCameraModeAction_, SIGNAL(triggered()), this, SLOT(displayPerspectiveModeMessage())) ;
    connect(orthographicCameraModeAction_, SIGNAL(triggered()), viewer_ , SLOT(setOrthographicDisplay())) ;
    connect(orthographicCameraModeAction_, SIGNAL(triggered()), this, SLOT(displayOrthographicModeMessage())) ;
    connect(twoDimensionModeAction_, SIGNAL(triggered()), viewer_ , SLOT(set2dMode())) ;
    connect(twoDimensionModeAction_, SIGNAL(triggered()), this, SLOT(display2dModeMessage())) ;

    connect(threeDimensionModeAction_, SIGNAL(triggered()), viewer_ , SLOT(set3dMode())) ;
    connect(threeDimensionModeAction_, SIGNAL(triggered()), this, SLOT(display3dModeMessage())) ;
	
    // Setting Render actions
    yafrayRenderAction_ = new QAction("Yafray", this) ;
    surfRenderAction_ = new QAction("Surf", this) ;
	
    connect(yafrayRenderAction_, SIGNAL(triggered(bool)), toolPropertiesDock_, SLOT(createYafrayRenderWidget())) ;
    connect(surfRenderAction_, SIGNAL(triggered(bool)), toolPropertiesDock_, SLOT(createSurfRenderWidget())) ;
	
    // Setting window actions
    toggleObjectToolDockAction_ = objectToolDock_->toggleViewAction() ;
    toggleObjectPropertiesDockAction_ = objectPropertiesDock_->toggleViewAction() ;
    toggleToolPropertiesDockAction_ = toolPropertiesDock_->toggleViewAction() ;
    toggleSceneInspectorDockAction_ = sceneInspectorDock_->toggleViewAction() ;
    toggleScriptingDockAction_ = scriptingDock_->toggleViewAction() ;
    toggleHistoryDockAction_ = historyDock_->toggleViewAction() ;
    toggleFullScreenAction_ = new QAction("Full screen", this) ;
    toggleFullScreenAction_->setShortcut(Qt::ALT + Qt::Key_Return) ;
	
    connect(toggleFullScreenAction_, SIGNAL(triggered()), viewer_, SLOT(toggleFullScreen())) ;
    connect(toggleFullScreenAction_, SIGNAL(triggered(bool)), this, SLOT(displayFullScreenMessage(bool))) ;
	
    loadScriptAction_ = new QAction("Load", this) ;
    loadScriptAction_->setIcon(QIcon(":images/32x32/fileopen.png")) ;
    loadScriptAction_->setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_O) ;
    saveScriptAction_ = new QAction("Save", this) ;
    saveScriptAction_->setIcon(QIcon(":images/32x32/filesave.png")) ;
    saveScriptAction_->setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_S) ;
    saveAsScriptAction_ = new QAction("Save", this) ;
    saveAsScriptAction_->setIcon(QIcon(":images/32x32/filesaveas.png")) ;
    saveAsScriptAction_->setShortcut(Qt::CTRL + Qt::ALT + Qt::SHIFT + Qt::Key_S) ;
    runScriptAction_ = new QAction("Run", this) ;
    runScriptAction_->setShortcut(Qt::CTRL + Qt::ALT + Qt::Key_R) ;
	
    connect(loadScriptAction_, SIGNAL(triggered()), scriptingDock_, SLOT(load())) ;
    connect(saveScriptAction_, SIGNAL(triggered()), scriptingDock_, SLOT(save())) ;
    connect(saveAsScriptAction_, SIGNAL(triggered()), scriptingDock_, SLOT(saveAs())) ;
    connect(runScriptAction_, SIGNAL(triggered()), scriptingDock_, SLOT(run())) ;
	
    // Setting Help actions
    //searchSampleAction_ = new QAction("Samples repository", this) ;
    showHelpDialogAction_ = new QAction("About Controls", this) ;  
    aboutPluginsAction_ = new QAction("About Plugins", this) ;
    showAboutDialogAction_ = new QAction("About Axel", this) ;
    checkForUpdatesAction_ = new QAction("Check for Updates", this) ;
	
    //connect(searchSampleAction_, SIGNAL(triggered()), this, SLOT(loadSampleSelector())) ;
    connect(showHelpDialogAction_, SIGNAL(triggered()), this, SLOT(help())) ;
    connect(aboutPluginsAction_, SIGNAL(triggered()), this, SLOT(aboutPlugins())) ;
    connect(showAboutDialogAction_, SIGNAL(triggered()), this, SLOT(showAboutDialog())) ;
    connect(checkForUpdatesAction_, SIGNAL(triggered()), this, SLOT(checkForUpdates())) ;
}
                                                                                                  
void MainWindow::createMenu(void) 
{
    fileMenu_ = menuBar()->addMenu("File") ;
    fileMenu_->addAction(fileNewAction_) ;
    fileMenu_->addAction(fileOpenAction_) ;
    fileMenu_->addAction(fileSaveAction_) ;
    fileMenu_->addAction(fileSaveAsAction_) ;
    fileMenu_->addAction(fileCloseAction_) ;
    fileMenu_->addSeparator() ;
    importMenu_ = fileMenu_->addMenu("Import") ;
    importMenu_->setObjectName("importMenu") ;
    exportMenu_ = fileMenu_->addMenu("Export") ;
    exportMenu_->setObjectName("exportMenu") ;
    importMenu_->addAction(importVectAction_) ;
    importMenu_->addAction(importOffAction_) ;
    importMenu_->addAction(importG2CurveAction_) ;
    importMenu_->addAction(importG2SurfaceAction_) ;
    importMenu_->addAction(importGcmlAction_) ;
    exportMenu_->addAction(exportVectAction_) ;
    exportMenu_->addAction(exportOffAction_) ;
    exportMenu_->addAction(exportGcmlAction_) ;
    fileMenu_->addAction(showPreferencesDialogAction_) ; 
    fileMenu_->addAction(saveSnaphotAction_) ;
    fileMenu_->addSeparator() ;
    fileMenu_->addAction(quitAction_) ;
	
    editMenu_ = menuBar()->addMenu("Edit") ;
    editMenu_->addAction(undoAction_) ;
    editMenu_->addAction(redoAction_) ;
	
    selectionMenu_ = menuBar()->addMenu("Selection") ;
    selectionMenu_->addAction(selectAllAction_) ;
    selectionMenu_->addAction(unselectAllAction_) ;
    selectionMenu_->addAction(deleteSelectionAction_) ;
    selectionMenu_->addSeparator() ;
    selectionMenu_->addAction(toggleEditionMode_) ;
    selectionMenu_->addAction(toggleVertexSelectionAction_) ;
    selectionMenu_->addAction(toggleEdgeSelectionAction_) ;
    selectionMenu_->addAction(toggleFaceSelectionAction_) ;
	
    displayMenu_ = menuBar()->addMenu("Display") ;
    displayMenu_->addAction(toggleAxisAction_) ;
    displayMenu_->addAction(toggleXYGridAction_) ;
    displayMenu_->addAction(toggleXZGridAction_) ;
    displayMenu_->addAction(toggleYZGridAction_) ;
    displayMenu_->addSeparator() ;
    displayMenu_->addAction(toggleLitWireDisplayAction_) ;
    displayMenu_->addAction(toggleFlatDisplayAction_) ;
    displayMenu_->addAction(toggleSmoothDisplayAction_) ;
    displayMenu_->addSeparator() ;
    displayMenu_->addAction(perspectiveCameraModeAction_) ;
    displayMenu_->addAction(orthographicCameraModeAction_) ;
    displayMenu_->addSeparator() ;
    displayMenu_->addAction(twoDimensionModeAction_);
    displayMenu_->addAction(threeDimensionModeAction_);
	
    renderMenu_ = menuBar()->addMenu("Render") ;
    renderMenu_->addAction(yafrayRenderAction_) ;
    renderMenu_->addAction(surfRenderAction_) ;
	
    windowMenu_ = menuBar()->addMenu("Window") ;
    windowMenu_->addAction(toggleObjectToolDockAction_) ;
    windowMenu_->addSeparator() ;
    windowMenu_->addAction(toggleHistoryDockAction_) ;
    windowMenu_->addAction(toggleSceneInspectorDockAction_) ;
    windowMenu_->addAction(toggleObjectPropertiesDockAction_) ;
    windowMenu_->addAction(toggleToolPropertiesDockAction_) ;
    windowMenu_->addAction(toggleScriptingDockAction_) ;
    windowMenu_->addSeparator() ;
    windowMenu_->addAction(toggleFullScreenAction_) ;
	
    scriptMenu_ = menuBar()->addMenu("Script") ;
    scriptMenu_->addAction(loadScriptAction_) ;
    scriptMenu_->addAction(saveScriptAction_) ;
    scriptMenu_->addAction(saveAsScriptAction_) ;
    scriptMenu_->addAction(runScriptAction_) ;
			
    helpMenu_ = menuBar()->addMenu("Help") ;
    helpMenu_->addAction(showHelpDialogAction_) ;
    helpMenu_->addAction(showAboutDialogAction_) ;
    helpMenu_->addAction(aboutPluginsAction_) ;
    helpMenu_->addSeparator() ;
    helpMenu_->addAction(checkForUpdatesAction_) ;
}        

void MainWindow::createToolBars(void)
{
//     fileToolBar = addToolBar(tr("File")) ;
//     fileToolBar->addAction(fileNewAction_) ;
//     fileToolBar->addAction(fileOpenAction_) ;
//     fileToolBar->addAction(fileSaveAction_) ;
//     fileToolBar->addAction(fileCloseAction_) ;

//     editToolBar = addToolBar(tr("Edit")) ;
//     editToolBar->addAction(undoAction_) ;
//     editToolBar->addAction(redoAction_) ;

//     displayToolBar = addToolBar("Display") ;
//     displayToolBar->addAction(toggleLitWireDisplayAction_) ;
//     displayToolBar->addAction(toggleFlatDisplayAction_) ;
//     displayToolBar->addAction(toggleSmoothDisplayAction_) ;
//     displayToolBar->addSeparator() ;
//     displayToolBar->addAction(perspectiveCameraModeAction_) ;
//     displayToolBar->addAction(orthographicCameraModeAction_) ;
//     displayToolBar->addSeparator() ;
//     displayToolBar->addAction(twoDimensionModeAction_) ;
//     displayToolBar->addAction(twoDimensionModeAction_) ;

//        fileToolBar->setObjectName("fileToolBar") ;
//        editToolBar->setObjectName("editToolBar") ;
//     displayToolBar->setObjectName("displayToolBar") ;
}

void MainWindow::setStatusBar(void) 
{
    algorithmWidget_ = new QAlgorithmWidget(this);
    statusBar()->addPermanentWidget(algorithmWidget_) ;

    selectedObjectsWidget_ = new QSelectedObjectsWidget(this) ;	
    statusBar()->addPermanentWidget(selectedObjectsWidget_) ;
	
    dimensionModeWidget_ = new QDimensionModeWidget(this);
    statusBar()->addPermanentWidget(dimensionModeWidget_);
    
    cameraModeWidget_ = new QCameraModeWidget(this) ;
    statusBar()->addPermanentWidget(cameraModeWidget_);    
}

Viewer * MainWindow::viewer(void) 
{
    return viewer_ ;
}

QUndoStack * MainWindow::stack(void)
{
    return undoStack_ ;
}

void MainWindow::showPreferencesDialog(void) 
{
    PreferencesDialog * dialog = new PreferencesDialog(this) ;
    dialog->show() ;
}

void MainWindow::showAboutDialog(void) 
{    
    QDialog * about = new QDialog;
    Ui::AboutDialog ui;
    ui.setupUi(about);
    about->show();
}

void MainWindow::fileNew(void) 
{
    MainWindow *other = new MainWindow;
    other->setCurrentFile("");
    other->move(x() + 40, y() + 40);
    other->show();
}

void MainWindow::fileOpen(QString s) 
{
    QString filename ;
    if(s.isNull()) filename = QFileDialog::getOpenFileName(
	this,
	"Choose an Axel file",
	".",
	"AXL files (*.axl)");
    else filename = s ;

    if (!filename.isEmpty()) {
        MainWindow *existing = findMainWindow(filename);
        if (existing) {
            existing->show();
            existing->raise();
            existing->activateWindow();
            return;
        }

        if (isUntitled && viewer_->listObjects().isEmpty()
	    && !isWindowModified()) {
	    setCurrentFile(filename);
	    IStreamAxl parser(filename, this) ;
	    parser.run() ;
	    setWindowModified(false);
        } else {
            MainWindow *other = new MainWindow;
	    other->setCurrentFile(filename);
	    IStreamAxl parser(filename, other) ;
	    parser.run() ;
	    other->setWindowModified(false);
            if (other->isUntitled) {
                delete other;
                return;
            }
            other->move(x() + 40, y() + 40);
            other->show();
        }
    }
}

void MainWindow::fileSaveAs(void) 
{
    QString filename = QFileDialog::getSaveFileName(
	this,
	"Choose an Axel file",
	".",
	"AXL files (*.axl)") ;
    
    if(!filename.isNull()) {
	OStreamAxl parser(filename, this) ;
	parser.run() ;
    }
}

void MainWindow::fileSave(void) 
{
    if(isUntitled){
	QString filename = QFileDialog::getSaveFileName(
	    this,
	    "Choose an Axel file",
	    ".",
	    "AXL files (*.axl)") ;

	if(!filename.isNull()) {
	    OStreamAxl parser(filename, this) ;
	    parser.run() ;
	    setCurrentFile(filename);
	    setWindowModified(false);
	}
    }
    else if(!curFile_.isNull()) {
        OStreamAxl parser(curFile_, this) ;
        parser.run() ;
	setWindowModified(false);
    }

}

void MainWindow::fileClose(void) 
{
    close();
}

void MainWindow::importVectSlot(QString s) 
{
    QString filename ;
    if(s.isNull()) filename = QFileDialog::getOpenFileName(
	this,
	"Choose a file to open",
	".",
	"Polylines File Format (*.vect)");
    else filename = s ;

    if(!filename.isNull()) {
	QFile * file = new QFile(filename) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamVect parser(file) ;
	QAxelObject * m = parser.run() ;
	if(m)
	  viewer_->addObject(m) ;
    }
}

void MainWindow::importOffSlot(QString s) 
{
    QString filename ;
    if(s.isNull()) filename = QFileDialog::getOpenFileName(
	this,
	"Choose a file to open",
	".",
	"Object File Format (*.off)");
    else filename = s ;

    if(!filename.isNull()) {
	QFile * file = new QFile(filename) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamOff parser(file) ;
	QAxelObject * m = parser.run() ;
	if(m) 
	  viewer_->addObject(m) ;
    }
}

void MainWindow::importG2CurveSlot(QString s)
{
    QString filename ;
    if(s.isNull()) filename = QFileDialog::getOpenFileName(
	this,
	"Choose a file to open",
	".",
	"Sintef G2 files (*.g2)");
    else filename = s ;

    if(!filename.isNull()) {
	QFile * file = new QFile(filename) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamG2 parser(file) ;
	QAxelObject* c = parser.readCurve() ;
	if(c) 
	  viewer_->addObject(c) ;

    }
}

void MainWindow::importG2SurfaceSlot(QString s)
{
    QString filename ;
    if(s.isNull()) filename = QFileDialog::getOpenFileName(
	this,
	"Choose a file to open",
	".",
	"Sintef G2 files (*.g2)");
    else filename = s ;

    if(!filename.isNull()) {
	QFile * file = new QFile(filename) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamG2 parser(file) ;
	QAxelObject* s = parser.readSurface() ;
	if(s) 
	  viewer_->addObject(s) ;
    }
}

void MainWindow::importGcmlSlot(QString s) 
{
    QString filename ;
    if(s.isNull()) filename = QFileDialog::getOpenFileName(
	this,
	"Choose an Gcml file",
	"./samples/gcml",
	"Gcml files (*.gcml)");
    else filename = s ;

    if(!filename.isNull()) {
	QFile * file = new QFile(filename) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamGcml parser(file, filename, viewer_);
	parser.run() ;
    }
}

void MainWindow::exportVectSlot(void) 
{
    QMessageBox::information(this, "Axel","Not implemented yet","&Ok",0) ;
}

void MainWindow::exportOffSlot(void) 
{
    QMessageBox::information(this, "Axel","Not implemented yet","&Ok",0) ;
}

void MainWindow::exportGcmlSlot(void) 
{
    QString filename = QFileDialog::getSaveFileName(this, "Choose a gcml filename to save under", "./samples/gcml/AxelOut.gcml","Gcml files (*.gcml)");

    OStreamGcml parser(filename, viewer_);
    parser.run();
}

void MainWindow::setWireDisplaySlot(bool checked) { if(checked) viewer_->setWireDisplay() ; }
void MainWindow::setLitWireDisplaySlot(bool checked) { if(checked) viewer_->setLitWireDisplay() ; }
void MainWindow::setFlatDisplaySlot(bool checked) { if(checked) viewer_->setFlatDisplay() ; }
void MainWindow::setSmoothDisplaySlot(bool checked) { if(checked) viewer_->setSmoothDisplay() ; }
  
void MainWindow::setVertexEditionMode(bool checked) { if(checked) viewer_->setMeshEditionMode(Viewer::VertexEdition) ; }
void MainWindow::setEdgeEditionMode(bool checked) { if(checked) viewer_->setMeshEditionMode(Viewer::EdgeEdition) ; }
void MainWindow::setFaceEditionMode(bool checked) { if(checked) viewer_->setMeshEditionMode(Viewer::FaceEdition) ; }

void MainWindow::saveSnapshot(void) 
{
    viewer_->saveSnapshot(false, false) ;
}

void MainWindow::aboutPlugins()
{
    PluginManager::instance()->aboutPlugins(this) ;
}

//************** AF SandBox
void MainWindow::addObject(const Vec &v)
{
    if(objectToolDock_)
    {
	if (objectToolDock_->getSelectedObjectType() == 1)
	{
	    QPoint3f * p = new QPoint3f(v.x,v.y,v.z) ;
	    viewer_->addObject(p) ;
	}
	if (objectToolDock_->getSelectedObjectType() == 2)
	{
	    std::cout << "plane added" << endl;
	}
    }
}
//*************************

void MainWindow::help(void)
{
    QDialog * dialog = new QDialog(this) ;
    dialog->setWindowTitle("Axel Controls") ;
    QVBoxLayout * layout = new QVBoxLayout ;
    static QString label[] = {" &Keyboard ", " &Mouse "} ;

    helpWidget_ = new QTabWidget(dialog);
    helpWidget_->setWindowTitle("Help");

    for (int i=0; i<2; ++i) {
	QTextEdit * tab = new QTextEdit(helpWidget_);
	tab->setReadOnly(true);
	helpWidget_->insertTab(i, tab, label[i]);
    }
	
    for (int i=0; i<2; ++i) {
	QString text;
	switch (i) {
	case 0 : text = viewer_->keyboardString(); break;
	case 1 : text = viewer_->mouseString();	  break;
	default : break;
	}

	QTextEdit* textEdit = (QTextEdit*)(helpWidget_->widget(i));
	textEdit->setHtml(text);
    }
	
    QDialogButtonBox * buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel) ;

    layout->addWidget(helpWidget_) ;
    layout->addWidget(buttonBox) ;

    connect(buttonBox, SIGNAL(accepted()), dialog, SLOT(accept())) ;
    connect(buttonBox, SIGNAL(rejected()), dialog, SLOT(reject())) ;

    dialog->setLayout(layout) ;
    dialog->show() ;
}

void MainWindow::checkForUpdates(void)
{
# ifdef Q_WS_MAC
    CocoaUtil::checkForUpdates() ;
# else
    QMessageBox::information(this, "Axel","Currently only available under MacOSX","&Ok",0) ;
# endif
}

QFrame * MainWindow::deamonsFrame(void)
{
    return deamonsFrame_ ;
}

QCameraModeWidget * MainWindow::cameraModeWidget(void)
{
    return cameraModeWidget_ ;
}

QAlgorithmWidget * MainWindow::algorithmWidget(void)
{
    return algorithmWidget_ ;
}

void MainWindow::loadSampleSelector(void)
{
//    centerArea_->setCurrentWidget(sampleSelector_) ;
}

void MainWindow::unloadSampleSelector(QString s)
{
    Q_UNUSED(s) ;
//    centerArea_->setCurrentWidget(viewer_) ;
//    fileOpen(s) ;
}

void MainWindow::loadImplicitCurvesArrangementDock(void)
{
    ImplicitCurvesArrangementDock * dock = new ImplicitCurvesArrangementDock("Implicit Curves Arrangement", this) ;
    dock->setAllowedAreas(Qt::BottomDockWidgetArea) ;
    scriptingDock_->addObject((QObject *)dock) ;
    addDockWidget(Qt::BottomDockWidgetArea, dock) ;
    tabifyDockWidget(scriptingDock_, dock) ;
}
