/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ObjectPropertiesDock
 * 2006-03-31
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef OBJECTPROPERTIESDOCK_H
# define OBJECTPROPERTIESDOCK_H
                          
# include <QDockWidget>      
# include <QPointer>

# include <QGui/QPropertyEditor.h>

# include <QAxel/QAxelObject.h>
# include <QAxel/QPoint3f.h>

using namespace QAxel ; 

class Viewer ;            
class QSplitter ;
class QParameterSpaceView ;

class ObjectPropertiesDock : public QDockWidget 
{
    Q_OBJECT       

public:
    ObjectPropertiesDock(Viewer * viewer, const QString & title, QWidget * parent) ;
    
    virtual Viewer * viewer(void) const ;
	
    virtual bool isReadOnly() const;
    virtual void setReadOnly(bool readOnly);
    virtual void setPropertyValue(const QString &name, const QVariant &value, bool changed = true);
    
	
    virtual QAxelObject * object() const { 
	return m_object ;
    }
    
    virtual QString currentPropertyName() const;

signals:
    void propertyChanged(void) ;
    void addObject(QAxelObject *) ;
    void updated(void) ;

private slots:
    void firePropertyChanged(IProperty *property);
    void resetProperty(const QString &prop_name);
    virtual void setObject(QAxelObject *object);
    void clear(void) ;

private:
    IProperty *propertyByName(IProperty *p, const QString &name);
    void clearDirty(IProperty *p);
    void createPropertySheet(PropertyCollection *root, QAxelObject *object);
    static IProperty *createSpecialProperty(const QVariant &value, const QString &name);
    
private:
    QPropertyEditor * m_editor ;
    IPropertyGroup * m_properties ;
    QPointer<QAxelObject> m_object ;
// 	QDesignerFormEditorInterface *m_core; --> viewer
// 	QDesignerPropertySheetExtension *m_prop_sheet;
    
private:
    Viewer              * viewer_ ;
    QSplitter           * splitter ;
    QParameterSpaceView * parameterSpaceView ;
    bool                  dirty ;
} ;

# endif
