/****************************************************************************
 * A x e l
 *****************************************************************************
 * ObjectPropertiesSheet
 * 2006-11-08
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <iostream>

# include <QGui/Viewer.h>
# include <QGui/ObjectPropertiesDock.h>
# include <QGui/QPropertyEditor.h>
# include <QGui/PropertyEditorModel.h>

# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QConstraintDistance.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

using namespace QAxel ;

/*****************************************************************************
 * A structure for properties groups
 *****************************************************************************/
struct IGroup
{
	QString name;
	QList<IProperty*> properties;

	inline IGroup() {}
	inline IGroup(const QString &n): name(n) {}

	inline bool operator == (const IGroup &other) const
	{ return name == other.name; }
} ;

/*****************************************************************************
 * The creation of the properties sheet
 *****************************************************************************/
void ObjectPropertiesDock::createPropertySheet(PropertyCollection *root, QAxelObject *object)
{
   	QList<IGroup> groups ;

	/*****************************************************************************
	 * QAxelObject Properties
	 *****************************************************************************/	
	IGroup geometricObjectGroup("QAxelObject") ;
	
	{ // Name Property //////////////////////////////////////////////////////
		QString name = "Name" ;
		QVariant value = object->name() ;
		IProperty * p = 0;
		p = new StringProperty(qvariant_cast<QString>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = geometricObjectGroup.properties;
		groupProperties.append(p);
	} //////////////////////////////////////////////////////////////////////
	
	{ // Type Property /////////////////////////////////////////////////////
		QString name = "Type" ;
		QVariant value = object->type() ;
		IProperty * p = 0;
		p = new StringProperty(qvariant_cast<QString>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = geometricObjectGroup.properties;
		groupProperties.append(p);
	} //////////////////////////////////////////////////////////////////////
		
	{ // Color Property ////////////////////////////////////////////////////
		QString name = "Color" ;
		QVariant value = *(object->color()) ;
		IProperty * p = 0;
		p = new ColorProperty(qvariant_cast<QColor>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = geometricObjectGroup.properties;
		groupProperties.append(p);
	} //////////////////////////////////////////////////////////////////////
	
	groups.append(geometricObjectGroup) ;
	
	/*****************************************************************************
	 * QPoint3f Properties
	 *****************************************************************************/
	
	if(QPoint3f * point = qobject_cast<QPoint3f *>(object)) {
		
		IGroup pointPropertiesGroup("QPoint") ;
		
		{ // X coordinate /////////////////////////////////////////////////////
			QString name = "x" ;
			QVariant value = point->x() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = pointPropertiesGroup.properties;
			groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////
	
		{ // Y coordinate /////////////////////////////////////////////////////
		QString name = "y" ;
		QVariant value = point->y() ;
		IProperty * p = 0;
		p = new DoubleProperty(qvariant_cast<double>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = pointPropertiesGroup.properties;
		groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////
	
		{ // Z coordinate /////////////////////////////////////////////////////
		QString name = "z" ;
		QVariant value = point->z() ;
		IProperty * p = 0;
		p = new DoubleProperty(qvariant_cast<double>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = pointPropertiesGroup.properties;
		groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////
		
		groups.append(pointPropertiesGroup) ;
	}
	
	/*****************************************************************************
	 * QPlane Properties
	 *****************************************************************************/
	
	if(QPlane * plane = qobject_cast<QPlane *>(object)) {
		
		IGroup planePropertiesGroup("QPlane") ;
		
		{ // A coefficient ///////////////////////////////////////////////////////
			QString name = "a" ;
			QVariant value = plane->a() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = planePropertiesGroup.properties;
			groupProperties.append(p);
		} ////////////////////////////////////////////////////////////////////////
		
		{ // B coefficient ///////////////////////////////////////////////////////
			QString name = "b" ;
			QVariant value = plane->b() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = planePropertiesGroup.properties;
			groupProperties.append(p);
		} ////////////////////////////////////////////////////////////////////////
		
		{ // C coefficient ///////////////////////////////////////////////////////
			QString name = "c" ;
			QVariant value = plane->c() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = planePropertiesGroup.properties;
			groupProperties.append(p);
		} ////////////////////////////////////////////////////////////////////////
		
		{ // D coefficient ///////////////////////////////////////////////////////
			QString name = "d" ;
			QVariant value = plane->d() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = planePropertiesGroup.properties;
			groupProperties.append(p);
		} ///////////////////////////////////////////////////////////////////////
		
		{ // Visual Size ////////////////////////////////////////////////////////
			QString name = "size" ;
			QVariant value = plane->size() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = planePropertiesGroup.properties;
			groupProperties.append(p);
		} ///////////////////////////////////////////////////////////////////////
		
		groups.append(planePropertiesGroup) ;
	}
	
	/*****************************************************************************
	 * QConstraintDistance Properties
	 *****************************************************************************/
	
	if(QConstraintDistance * distance = qobject_cast<QConstraintDistance *>(object)) 
	{
		IGroup distanceConstraintPropertiesGroup("QConstraintDistance") ;
	
		{ // Value ///////////////////////////////////////////////////////
			QString name = "value" ;
			QVariant value = distance->getValue() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = distanceConstraintPropertiesGroup.properties;
			groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////
		
		groups.append(distanceConstraintPropertiesGroup) ;
	}
	
	/*****************************************************************************
	 * QCurve Properties
	 *****************************************************************************/
	
	if(QCurve * curve = qobject_cast<QCurve *>(object)) {
		
		IGroup curvePropertiesGroup("QCurve") ;
	
		{ // Draw Bounding box //////////////////////////////////////////////////
			QString name = "Draw Bounding Box" ;
			QVariant value = curve->bbDrawn() ;
			IProperty * p = 0;
			p = new BoolProperty(qvariant_cast<bool>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = curvePropertiesGroup.properties;
			groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////

// 		{ // xmin ///////////////////////////////////////////////////////
// 			QString name = "xmin" ;
// 			QVariant value = curve->boundingbox()->xmin() ;
// 			IProperty * p = 0;
// 			p = new DoubleProperty(qvariant_cast<double>(value), name);
// 			p->setHasReset(false);
// 			p->setChanged(false);
// 			p->setDirty(false);
// 			QList<IProperty*> &groupProperties = curvePropertiesGroup.properties;
// 			groupProperties.append(p);
// 		} /////////////////////////////////////////////////////////////////
		
// 		{ // xmax ///////////////////////////////////////////////////////
// 			QString name = "xmax" ;
// 			QVariant value = curve->boundingbox()->xmax() ;
// 			IProperty * p = 0;
// 			p = new DoubleProperty(qvariant_cast<double>(value), name);
// 			p->setHasReset(false);
// 			p->setChanged(false);
// 			p->setDirty(false);
// 			QList<IProperty*> &groupProperties = curvePropertiesGroup.properties;
// 			groupProperties.append(p);
// 		} /////////////////////////////////////////////////////////////////
		
// 		{ // ymin ///////////////////////////////////////////////////////
// 			QString name = "ymin" ;
// 			QVariant value = curve->boundingbox()->ymin() ;
// 			IProperty * p = 0;
// 			p = new DoubleProperty(qvariant_cast<double>(value), name);
// 			p->setHasReset(false);
// 			p->setChanged(false);
// 			p->setDirty(false);
// 			QList<IProperty*> &groupProperties = curvePropertiesGroup.properties;
// 			groupProperties.append(p);
// 		} /////////////////////////////////////////////////////////////////
		
// 		{ // ymax ///////////////////////////////////////////////////////
// 			QString name = "ymax" ;
// 			QVariant value = curve->boundingbox()->ymax() ;
// 			IProperty * p = 0;
// 			p = new DoubleProperty(qvariant_cast<double>(value), name);
// 			p->setHasReset(false);
// 			p->setChanged(false);
// 			p->setDirty(false);
// 			QList<IProperty*> &groupProperties = curvePropertiesGroup.properties;
// 			groupProperties.append(p);
// 		} /////////////////////////////////////////////////////////////////
		
		groups.append(curvePropertiesGroup) ;
		
		/*****************************************************************************
		 * QImplicitCurve2D Properties
		 *****************************************************************************/
		
		if(QImplicitCurve2D * implicitcurve2D = qobject_cast<QImplicitCurve2D *>(curve)) {
			IGroup implicitCurve2DPropertiesGroup("QImplicitCurve 2D") ;
			
			{ // Polynomial Property ////////////////////////////////////////////
				QString name = "Polynomial" ;
				QVariant value = implicitcurve2D->equation() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = implicitCurve2DPropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////
			
			groups.append(implicitCurve2DPropertiesGroup) ;
		}
		
		/*****************************************************************************
		 * QImplicitCurve3D Properties
		 *****************************************************************************/
		
		if(QImplicitCurve3D * implicitcurve3D = qobject_cast<QImplicitCurve3D *>(curve)) {
			
			IGroup implicitCurve3DPropertiesGroup("QImplicitCurve3D") ;
			
			{ // First Polynomial Property ////////////////////////////////////////
				QString name = "First Polynomial" ;
				QVariant value = implicitcurve3D->equation1() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = implicitCurve3DPropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////
			
			{ // Second Polynomial Property ////////////////////////////////////
				QString name = "Second Polynomial" ;
				QVariant value = implicitcurve3D->equation2() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = implicitCurve3DPropertiesGroup.properties;
				groupProperties.append(p);
			}
			//////////////////////////////////////////////////////////////////////

			groups.append(implicitCurve3DPropertiesGroup) ;
		}
		
		/*****************************************************************************
		 * QRationalCurve Properties
		 *****************************************************************************/
		
		if(QRationalCurve * rationalCurve = qobject_cast<QRationalCurve *>(curve)) {
			
			IGroup rationalCurvePropertiesGroup("QRationalCurve") ;
			
			{ // x(t) ////////////////////////////////////////
				QString name = "x(t)" ;
				QVariant value = rationalCurve->px() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = rationalCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////
			
			{ // y(t) ////////////////////////////////////
				QString name = "y(t)" ;
				QVariant value = rationalCurve->py() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = rationalCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////

			{ // z(t) ////////////////////////////////////
				QString name = "z(t)" ;
				QVariant value = rationalCurve->pz() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = rationalCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////

			{ // w(t) ////////////////////////////////////
				QString name = "w(t)" ;
				QVariant value = rationalCurve->pw() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = rationalCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////
			
			{ // tmin ////////////////////////////////////
				QString name = "tmin" ;
				QVariant value = rationalCurve->tmin() ;
				IProperty * p = 0;
				p = new DoubleProperty(qvariant_cast<double>(value), name); 
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = rationalCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////

			{ // tmax ////////////////////////////////////
				QString name = "tmin" ;
				QVariant value = rationalCurve->tmin() ;
				IProperty * p = 0;
				p = new DoubleProperty(qvariant_cast<double>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = rationalCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////

			groups.append(rationalCurvePropertiesGroup) ;
		}
		
		/*****************************************************************************
		 * QBSplineCurve Properties
		 *****************************************************************************/
		
		if(QBSplineCurve * bsplineCurve = qobject_cast<QBSplineCurve *>(curve)) {
			
			IGroup bsplineCurvePropertiesGroup("QBSpline") ;
			
			{ // Order ////////////////////////////////////
				QString name = "Order" ;
				QVariant value = bsplineCurve->order ;
				IProperty * p = 0;
				p = new IntProperty(qvariant_cast<int>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = bsplineCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////

			{ // Number ////////////////////////////////////
				QString name = "Number" ;
				QVariant value = bsplineCurve->number ;
				IProperty * p = 0;
				p = new IntProperty(qvariant_cast<int>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = bsplineCurvePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////

			groups.append(bsplineCurvePropertiesGroup) ;
		}
		
	}
	
	/*****************************************************************************
	 * QSurface Properties
	 *****************************************************************************/
	
	if(QSurface * surface = qobject_cast<QSurface *>(object)) {
		IGroup surfacePropertiesGroup("QSurface") ;
		
		{ // Draw Bounding box /////////////////////////////////////////////////
			QString name = "Draw Bounding Box" ;
			QVariant value = surface->bbDrawn() ;
			IProperty * p = 0;
			p = new BoolProperty(qvariant_cast<bool>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = surfacePropertiesGroup.properties;
			groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////
		
		groups.append(surfacePropertiesGroup) ;
		
		/*****************************************************************************
		 * QImplicitSurface Properties
		 *****************************************************************************/
		
		if(QImplicitSurface * implicitsurface = qobject_cast<QImplicitSurface *>(surface)) {
			IGroup implicitSurfacePropertiesGroup("QImplicitSurface") ;
			
			{ // Polynomial Property ///////////////////////////////////////////////
				QString name = "Polynomial" ;
				QVariant value = implicitsurface->equation() ;
				IProperty * p = 0;
				p = new StringProperty(qvariant_cast<QString>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = implicitSurfacePropertiesGroup.properties;
				groupProperties.append(p);
			} //////////////////////////////////////////////////////////////////////
			
			{ // Draw Singularities //////////////////////////////////////////////////
				QString name = "Draw Singularities" ;
				QVariant value = implicitsurface->singularitiesDrawn() ;
				IProperty * p = 0;
				p = new BoolProperty(qvariant_cast<bool>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = implicitSurfacePropertiesGroup.properties;
				groupProperties.append(p);
			} /////////////////////////////////////////////////////////////////////
			
			{ // # Singularities //////////////////////////////////////////////////
				QString name = "Number of Singularities" ;
				QVariant value = implicitsurface->nbSingularities() ;
				IProperty * p = 0;
				p = new IntProperty(qvariant_cast<int>(value), name);        
				p->setHasReset(false);
				p->setChanged(false);
				p->setDirty(false);
				QList<IProperty*> &groupProperties = implicitSurfacePropertiesGroup.properties;
				groupProperties.append(p);
			} /////////////////////////////////////////////////////////////////////
			
			groups.append(implicitSurfacePropertiesGroup) ;
		}
	}
	
	/*****************************************************************************
	 * QVertex Properties
	 *****************************************************************************/

	if(QVertex * vertex = qobject_cast<QVertex *>(object)) {
		IGroup vertexPropertiesGroup("QVertex") ;
	
		{ // X coordinate /////////////////////////////////////////////////////
			QString name = "x" ;
			QVariant value = vertex->x() ;
			IProperty * p = 0;
			p = new DoubleProperty(qvariant_cast<double>(value), name);
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = vertexPropertiesGroup.properties;
			groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////

		{ // Y coordinate /////////////////////////////////////////////////////
		QString name = "y" ;
		QVariant value = vertex->y() ;
		IProperty * p = 0;
		p = new DoubleProperty(qvariant_cast<double>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = vertexPropertiesGroup.properties;
		groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////

		{ // Z coordinate /////////////////////////////////////////////////////
		QString name = "z" ;
		QVariant value = vertex->z() ;
		IProperty * p = 0;
		p = new DoubleProperty(qvariant_cast<double>(value), name);        
		p->setHasReset(false);
		p->setChanged(false);
		p->setDirty(false);
		QList<IProperty*> &groupProperties = vertexPropertiesGroup.properties;
		groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////
		
		groups.append(vertexPropertiesGroup) ;
	}
	
	/*****************************************************************************
	 * QEdge Properties
	 *****************************************************************************/
	
	if(/*QEdge * edge = */qobject_cast<QEdge *>(object)) {
		IGroup edgePropertiesGroup("QEdge") ;
		groups.append(edgePropertiesGroup) ;
	}
	
	/*****************************************************************************
	 * QFace Properties
	 *****************************************************************************/
	
	if(/*QFace * face = */qobject_cast<QFace *>(object)) {
		IGroup facePropertiesGroup("QFace") ;
		groups.append(facePropertiesGroup) ;
	}
	
	/*****************************************************************************
	 * QMesh Properties
	 *****************************************************************************/
	
	if(QMesh * mesh = qobject_cast<QMesh *>(object)) {
		IGroup meshPropertiesGroup("QMesh") ;
		
		// # vertices ///////////////////////////////////////////////////////
		{
			QString name = "Number of vertices" ;
			QVariant value = mesh->vertices_.size() ;
			IProperty * p = 0;
			p = new IntProperty(qvariant_cast<int>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(true);
			QList<IProperty*> &groupProperties = meshPropertiesGroup.properties;
			groupProperties.append(p);
		}
		/////////////////////////////////////////////////////////////////////
		
		// # edges ///////////////////////////////////////////////////////
		{
			QString name = "Number of edges" ;
			QVariant value = mesh->edges_.size() ;
			IProperty * p = 0;
			p = new IntProperty(qvariant_cast<int>(value), name);        
			QList<IProperty*> &groupProperties = meshPropertiesGroup.properties;
			groupProperties.append(p);
		}
		/////////////////////////////////////////////////////////////////////
		
		// # faces ///////////////////////////////////////////////////////
		{
			QString name = "Number of faces" ;
			QVariant value = mesh->faces_.size() ;
			IProperty * p = 0;
			p = new IntProperty(qvariant_cast<int>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = meshPropertiesGroup.properties;
			groupProperties.append(p);
		}
		/////////////////////////////////////////////////////////////////////

		{ // Draw Edges /////////////////////////////////////////////////////
			QString name = "Draw Edges" ;
			QVariant value = mesh->drawEdges ;
			IProperty * p = 0;
			p = new BoolProperty(qvariant_cast<bool>(value), name);        
			p->setHasReset(false);
			p->setChanged(false);
			p->setDirty(false);
			QList<IProperty*> &groupProperties = meshPropertiesGroup.properties;
			groupProperties.append(p);
		} /////////////////////////////////////////////////////////////////////
		
		groups.append(meshPropertiesGroup) ;
	}
	
	foreach (IGroup g, groups) 
	{
		root->addProperty(new SeparatorProperty(QString(), g.name));
		foreach (IProperty *p, g.properties) 
		{
			root->addProperty(p);
		}
	}
}
