# include <QtGui>

# include <QAxel/PluginManager.h>

# include <QGui/PluginDialog.h>

# include <QSisl/QSislInterface.h>

# include <QSynaps/QSynapsInterface.h>

# include <QCgal/QCgalInterface.h>

# include <QCore/QCoreInterface.h>

PluginDialog::PluginDialog(const QString &path, const QStringList &fileNames, QWidget *parent) : QDialog(parent)
{
    label = new QLabel;
    label->setWordWrap(true);
    
    QStringList headerLabels;
    headerLabels << tr("Components");

    treeWidget = new QTreeWidget;
    treeWidget->setAlternatingRowColors(false);
    //    treeWidget->setSelectionMode(QAbstractItemView::MultiSelection);
    treeWidget->setHeaderLabels(headerLabels);
    treeWidget->header()->hide();

    okButton = new QPushButton(tr("OK"));
    okButton->setDefault(true);

    connect(okButton, SIGNAL(clicked()), this, SLOT(close()));

    QGridLayout *mainLayout = new QGridLayout;
    mainLayout->setColumnStretch(0, 1);
    mainLayout->setColumnStretch(2, 1);
    mainLayout->addWidget(label, 0, 0, 1, 3);
    mainLayout->addWidget(treeWidget, 1, 0, 1, 3);
    mainLayout->addWidget(okButton, 2, 1);
    setLayout(mainLayout);

    interfaceIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirOpenIcon),
                            QIcon::Normal, QIcon::On);
    interfaceIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirClosedIcon),
                            QIcon::Normal, QIcon::Off);
    featureIcon.addPixmap(style()->standardPixmap(QStyle::SP_FileIcon));

    setWindowTitle(tr("Plugin Information"));
    setWindowFlags(Qt::Tool) ;
    populateTreeWidget(path, fileNames);

    connect(treeWidget, SIGNAL(itemClicked(QTreeWidgetItem*,int)), this, SLOT(itemClicked(QTreeWidgetItem*,int))) ;
}

void PluginDialog::populateTreeWidget(const QString &path,
                                      const QStringList &fileNames)
{
    if (fileNames.isEmpty()) {
	label->setText(tr("Axel couldn't find any plugins in the %1 directory.").arg(QDir::convertSeparators(path)));
	treeWidget->hide();
    } else {
	label->setText(tr("Axel found the following plugins in the %1 directory:").arg(QDir::convertSeparators(path)));
	
	QDir dir(path);
	
	foreach (QString fileName, fileNames) {
	    // QPluginLoader * loader = PluginFactory::instance()->loader(dir.absoluteFilePath(fileName));
	    QObject * plugin = PluginManager::instance()->plugin(fileName.section('.',0,0).remove("libQ"));
	    
	    QTreeWidgetItem *pluginItem = new QTreeWidgetItem(treeWidget);
	    pluginItem->setText(0, fileName);
	    treeWidget->setItemExpanded(pluginItem, true);
	    
	    QFont boldFont = pluginItem->font(0);
	    boldFont.setBold(true);
	    pluginItem->setFont(0, boldFont);
	    pluginItem->setFlags(Qt::ItemIsUserCheckable) ;
	    pluginItem->setCheckState(0, Qt::Checked);
	    
	    if (plugin) {

		QSislInterface *iSisl = qobject_cast<QSislInterface *>(plugin);
		if (iSisl) addItems(pluginItem, "Sisl Interface", iSisl->features());
		
		QSynapsInterface *iSynaps = qobject_cast<QSynapsInterface *>(plugin);
		if (iSynaps) addItems(pluginItem, "Synaps Interface", iSynaps->features());
		
		QCgalInterface *iCgal = qobject_cast<QCgalInterface *>(plugin);
		if (iCgal) addItems(pluginItem, "Cgal Interface", iCgal->features());
		
		QCoreInterface *iCore = qobject_cast<QCoreInterface *>(plugin);
		if (iCore) addItems(pluginItem, "Core Interface", iCore->features());
	    }
	}
    }
}

void PluginDialog::addItems(QTreeWidgetItem *pluginItem,
                            const char *interfaceName,
                            const QStringList &features)
{
    Q_UNUSED(features) ;
    QTreeWidgetItem *interfaceItem = new QTreeWidgetItem(pluginItem);
    interfaceItem->setText(0, interfaceName);
    interfaceItem->setIcon(0, interfaceIcon);

    //foreach (QString feature, features) {
    //    if (feature.endsWith("..."))
    //        feature.chop(3);
    //    QTreeWidgetItem *featureItem = new QTreeWidgetItem(interfaceItem);
    //    featureItem->setText(0, feature);
    //    featureItem->setIcon(0, featureIcon);
    //}
}

void PluginDialog::itemClicked(QTreeWidgetItem * item, int i)
{
    Q_UNUSED(i) ;
    //  PluginManager::instance()->unloadPlugin(item->text(0)) ;
    cout << item->text(0).toAscii().constData() << "has been clicked" << endl ;
}
