/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QGraphWidget
 * 2006-07-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QGRAPHWIDGET_H
# define QGRAPHWIDGET_H

# include <QSet>

# include <QData/QGraph.h> 
# include <QData/QGraphNode.h> 
# include <QData/QGraphEdge.h> 

# include <QGui/Viewer.h>
# include <QGui/QGraphicNode.h> 
# include <QGui/QGraphicEdge.h> 

# include <QtGui/QGraphicsView>

using namespace QData ;

class QGraphWidget : public QGraphicsView
{
    Q_OBJECT

public:
    QGraphWidget(QWidget * parent);

    enum State { NONE = 0, PANNING = 1} ;

public slots:
    void setGraph(QGraph * g) ;
    void setRandomLayout(void) ;
    void setConcentricLayout(void) ;
    void setColimaconLayout(void) ;
    void setDirectedTreeLayout(void) ;
    void update(void) ;
    void clear(void) ;
    void setViewer(Viewer * v) { viewer = v ; }
    
protected:
    void keyPressEvent(QKeyEvent * event);
    void mousePressEvent(QMouseEvent * e) ;
    void mouseReleaseEvent(QMouseEvent * e) ;
    void mouseMoveEvent(QMouseEvent * e) ;
    void wheelEvent(QWheelEvent *event) ;

    void drawForeground(QPainter * painter, const QRectF & rect) ;
    void drawBackground(QPainter * painter, const QRectF & rect);
    void scaleView(qreal scaleFactor);
    void resetNodesPositions(void) ;
    void setDirectedTreeLayout(QGraphNode * n, QPointF& point, int depth, int delta, int siblings) ;
    float middleAbsissaOfChildNodes(QGraphNode * n) ;
    
    State getState(void) { return state ; }
    void setState(State s) { state = s ; }

private:
    QGraph * graph ;
    QMap<QGraphNode *, QGraphicNode *> nodeMap ;
    QMap<QGraphEdge *, QGraphicEdge *> edgeMap ;
    QSet<QGraphNode *> marked ;

private:
    QPointF start ;
    State state ;
    Viewer * viewer ;
} ;

#endif
