/****************************************************************************
**
** Copyright (C) 2006-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** Licensees holding valid Qt Preview licenses may use this file in
** accordance with the Qt Preview License Agreement provided with the
** Software.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about Qt Commercial License Agreements.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

# ifndef QGRAPHICNODE_H
# define QGRAPHICNODE_H

# include <QColor>
# include <QGraphicsItem>
# include <QList>

class QGraphicEdge;
class QGraphWidget;
class QGraphicsSceneMouseEvent;

class QGraphicNode : public QGraphicsItem
{
public:
    QGraphicNode(QGraphWidget *graphWidget);
    QGraphicNode(QGraphWidget *graphWidget, QString label, QColor lightColor = Qt::yellow, QColor darkColor = Qt::darkYellow);
    
    void setColors(QColor lightColor, QColor darkColor) ;
    void setLightColor(QColor lightColor) ;
    void setDarkColor(QColor darkColor) ;
   
    QColor lightColor(void) { return n_lightColor ; }
    QColor darkColor(void)  { return n_darkColor ; }
 
    void addEdge(QGraphicEdge *edge);
    QList<QGraphicEdge *> edges() const;
    
    enum { Type = UserType + 1 };
    int type() const { return Type; }

    void calculateForces();
    bool advance();

    QRectF boundingRect() const;
    QPainterPath shape() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

protected:
    QVariant itemChange(GraphicsItemChange change, const QVariant &value);

    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    
private:
    QList<QGraphicEdge *> edgeList;
    QPointF newPos;
    QGraphWidget *graph;
    QString label ;
    QColor n_lightColor ;
    QColor n_darkColor ;
} ;

#endif
