/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ToolPropertiesDock
 * 2006-03-31
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 * Current widgets of scrollArea are destroyed by calling 
 * scrollArea->setWidget().
 ****************************************************************************/
                         
# include <QScrollArea>
# include <QGui/MainWindow.h>
# include <QGui/ToolPropertiesDock.h>
# include <QGui/ToolPropertiesWidgets.h>

ToolPropertiesDock::ToolPropertiesDock(MainWindow * mw, const QString & title, QWidget * parent) : QDockWidget(title, parent)
{
    mainwindow = mw ;
    
    scrollArea = new QScrollArea(this) ;
    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded) ;
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded) ;
    scrollArea->setWidgetResizable(true) ;
    scrollArea->setWidget(new NoToolWidget(mainwindow, scrollArea->viewport())) ;
    
    setWidget(scrollArea) ;
}

void ToolPropertiesDock::createNoToolWidget(void) { scrollArea->setWidget(new NoToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createPointWidget(void) { scrollArea->setWidget(new PointToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createPlaneWidget(void) { scrollArea->setWidget(new PlaneToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createCurveWidget(void) { scrollArea->setWidget(new CurveToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createSurfaceWidget(void) { scrollArea->setWidget(new SurfaceToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createConvertToMeshWidget(void) { scrollArea->setWidget(new ConvertToMeshToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createInterpolationBSplineCurveWidget(void) { scrollArea->setWidget(new InterpolationBSplineCurveToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createApproximationBSplineCurveWidget(void) { scrollArea->setWidget(new ApproximationBSplineCurveToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createApproximationBSplineSurfaceWidget(void) { scrollArea->setWidget(new ApproximationBSplineSurfaceToolWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createJoinBSplineCurveWidget(void) { scrollArea->setWidget(new JoinBSplineCurveToolWidget(mainwindow, scrollArea->viewport())) ; }

void ToolPropertiesDock::createCurvesArrangementWidget(void) { scrollArea->setWidget(new CurveArrangementWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createSurfacesArrangementWidget(void) { /* scrollArea->setWidget(new SurfaceArrangementWidget(mainwindow, scrollArea->viewport())) ;*/ }
void ToolPropertiesDock::createFirstDerivativeWidget(void) { scrollArea->setWidget(new FirstDerivativeWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createSecondDerivativeWidget(void) { scrollArea->setWidget(new SecondDerivativeWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createJacobiCurveWidget(void) { scrollArea->setWidget(new JacobiCurveWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createHessianCurveWidget(void) { scrollArea->setWidget(new HessianCurveWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createGraphedSurfaceWidget(void) { scrollArea->setWidget(new GraphedSurfaceWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createIcIntersectionWidget(void)  { scrollArea->setWidget(new IcIntersectionWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createPcIntersectionWidget(void)  { scrollArea->setWidget(new PcIntersectionWidget(mainwindow, scrollArea->viewport())) ; }	
void ToolPropertiesDock::createIsIntersectionWidget(void)  { scrollArea->setWidget(new IsIntersectionWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createPsIntersectionWidget(void)  { scrollArea->setWidget(new PsIntersectionWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createDistanceConstraintWidget(void)  { scrollArea->setWidget(new DistanceConstraintWidget(mainwindow, scrollArea->viewport())) ; }	
void ToolPropertiesDock::createIncidenceConstraintWidget(void)  { scrollArea->setWidget(new IncidenceConstraintWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createAngleConstraintWidget(void)  { scrollArea->setWidget(new AngleConstraintWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createSurfRenderWidget(void) { scrollArea->setWidget(new SurfRenderWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createYafrayRenderWidget(void) { scrollArea->setWidget(new YafrayRenderWidget(mainwindow, scrollArea->viewport())) ; }
void ToolPropertiesDock::createBezierClipperWidget(void) { scrollArea->setWidget(new BezierClipperToolWidget(mainwindow, scrollArea->viewport())) ; }
