/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ViewerMouseEvents.h
 * 2006-11-08
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QFile>
# include <QCursor>
# include <QAction>
# include <QMenu>

# include <QGui/Viewer.h>
# include <QGui/MainWindow.h>

# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QCurve.h>
# include <QAxel/QSurface.h>

# include <QStream/IStream/IStreamAxl.h>

using namespace QStream ;

void Viewer::mousePressEvent(QMouseEvent* e) 
{
    //************** AF SandBox
    if(directMode) {
	// Start selection. Mode is ADD with Shift key and TOGGLE with Alt key.
	rectangle_ = QRect(e->pos(), e->pos());
	if(e->modifiers() == Qt::ShiftModifier) {
	    selectionMode_ = ADD;
	} else {
	    if (e->modifiers() == Qt::ControlModifier){
		startManipulation();
		QGLViewer::mousePressEvent(e);
	    } else {
		if (e->button() == Qt::LeftButton) {
		    selectionMode_ = DIRECT;
		    addObjectDirectly = true;
		}
		
		if(selection_.empty()) {
		    setMouseBinding(Qt::LeftButton, CAMERA, ROTATE);
		    setMouseBinding(Qt::RightButton, CAMERA, TRANSLATE);
		    setWheelBinding(Qt::NoModifier, CAMERA, ZOOM);
		    // QGLViewer::mousePressEvent(e);
		} else {
		    setMouseBinding(Qt::LeftButton, FRAME, ROTATE);
		    setMouseBinding(Qt::RightButton, FRAME, TRANSLATE);
		    setWheelBinding(Qt::NoModifier, FRAME, ZOOM);
		    // QGLViewer::mousePressEvent(e);
		}
		startManipulation();
		QGLViewer::mousePressEvent(e);
	    }
	}
	QGLViewer::mousePressEvent(e);
    //************** AF SandBox		
    } else {
//***********************AF
	// Start selection. Mode is ADD with Shift key and TOGGLE with Alt key.
	rectangle_ = QRect(e->pos(), e->pos());
	
	if ((e->button() == Qt::LeftButton) && (e->modifiers() == Qt::ShiftModifier))
	    selectionMode_ = ADD;
	else
	    if ((e->button() == Qt::LeftButton) && (e->modifiers() == Qt::AltModifier))
		selectionMode_ = REMOVE;
	    else {
		if (e->modifiers() == Qt::ControlModifier) startManipulation();
		QGLViewer::mousePressEvent(e);
	    }
//***********************AF
    }
}

void Viewer::mouseMoveEvent(QMouseEvent* e) 
{
    //************** AF SandBox
    if (directMode) {
	if(selectionMode_ != NONE) {
	    addObjectDirectly=false;
	    if(!selection_.empty()) {
		emit manipulation(selection_.first());
	    }
	    // 			if(selection_.empty())
	}
	QGLViewer::mouseMoveEvent(e) ;
    } else {
//***********************AF
	if (selectionMode_ != NONE) {
	    // Updates rectangle_ coordinates and redraws rectangle
	    rectangle_.setBottomRight(e->pos());
	    updateGL();
	} else {
	    QGLViewer::mouseMoveEvent(e) ;
	    if(!selection_.empty()) emit manipulation(selection_.first()) ;
	}
//***********************AF
    }

    QPoint3f * p = convertClickToPoint(e) ;
    
    Vec oldPos(v_oldMousePosition->x(), 
	       v_oldMousePosition->y(), 
	       v_oldMousePosition->z()) ;
    Vec newPos(p->x(),
	       p->y(),
	       p->z()) ;
    Vec translation = newPos - oldPos ;


    v_oldMousePosition = p ;

    emit mouseMoved(p) ;
//    emit prout(translation) ;
    if(mf_move) {
	foreach(QAxelObject * p, listSelectedObjectsFromType("Point3f"))
	    p->frame()->translate(translation) ;
    }  
    updateGL() ;
}

void Viewer::mouseReleaseEvent(QMouseEvent* e)
{
    //************** AF SandBox
    if(directMode) {
	if(selectionMode_ != NONE) {
	    if(addObjectDirectly) {
		rectangle_.setBottomRight(e->pos());
		rectangle_ = rectangle_.normalized();
		setSelectRegionWidth(rectangle_.width());
		setSelectRegionHeight(rectangle_.height());
		select(rectangle_.center());
		updateGL();
	    }
	}

	QGLViewer::mouseReleaseEvent(e);
    //***********************AF
    } else if(listExtrusionObjects().size() == 1) {
	
	QList<QAxelObject *> l = listExtrusionObjects() ;
	
	QBSplineCurve * c = qobject_cast<QBSplineCurve *>(l.first()) ;

	QBSplineSurface * s = qobject_cast<QBSplineSurface *>(l.first()) ;
	
	if(e->button() == Qt::LeftButton) {
	    
	    QList<QAxelObject *> l = listSelectedObjectsFromType("Point3f") ;

	    if(c) {
		c->setMode(QAxelObject::Edit) ;    
		foreach(QAxelObject * p, l)
		    disconnect(this, SIGNAL(mouseMoved(QPoint3f*)), qobject_cast<QPoint3f *>(p), SLOT(setPosition(QPoint3f*))) ;		
	    }

	    if(s) {
		s->setMode(QAxelObject::Edit) ;    
//		disconnect(this, SIGNAL(mouseTranslated(Vec &v)), manipulatedFrame(), SLOT(translate(v))) ;
	    }	    
			
	    setMouseTracking(false) ;
	    
	    if(mf_move)
		mf_move = false ;

	    update() ;

	} else if (e->button() == Qt::RightButton) {
	    
 	    QList<QAxelObject *> l = listSelectedObjectsFromType("Point3f") ;
	    
	    QPoint3f * cp = qobject_cast<QPoint3f *>(l.first()) ;
	    	    
	    removeObjectFromSelection(cp) ;
 	    disconnect(this, SIGNAL(mouseMoved(QPoint3f*)), cp, SLOT(setPosition(QPoint3f*))) ;

	    if(cp == c->points.last())
		cp->setPosition(c->points.first()) ;	    
	    else
		cp->setPosition(c->points.last()) ;

	    c->cuopen = 0 ;
	    c->compute() ;
	    
	    c->setMode(QAxelObject::Edit) ;
	    setMouseTracking(false) ;	    
	    
	    update() ;
	    
	}
    } else if(!(listGrabbedObjects().empty())) {
    
	QList<QAxelObject *> l = listGrabbedObjects() ;
	
	foreach(QAxelObject * o , l) 
	    o->setMode(QAxelObject::None) ;

	setMouseTracking(false) ;
	
	update() ;

    } else {  
	//***********************AF
	if (selectionMode_ != NONE) {
	    // Actual selection on the rectangular area.
	    // Possibly swap left/right and top/bottom to make rectangle_ valid.
	    rectangle_ = rectangle_.normalized();
	    
	    // Define selection window dimensions
	    setSelectRegionWidth(rectangle_.width());
	    setSelectRegionHeight(rectangle_.height());
	    // Compute rectangle center and perform selection
	    select(rectangle_.center());
	    // Update display to show new selected objects
	    updateGL();
	} else QGLViewer::mouseReleaseEvent(e);
	//***********************AF
    }

}

void Viewer::mouseDoubleClickEvent(QMouseEvent * e)
{
    Q_UNUSED(e) ;
    QGLViewer::mouseDoubleClickEvent(e) ;
}

void Viewer::dropEvent(QDropEvent * event)
{
    const QMimeData *mimeData = event->mimeData();

    if (mimeData->hasText()) {
      cout << "about to parse" << mimeData->text().remove("file://").toAscii().constData() << endl ;
      IStreamAxl parser(mimeData->text().remove("file://"), dynamic_cast<MainWindow *>(parent()));
      parser.run() ;
    }
    event->acceptProposedAction();
}

void Viewer::dragEnterEvent(QDragEnterEvent *event)
{
    event->acceptProposedAction();
}

QPoint3f * Viewer::convertClickToPoint(QMouseEvent * e)
{
    using qglviewer::Camera ;
    using qglviewer::Vec ;
    
    Vec orig, dir ;
    QPoint p = QPoint(e->x(),e->y()) ; 
    camera()->convertClickToLine(p, orig, dir) ;

    float t = - orig.z/dir.z ;
    Vec poi = orig + t*dir ;
    
    QPoint3f * point = new QPoint3f(poi.x, poi.y, 0.0) ;
    
    return point ;
    
}

/*bool Viewer::editionMode(void)
{
    if(
    
    }*/
