/******************************************************************************
 * A x e l                                                                    *
 ******************************************************************************
 * QRationalSurfaceIntersection                                               *
 * 2007-02-24                                                                 *
 * Séphane CHAU                                                               *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                    *
 ******************************************************************************
 *                                                                            *
 ******************************************************************************/

# ifndef QRATIONALSURFACEINTERSECTION_H
# define QRATIONALSURFACEINTERSECTION_H

# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QIntersection/Surfaces/QSurfaceIntersection.h>
# include <QAxel/QPoint4f.h>
# include <QData/QCell4D.h>
# include <QData/QTreeNode4D.h>
# include <QData/QHexatree.h>

using namespace QAxel ;
using namespace QData ;

namespace QIntersection {

typedef QAxelObject Object_t ;
typedef QPoint4f Point_t ;
typedef QCell4D Cell_t ;
typedef QPair<Point_t *, Point_t *> Segment_t;
// typedef QTreeNode4D<QList<Segment_t> , Cell_t *> Node_t ;
typedef QTreeNode4D<Segment_t , Cell_t *> Node_t ;
// typedef QHexatree<QList<Segment_t> , Cell_t *> Hexatree_t ;
typedef QHexatree<Segment_t , Cell_t *> Hexatree_t ;
typedef QList< QPair<QPoint3f*, QPoint3f *> > topology_t ;

// Then 4D implicit intersection curve of 2 rational surfaces has 4 variables corresponding to the direction of the 4D space.
enum DIRECTION{DIR_S, DIR_T, DIR_U, DIR_V};


class QRationalSurfaceIntersection : public QSurfaceIntersection
{
    Q_OBJECT
	
public:
	QRationalSurfaceIntersection(QRationalSurface * s1, QRationalSurface * s2, MainWindow * mw) ;
	~QRationalSurfaceIntersection(void) ;
	QImplicitCurve4D * getIntersectionCurve4D(void);
	bool isRegular(QImplicitCurve4D * c, Node_t * no, int i) ;
	// QList<Cell_t *> subdivide(QImplicitCurve4D * c, Node_t * n) ;
	void subdivide(QImplicitCurve4D * c, Node_t * n) ;
	// bool order(Point_t * p1, Point_t * p2, int i) ;
	void sort4D(QList<Point_t *> l, int i);
	QList<Segment_t> topology(Node_t * no, int i);
	QList<Segment_t> getTopology4D(QImplicitCurve4D * c) ;
	QList<Segment_t> dummy(Node_t * n);
	QList<Segment_t> fusion(Hexatree_t hexatree);
	QList<Segment_t> fusion(Node_t * node);
	// QList<Segment_t> merge(QList<Segment_t> top1, QList<Segment_t> top2);
	topology_t getTopology3D(QList<Segment_t> top4D);
	
public slots:
	void run(void) ;

private:
	QImplicitCurve4D * curve ;
	QRationalSurface * s1 ;
	QRationalSurface * s2 ;
	
protected:
	bool isLocallyInjective(QImplicitCurve4D * c, DIRECTION dir) ;

} ;

} ; // namespace QIntersection ;

# endif // QRATIONALSURFACEINTERSECTION_H
