# include <QString>
# include <QPair>
# include <QAxel/PluginManager.h>
# include <QSynaps/QSynapsInterface.h>
# include <QData/QCell4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QIntersection/Surfaces/QSurfaceIntersection.h>
# include <QAxel/QPoint4f.h>
# include <QData/QCell4D.h>
# include <QData/QTreeNode4D.h>
# include <QData/QHexatree.h>
# include <QGui/color.h>

using namespace QAxel ;
using namespace QData ;
using namespace std ;

// Create rational surfaces 
QList< QPair<QRationalSurface *, QRationalSurface *> > createSurfaces(void)
{
	QList < QPair<QRationalSurface *, QRationalSurface *> > lsurf;
	QRationalSurface * S1 = new QRationalSurface("s^2t^2+st^3","s^2t+st^2","s^2t^2+st-t^2","1",0,1,0,1);
	QRationalSurface * S2 = new QRationalSurface("3u^2+u^2v^3","u^2v^2+uv^3","2u^2v^2-v^4","1",0,1,0,1);
	lsurf << qMakePair(S1,S2);
	
	return lsurf;
}

QList< QImplicitCurve4D * > createCurves(void)
{
	QList< QImplicitCurve4D * > lcurve;
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
	QList < QPair<QRationalSurface *, QRationalSurface *> > lsurf = createSurfaces();
	for (int i = 0; i < lsurf.size(); ++i)
		lcurve << iSynaps->computeIntersectionEquation4D(lsurf[i].first, lsurf[i].second);
	return lcurve;
}

QList< QBoundingBox4D * > createBoundingBoxes(void)
{
	QList< QBoundingBox4D * > lbb;

	QBoundingBox4D * bb = new QBoundingBox4D(0,1,0,1,0,1,0,1);
	lbb << bb;
	bb = new QBoundingBox4D(0,0.5,0,0.5,0,0.5,0,0.5);
	lbb << bb;
	bb = new QBoundingBox4D(0,0.25,0,0.25,0,0.25,0,0.25);
	lbb << bb;
	bb = new QBoundingBox4D(0,0.1,0,0.1,0,0.1,0,0.1);
	lbb << bb;
	bb = new QBoundingBox4D(0,0.05,0,0.05,0,0.05,0,0.05);
	lbb << bb;
	bb = new QBoundingBox4D(0,0.005,0,0.005,0,0.005,0,0.005);
	lbb << bb;
	bb = new QBoundingBox4D(1.5,2,1.5,2,1.5,2,1.5,2);
	lbb << bb;
	return lbb;
}

//testing of QSynapsPlugin::isLocallyInjective4D

void test_isLocallyInjective4D(void)
{
	// QImplicitCurve4D * NoRoot = new QImplicitCurve4D("s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", 1,2,1,2,1,2,1,2); // NAN
	// QImplicitCurve4D * NoRoot = new QImplicitCurve4D("s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", 0,2,0,2,0,2,0,2);
	QImplicitCurve4D * NoRoot = new QImplicitCurve4D("s^2*t^2+u^2*v^2+s*t*u*v^3+v^4*s^4-0.5v+v^6", "s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", 1e-2,0.2,1e-2,0.2,1e-2,0.2,1e-2,0.2);
	// QImplicitCurve4D * NoRoot = new QImplicitCurve4D("s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2", -1,2,-1,2,-1,2,-1,2);
	// QImplicitCurve4D * NoRoot = new QImplicitCurve4D("-s^2*t^2-u^2*v^2", "s^2*t^2+u^2*v^2", "s^2*t^2+u^2*v^2",-2,-1,-2,-1,-2,-1,-2,-1);
	// QImplicitCurve4D * NoRoot = new QImplicitCurve4D("s^2*t^2+u^2*v^2-2.5", "-s^2*t^2-u^2*v^2", "-s^2*t^2-u^2*v^2", -2,-1,-2,-1,-2,-1,-2,-1);
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;

	// QList< QImplicitCurve4D * > lcurve = createCurves();
	// QList< QBoundingBox4D * > lbb = createBoundingBoxes();
	// for (int i = 0; i < lcurve.size(); ++i)
	// 	for (int j = 0; j < lbb.size(); ++j)
	// 		for (int k = 0; k < 4; ++k)
	// 		{
	// 			cout << FG_RED << "f = " << lcurve[i]->eqf() << NOCOLOR << endl;
	// 			cout << FG_RED << "g = " << lcurve[i]->eqg() << NOCOLOR << endl;
	// 			cout << FG_RED << "h = " << lcurve[i]->eqh() << NOCOLOR << endl;
	// 			cout << lbb[j]->toString().toLatin1().data() << endl;
	// 			cout << iSynaps->isLocallyInjective4D(lcurve[i], lbb[j], k) << endl; //getchar();
	// 			// if (iSynaps->isLocallyInjective4D(lcurve[i], lbb[j], k)) cout << "OK" << endl; getchar();
	// 		}
	cout << __func__ << " : " << iSynaps->isLocallyInjective4D(NoRoot, NoRoot->bb(), 0) << endl;
}


void test_isBorderMonotonic4D(void)
{
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps"));
	QList< QImplicitCurve4D * > lc = createCurves();
	for (int i = 0; i < lc.size(); ++i)
	{
		// cout << i << "/" << lc.size() << endl; getchar();
		QString f = lc[i]->f();
		QString g = lc[i]->g();
		cout << __func__ << " : " << iSynaps->isBorderMonotonic4D(f,g,"s t u v",0,createBoundingBoxes()[6]) << endl;
	}
}
