/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QP5GlovePlugin
 * 2006-12-08
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# include <QtGui>

# ifdef Q_OS_DARWIN
# include <OpenGL/glu.h>
# else
# include <GL/glu.h>
# endif

# include <QP5Glove/QP5GlovePlugin.h>

# include <math.h>

# include <p5sdk.h>
# include <estimator.h>
# ifdef Q_WS_MAC
# include <p5glove.h>
# else
# include <p5client.h>
# endif

# define POS_FILTER_SIZE 10
# define ROT_FILTER_SIZE 17

using namespace std ;

QString QP5GlovePlugin::name() const
{
	return "P5Glove" ;
}

QStringList QP5GlovePlugin::features(void) const
{
    return QStringList() ;
}

void QP5GlovePlugin::init(MainWindow * window)
{
	gl_rescale = 1.0 / 100.0f;
	use_abs_pos = 1;
#ifdef Q_WS_X11
	cli_Init ();
#else
  	glove=p5glove_open();
	if (glove == NULL) {
		fprintf(stderr, "P5GlovePlugin: Can't open glove interface\n") ;
	}
	memset(&info,0,sizeof(info));
#endif

	est_pos[0].Init (POS_FILTER_SIZE);
  	est_pos[1].Init (POS_FILTER_SIZE);
  	est_pos[2].Init (POS_FILTER_SIZE);
  	est_ypr[0].Init (ROT_FILTER_SIZE);
  	est_ypr[1].Init (ROT_FILTER_SIZE);
  	est_ypr[2].Init (ROT_FILTER_SIZE);

  	GetGloveInput();
  	GetZeroPosition();
}

void QP5GlovePlugin::idle(void)
{
	GetGloveInput() ;
}

void QP5GlovePlugin::drawIRs(void)
{
  idle() ;

	glPushMatrix() ;
	glTranslatef (pos.x, pos.y, pos.z);

  glRotatef( 135, 1.0f, 0.0f, 0.0f);
  
  glRotatef (ypr.roll, 0.0f, 1.0f, 0.0f);
  glRotatef (ypr.pitch, 1.0f, 0.0f, 0.0f);
  glRotatef (ypr.yaw, 0.0f, 0.0f, 1.0f);

	const int NPOINTS=8;
	const float pscale=0.1f;
	const float zsize=0.4f;
	int i;
	GLfloat pdata[NPOINTS][2] = 
	{
		{20,63}, {16,47}, {25,29}, {35,28}, {43,31}, {51,36}, {52,49}, {45,63}
	};

	GLfloat fingers_base[5][2] = 
	{
		{17,53}, {25,28}, {35,28}, {43,30}, {51,37}
	};
	GLfloat fingers_top[5][2] = 
	{
		{1,36}, {25,5}, {35,0}, {43,5}, {51,21}
	};
	
	// hand color
	glColor3f( 251.0f/355.0f, 156.0f/355.0f, 112.0f/355.0f);

	float xc=0.0f,yc=0.0f;
	for (i=0;i<NPOINTS;i++)
	{
		xc += pscale*pdata[i][0];
		yc += pscale*pdata[i][1];
	};
	xc /= (float)NPOINTS;
	yc /= (float)NPOINTS;

	glTranslatef(-xc, -yc, -zsize);

	glBegin(GL_QUADS);
	glVertex3f(pscale*pdata[NPOINTS-1][0],pscale*pdata[NPOINTS-1][1], zsize);
	glVertex3f(pscale*pdata[NPOINTS-1][0],pscale*pdata[NPOINTS-1][1], -zsize);
	glVertex3f(pscale*pdata[0][0],pscale*pdata[0][1], -zsize);
	glVertex3f(pscale*pdata[0][0],pscale*pdata[0][1], zsize);
	glEnd();

	
	for (i=0;i<NPOINTS-1;i++)
	{
		glBegin(GL_QUADS);
		glVertex3f(pscale*pdata[i][0],pscale*pdata[i][1], zsize);
		glVertex3f(pscale*pdata[i][0],pscale*pdata[i][1], -zsize);
		glVertex3f(pscale*pdata[i+1][0],pscale*pdata[i+1][1], -zsize);
		glVertex3f(pscale*pdata[i+1][0],pscale*pdata[i+1][1], zsize);
		glEnd();
	}

	glBegin(GL_POLYGON);
	for (i=0;i<NPOINTS;i++)
	{
		glVertex3f(pscale*pdata[i][0],pscale*pdata[i][1], zsize);
	}
	glEnd();

	glBegin(GL_POLYGON);
	for (i=NPOINTS-1;i>=0;i--)
	{
		glVertex3f(pscale*pdata[i][0],pscale*pdata[i][1], -zsize);
	}
	glEnd();

	// finger colors
	glColor3f( 251.0f/255.0f, 156.0f/255.0f, 112.0f/255.0f);

	for (i=0;i<5;i++)
	{
		#define IS_THUMB (i==0)
		GLUquadricObj *quadObj;
		quadObj = gluNewQuadric ();
		gluQuadricDrawStyle (quadObj, GLU_LINE);
		gluQuadricNormals (quadObj, GLU_SMOOTH);

		float xf = 90 + atan2(
				fingers_top[i][1] - fingers_base[i][1], 
				fingers_top[i][0] - fingers_base[i][0]) * 180.0f / 3.14159f;

		float tsize = 1.0 * pscale * sqrt(
				pow(fingers_top[i][1] - fingers_base[i][1],2) + 
				pow(fingers_top[i][0] - fingers_base[i][0],2) );


		glTranslatef(pscale*fingers_base[i][0], pscale*fingers_base[i][1], 0);

		// glutSolidTorus(1,5, 10,20);

		int nsf = 8;
		int numslices = 5;
		int numstacks = 5;
		int n;
		float radius = zsize*0.8;
		
		for (n=0; n<nsf; n++)
		{
			float rangle;
			if (IS_THUMB)
				rangle = -45 + 3 * n * finger_bend[i]/(float)nsf;
			else
				rangle = 90 - 3 * n * finger_bend[i]/(float)nsf;

			if (IS_THUMB) // thumb rotates differently
				glRotatef( +rangle, 0, 1, 0);
			else
				glRotatef( +rangle, 1, 0, 0);
			glRotatef( xf, 0, 1, 0);
			
			float partsize = tsize/(float)nsf;

			glTranslatef(0,0, n*partsize+partsize/2);

			if (IS_THUMB)
				glRotatef( +3 * finger_bend[i]/(float)nsf, 0, 1, 0);
			else
				glRotatef( -3 * finger_bend[i]/(float)nsf, 1, 0, 0);

			gluCylinder(quadObj, radius, radius, partsize/2, numslices, numstacks);

		        glTranslatef(0,0, partsize/2);
		        gluSphere(quadObj, radius, numslices, numstacks);
		        glTranslatef(0,0, -partsize/2);
			
			glRotatef( 180, 0, 1, 0);

			gluCylinder(quadObj, radius, radius, partsize/2, numslices, numstacks);

		        glTranslatef(0,0, partsize/2);
		        gluSphere(quadObj, radius, numslices, 10);
		        glTranslatef(0,0, -partsize/2);

			glRotatef( -180, 0, 1, 0);

			if (IS_THUMB)
				glRotatef( -3 * finger_bend[i]/(float)nsf, 0, 1, 0);
			else
				glRotatef( +3 * finger_bend[i]/(float)nsf, 1, 0, 0);
			
			glTranslatef(0,0, -n*partsize-partsize/2);

			glRotatef( -xf, 0, 1, 0);
			if (IS_THUMB) // thumb rotates differently
				glRotatef( -rangle, 0, 1, 0);
			else
				glRotatef( -rangle, 1, 0, 0);
		}


	        glTranslatef(-pscale*fingers_base[i][0], -pscale*fingers_base[i][1], 0);
	        gluDeleteQuadric(quadObj);
	}


/*	
	for (i=0;i<5;i++)
	{
		#define IS_THUMB (i==0)
		GLUquadricObj *quadObj;
		quadObj = gluNewQuadric ();
		gluQuadricDrawStyle (quadObj, GLU_FILL);
		gluQuadricNormals (quadObj, GLU_SMOOTH);

		float xf = 90 + atan2(
				fingers_top[i][1] - fingers_base[i][1], 
				fingers_top[i][0] - fingers_base[i][0]) * 180.0f / 3.14159f;

		float tsize = pscale * sqrt(
				pow(fingers_top[i][1] - fingers_base[i][1],2) + 
				pow(fingers_top[i][0] - fingers_base[i][0],2) );

		float rxangle;
		if (IS_THUMB)
			rxangle = -45 + 3 * finger_bend[i];
		else
			rxangle = 90 - 2.5 * finger_bend[i];

		glTranslatef(pscale*fingers_base[i][0], pscale*fingers_base[i][1], 0);
		if (IS_THUMB) // thumb rotates differently
			glRotatef( +rxangle, 0, 1, 0);
		else
			glRotatef( +rxangle, 1, 0, 0);
		glRotatef( xf, 0, 1, 0);
		
		gluCylinder(quadObj, zsize*0.8, zsize*0.4, tsize, 15, 10);
	        gluSphere(quadObj, zsize*0.8, 15, 10);
	        
	        glTranslatef(0,0, tsize);
	        gluSphere(quadObj, zsize*0.4, 15, 10);
	        glTranslatef(0,0, -tsize);

		glRotatef( -xf, 0, 1, 0);
		if (IS_THUMB) // thumb rotates differently
			glRotatef( -rxangle, 0, 1, 0);
		else
			glRotatef( -rxangle, 1, 0, 0);

	        glTranslatef(-pscale*fingers_base[i][0], -pscale*fingers_base[i][1], 0);
	        gluDeleteQuadric(quadObj);
	}
*/

	glPopMatrix() ;
}

void QP5GlovePlugin::end(void)
{
# if defined(Q_WS_X11)
	cli_Close() ;
# else
	p5glove_close(glove) ;
# endif
}

void QP5GlovePlugin::ProcessGloveInput()
{
# ifdef Q_WS_MAC
	p5glove_process_sample(glove, &info);
	/* Point Z the other way */
	int i;
	for (i=0; i < 8; i++)
	        if (info.ir[i].visible)
	                info.ir[i].z *= -1.0;
	
	gpos.x = 0.0f ;
	gpos.y = 0.0f ;
	gpos.z = 0.0f ;
	ypr.yaw = 0.0f ;
	ypr.pitch = 0.0f ;
	ypr.roll =  0.0f ;

	  finger_bend[0] = info.finger[P5GLOVE_THUMB] ;
	  finger_bend[1] = info.finger[P5GLOVE_INDEX] ;
	  finger_bend[2] = info.finger[P5GLOVE_MIDDLE] ;
	  finger_bend[3] = info.finger[P5GLOVE_RING] ;
	  finger_bend[4] = info.finger[P5GLOVE_PINKY] ;
# endif
	
  int k;
  for (k=0;k<3;k++)
  {
    est_pos[k].AddSample(gpos.v[k]);
    est_ypr[k].AddSample(ypr.v[k]);
  };
  
  pos.x = -(est_pos[0].GetEstimate() - zeropos.x) * gl_rescale;
  pos.y = +(est_pos[1].GetEstimate() - zeropos.y) * gl_rescale;
  pos.z = +(est_pos[2].GetEstimate() - zeropos.z) * gl_rescale;
  
  ypr.yaw = est_ypr[0].GetEstimate() - zerorot.yaw;
  ypr.pitch = est_ypr[1].GetEstimate() - zerorot.pitch;
  ypr.roll = -(est_ypr[2].GetEstimate() - zerorot.roll);
}

void QP5GlovePlugin::GetGloveInput()
{
# if defined(Q_WS_X11)
  cli_GetP5Data (0, &p5data);
  glove_error = p5data.m_error_estimate;
  if (use_abs_pos)
  {
    gpos = p5data.m_absolute_pos;
    ypr = p5data.m_absolute_rot;
  }
  else
  {
    gpos = p5data.m_relative_pos;
    ypr = p5data.m_relative_rot;
  }

  for (int i=0;i<5;i++)
     finger_bend[i] = p5data.m_bend[i];
# else
  int err=p5glove_sample(glove,&info) ;
# endif

  ProcessGloveInput();
}

void QP5GlovePlugin::InitPRFilters()
{
  int k;
  for (k=0;k<3;k++)
  {
    est_pos[k].ZeroAll();
    est_ypr[k].ZeroAll();
  };
}

void QP5GlovePlugin::GetZeroPosition()
{
# if defined(Q_WS_X11)
  if (use_abs_pos)
  {
    zeropos = p5data.m_absolute_pos;
    zerorot.yaw = 0.0f;
    zerorot.pitch = 0.0f;
    zerorot.roll = 0.0f;
  }
  else
  {
    zeropos = p5data.m_relative_pos;
    zerorot = p5data.m_relative_rot;
  }
# else
  zeropos.x = 0.0f ;
  zeropos.y = 0.0f ;
  zeropos.z = 0.0f ;
  zerorot.yaw = 0.0f ;
  zerorot.pitch = 0.0f ;
  zerorot.roll = 0.0f ;
# endif
}

Q_EXPORT_PLUGIN(QP5GlovePlugin)
