/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QSislInterface
 * 2006-05-13
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006-2007 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# ifndef QSISLINTERFACE_H
# define QSISLINTERFACE_H

# include <QObject>
# include <QStringList>
# include <QFile>
# include <QList>

# include <QAxel/QPluginInterface.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QBSplineSurface.h>

# include <QData/QCell.h>

using namespace QAxel ;
using namespace QData ;

/****************************************************************************/
/*!
  \class QSislInterface QSislInterface.h
  \brief The class QSislInterface provide a link between Axel and the Sisl
  library.

  More ...
*/
/****************************************************************************/
class QSislInterface : public QAxel::QPluginInterface
{
public :
    virtual ~QSislInterface() {}
    
    virtual QString name() const = 0 ;
    virtual QStringList features() const = 0 ;
    
    /*! @name Creation */
    virtual void create(QBSplineCurve * curve) = 0 ;
    virtual void create(QBSplineSurface * surface) = 0 ;
    
    /*! @name IOStream */
    virtual QBSplineCurve * readCurve(QFile * file) = 0 ;
    virtual QBSplineSurface * readSurface(QFile * file) = 0 ;
    
    /*! @name Closest points */
    virtual QPoint3f * closestPoint(QBSplineCurve * curve, QPoint3f * p) = 0 ;

    virtual QBSplineCurve   * approximateBSplineCurve(QList<QPoint3f *> points ,int, int, int,  int) = 0 ;

    virtual QBSplineSurface * approximateBSplineSurface(QList<QPoint3f *> points ,int, int, int, int,  int) = 0 ;

    /*! @name Interpolation */
    virtual QBSplineCurve * interpolate(QList<QPoint3f *> points) = 0 ;

    /*! @name Joining */
    virtual QBSplineCurve * join(QBSplineCurve * c1, QBSplineCurve * c2) = 0 ;

    /*! @name Blending */
    virtual QBSplineCurve * blend(QBSplineCurve * curve1, QBSplineCurve * curve2) = 0 ;

    /*! @name Offset */
    virtual QBSplineCurve * offset(QBSplineCurve * curve, Vec direction, double distance) = 0 ;

    /*! @name Intersection */
    virtual QList<QPoint3f *> intersection(QBSplineCurve * curve, QBSplineCurve * curve2) = 0 ;
    virtual QList<QPoint3f *> intersection(QBSplineCurve * curve, QCell * cell) = 0 ;
    virtual QList<QPoint3f *> intersection(QBSplineCurve * curve, QPair<QPoint3f *, QPoint3f *> line) = 0 ;
    virtual QList<QPoint3f *> intersection(QBSplineCurve * curve, QPlane * plane) = 0 ;

    /*! @name Bounding */
    virtual QBoundingBox * bounding(QBSplineCurve   * curve) = 0 ;
    virtual QBoundingBox * bounding(QBSplineSurface * surface) = 0 ;
} ;

Q_DECLARE_INTERFACE(QSislInterface, "Axel::Interfaces::Sisl/0.11")

# endif
