/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QSislPlugin
 * 2006-05-13
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# include <QtGui>

# include <QAxel/QPoint3f.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QSingularity.h>
# include <QAxel/QCurve.h>
# include <QAxel/QSurface.h>
# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

# include <QSisl/QSislPlugin.h>

# include <iostream>
# include <fstream>
# include <sisl.h>
# include <sislP.h>
# include <GoReadWrite.h>

using namespace std ;

QBSplineCurve * toQBSplineCurve(SISLCurve * sislcurve) 
{
    QBSplineCurve * c = new QBSplineCurve(NULL) ;
    c->number = sislcurve->in ;
    c->order =  sislcurve->ik ;
    c->knots =  sislcurve->et ;
    for(int i = 0 ; i < sislcurve->in ; i++)
	c->push_point(new QPoint3f(sislcurve->ecoef[3*i+0], sislcurve->ecoef[3*i+1], sislcurve->ecoef[3*i+2])) ;
    c->ikind =   sislcurve->ikind ;
    c->idim =    sislcurve->idim ;
    c->icopy =   sislcurve->icopy ;
    
    return c ;
}

QBSplineSurface * toQBSplineSurface(SISLSurf * sislsurface)
{
    QBSplineSurface * s = new QBSplineSurface(NULL) ;
    s->number1 = sislsurface->in1 ;
    s->number2 = sislsurface->in2 ;
    s->order1 =  sislsurface->ik1 ;
    s->order2 =  sislsurface->ik2 ;
    s->knots1 =  sislsurface->et1 ;
    s->knots2 =  sislsurface->et2 ;
    for(int i = 0 ; i < sislsurface->in1*sislsurface->in2 ; i++)
	s->push_point(new QPoint3f(sislsurface->ecoef[3*i+0], sislsurface->ecoef[3*i+1], sislsurface->ecoef[3*i+2])) ;
    s->ikind =   sislsurface->ikind ;
    s->idim =    sislsurface->idim ;
    s->icopy =   sislsurface->icopy ;
    
    return s ;
}

QString QSislPlugin::name(void) const
{
    return "Sisl" ;
}

QStringList QSislPlugin::features(void) const
{
    return QStringList() ;
}

void QSislPlugin::create(QBSplineCurve * curve)
{
    double coeff[3*curve->number] ;
    for(int i = 0 ; i < curve->points.size() ; i++) {
	QList<QPoint3f *> points = curve->points ;
	coeff[3*i+0] = points[i]->x() ;
	coeff[3*i+1] = points[i]->y() ;
	coeff[3*i+2] = points[i]->z() ;
    }
    SISLCurve * sislcurve = newCurve(curve->number, curve->order, curve->knots, coeff, curve->ikind, curve->idim, curve->icopy) ;

    curves.insert(curve, sislcurve) ;

    curve->setBoundingBox(bounding(curve)) ;
}

void QSislPlugin::create(QBSplineSurface * surface)
{
    double points[surface->points.size()*3] ;
    for(int i = 0 ; i<surface->points.size() ; i++) {
	points[3*i+0] = surface->points[i]->x() ;
	points[3*i+1] = surface->points[i]->y() ;
	points[3*i+2] = surface->points[i]->z() ;
    }
    
    SISLSurf * sislsurface = newSurf(surface->number1, surface->number2, 
				     surface->order1, surface->order2, 
				     surface->knots1, surface->knots2, 
				     points, surface->ikind, surface->idim, surface->icopy) ;

    surfaces.insert(surface, sislsurface) ;

    surface->setBoundingBox(bounding(surface)) ;
}

QBSplineCurve * QSislPlugin::readCurve(QFile * file)
{
    ifstream is(file->fileName().toLatin1().data()) ;
    SISLCurve * sislcurve = readGoCurve(is) ;    
    return toQBSplineCurve(sislcurve) ;
}

QBSplineSurface * QSislPlugin::readSurface(QFile * file)
{
    ifstream is(file->fileName().toLatin1().data()) ;
    SISLSurf * sislsurface = readGoSurface(is) ;    
    return toQBSplineSurface(sislsurface) ;
}

QPoint3f * QSislPlugin::closestPoint(QBSplineCurve * curve, QPoint3f * p)
{
    // calculating closest point                                                                                                                                                                             
    SISLCurve * c = curves[curve] ;
    double epoint[] = { p->x(), p->y(), p->z() } ;
    double aepsco = 1.0e-15 ;
    double aepsge = 1.0e-5 ;
    double dist ;
    double gpar ;
    int jstat, temp ; // status variable
    
    s1957(c,
	  epoint,
	  3,
	  aepsco,           // computational resolution                                                                                                     
	  aepsge,           // geometry resolution                                                  
	  &gpar,
	  &dist,
	  &jstat);
    
    if (jstat < 0) {
	cout << "Error occured inside call to SISL routine s1955." << endl ;
    } else if (jstat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1955. \n" << endl;
    }
    
    double coords[3] ;
    s1227(c, 			// we evaluate on the curve
          0,          	// calculate no derivatives
          gpar, 	    // parameter value on which to evaluate
          &temp,      	// not used for our purposes (gives parameter interval)
          coords, 		// result written here
          &jstat);
    
    QPoint3f * r = new QPoint3f(coords[0], coords[1], coords[2]) ;
    return r ;
}

QBSplineCurve * QSislPlugin::approximateBSplineCurve(QList<QPoint3f *> points, int iopen, int ikind, int order, int dimension)
{

    int inbpnt = points.size() ;

    cout << __func__ << " points  size " << inbpnt << endl ;

    double epoints[3*inbpnt] ;
    for(int i=0 ; i < inbpnt ; i++) {
	epoints[3*i+0] = points[i]->x() ;
	epoints[3*i+1] = points[i]->y() ;
	epoints[3*i+2] = points[i]->z() ;
    }
    double astpar = 0.0 ;
    int    idim   = dimension ;
    int    ik     = order ;
    
    SISLCurve * result = 0 ;
    int jstat ;
    
    s1630(epoints,
	  inbpnt,
	  astpar,
	  iopen,
	  idim,
	  ik,
	  &result,
	  &jstat) ;

    QBSplineCurve * c = toQBSplineCurve(result) ;
    c->ikind = ikind ;

    return c;
}

QBSplineSurface * QSislPlugin::approximateBSplineSurface(QList<QPoint3f *> points, int iopen1 , int iopen2 , int order1, int order2, int dimension)
{

    int inbput1 = 0, inbput2 = 0 ;
    double size = sqrt(points.size()) ;

//    cout << __func__ << " : value of size : " << size << endl ;

    if(size ==(int)size && size >= order1 && size >= order2) {
	inbput1 = (int)size ;
	inbput2 = (int)size ;
    }
    else {
	QMessageBox::warning(0, "Axel","Caution : Number of Points selected must be a perfect square and its square root must be >= both order1 & order2 !!","&Ok",0) ;
	return NULL ;
    }
	
    double * epoints = new double[inbput1*inbput2*3] ;
    for(int i = 0 ; i < inbput1*inbput2 ; i++){
	epoints[3*i+0] = points[i]->x() ;
	epoints[3*i+1] = points[i]->y() ;
	epoints[3*i+2] = points[i]->z() ;
    }
    int idim = dimension ;
    SISLSurf * result = 0 ;
    int jstat ;
    
    s1620(epoints,
	  inbput1,
	  inbput2,
	  2,
	  iopen1,
	  iopen2,
	  order1,
	  order2,
	  idim,
	  &result,
	  &jstat) ;

    return toQBSplineSurface(result) ;

}

QBSplineCurve * QSislPlugin::join(QBSplineCurve * c1, QBSplineCurve * c2)
{
    create(c1) ;
    create(c2) ;
    SISLCurve * sc1 = curves.value(c1) ;
    SISLCurve * sc2 = curves.value(c2) ;
    int end1 = 1 ;
    int end2 = 0 ;
    SISLCurve * newcurve ;
    int stat ;

    s1715(sc1,
	  sc2,
	  end1,
	  end2,
	  &newcurve,
	  &stat) ;

    cout << " sisl curve computed : status = " << stat << endl ; 

    return toQBSplineCurve(newcurve) ;
}

QBSplineCurve * QSislPlugin::interpolate(QList<QPoint3f *> points)
{
    int num_points = points.size() ;
    double cpoints[3*num_points] ;
    for(int i=0 ; i<num_points ; i++) {
	cpoints[3*i+0] = points[i]->x() ;
	cpoints[3*i+1] = points[i]->y() ;
	cpoints[3*i+2] = points[i]->z() ;
    }
    
    int types[num_points] ;
    for(int i=0 ; i<num_points ; i++) types[i] = 1 ;
    
    SISLCurve * result = 0 ;
    double cendpar ;
    double * gpar = 0 ;
    int jnbpar, jstat ;
    
    s1356(cpoints,		// pointer to where the coordinates are stored
	  num_points,	// number of points to be interpolated
	  3,			// input dimension
	  types,		// what type of information is stored at each point (1 means ordinary point)
	  0, 			// no additional condition at start point
	  0,			// no additional condition at end point
	  1, 			// open curve
	  4, 			// order of the spline curve to be produced
	  0, 			// parameter value to be used at start of curve
	  &cendpar,		// parameter value to be used at end of curve
	  &result, 		// the resulting spline curve
	  &gpar, 		// pointer to the parameter values of the points in the curve
	  &jnbpar, 		// number of unique parameter values
	  &jstat		// status message
	) ;
    
    return toQBSplineCurve(result) ;
}

QBSplineCurve * QSislPlugin::blend(QBSplineCurve * curve1, QBSplineCurve * curve2)
{
    SISLCurve* c1 = curves[curve1] ;
    SISLCurve* c2 = curves[curve2] ;
    
    double epsge = 1.0e-5; // geometric precision
    int blendtype = 0;     // generate polynomial segment
    int dim = 3;
    int order = 4;
    double c1_endpoint[3]; // endpoint of curve 1 (must be calculated)
    double c2_endpoint[3]; // endpoint of curve 2 (must be calculated)
    
    // The blend curve will extend from the endpoint of curve 1 to the
    // endpoint of curve 2.  As input, the SISL routine needs (approximate)
    // coordinates for these points on the curve.  We therefore preliminarly
    // need to evaluate these.  For this purpose, we use SISL routine s1227.
    
    // The end parameters of the curves' parametric domains can be found by 
    // looking at their knotvectors (pointed to by data member 'et').  
    // If the number of control points is 'n', then the knot numbered 'n'
    // would represent the end parameter (when counting from 0).  The number
    // of control points is indicated by the data member 'in'.
    double c1_endpar = c1->et[c1->in]; // end parameter of curve 1
    double c2_endpar = c2->et[c2->in]; // end parameter of curve 2
    int temp, jstat1, jstat2;
    
    // evaluating endpoint positions of both curves
    s1227(c1,          // input curve
	  0,           // evaluate position only (no derivatives)
	  c1_endpar,   // end parameter
	  &temp,       // indicates param. interval (not interesting for our purposes)
	  c1_endpoint, // this is what we want to calculate (3D position)
	  &jstat1);     // status variable (0 if everything all right)
    
    s1227(c2,          // input curve
	  0,           // evaluate position only (no derivatives)
	  c2_endpar,   // end parameter
	  &temp,       // indicates param. interval (not interesting for our purposes)
	  c2_endpoint, // this is what we want to calculate (3D position)
	  &jstat2);     // status variable (0 if everything all right)
    
    if (jstat1 < 0 || jstat2 < 0) {
	cout << "Error occured inside call to SISL routine s1227." << endl ;
    } else if (jstat1 > 0 || jstat2 > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1227." << endl ;
    }
    
    SISLCurve* blend_curve = 0;
    int jstat;
    
    s1606(c1,           // the first input curve
	  c2,           // the second input curve
	  epsge,        // geometric tolerance
	  c1_endpoint,  // endpoint of curve 1 (geometric)
	  c2_endpoint,  // endpoint of curve 2 (geometric)
	  blendtype,    // type of blend curve (circle, conic, polynomial)
	  dim,          // dimension (3D)
	  order,        // order of generated spline curve
	  &blend_curve, // the generated curve
	  &jstat);      // status message
    
    return toQBSplineCurve(blend_curve) ;
}

QBSplineCurve * QSislPlugin::offset(QBSplineCurve * curve, Vec dir, double distance)
{
    SISLCurve* c1 = curves[curve] ;
    
    double offset = double(distance);                // the offset between the 'old' curve and the result.
    double epsge = 1e-5;                        	   // geometric tolerance 
    double direction[] = {dir.x, dir.y, dir.z};      // the direction of the offset
    int dim = 3;                                     // the dimension of the Euclidean space
    int jstat;                                       // status variable
    SISLCurve* offset_curve = 0;
    
    s1360(c1,        // the 'old' curve
	  offset,        // the offset value
	  epsge,         // geometric tolerance
	  direction,     // offset direction
	  0,             // max step length.  0 indicate the longest box side of 's1'
	  dim,           // the dimension
	  &offset_curve, // the resulting offset curve
	  &jstat);       // status variable
    
    if (jstat < 0) {
	cout << "Error occured inside call to SISL routine s1360." << endl ;
    } else if (jstat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1360. \n" << endl ;
    }
    
    return toQBSplineCurve(offset_curve);
}

QList<QPoint3f *> QSislPlugin::intersection(QBSplineCurve * curve1, QBSplineCurve * curve2)
{
    SISLCurve * c1 = curves[curve1] ;
    SISLCurve * c2 = curves[curve2] ;

    double epsco = 1.0e-15;  // computational epsilon
    double epsge = 1.0e-5;   // geometric tolerance
    int num_int_points = 0;  // number of found intersection points
    double* intpar1 = 0;     // parameter values for the first curve in the intersections
    double* intpar2 = 0;     // parameter values for the second curve in the intersections
    int num_int_curves = 0;  // number of intersection curves
    SISLIntcurve** intcurve = 0; // pointer to array of detected intersection curves
    int jstat;               // status variable

    s1857(c1,              // first curve 
	  c2,              // second curve
	  epsco,           // computational resolution
	  epsge,           // geometry resolution
	  &num_int_points, // number of single intersection points
	  &intpar1,        // pointer to array of parameter values
	  &intpar2,        //               "
	  &num_int_curves, // number of detected intersection curves
	  &intcurve,       // pointer to array of detected intersection curves.
	  &jstat);
  
    if (jstat < 0) {
	cout << "Error occured inside call to SISL routine s1857." << endl ;
    } else if (jstat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1857." << endl;
    }

    // In this example, we do not expect to find intersection curves, since the
    // input curves do not overlap for more than one point at a time.
  
    cout << "Number of intersection points detected: " << num_int_points << endl;
    cout << "Number of intersection curves detected: " << num_int_curves << endl;
  
    // evaluating intersection points and writing them to file
    vector<double> point_coords_3D(3*num_int_points);
    int i;
    for (i = 0; i < num_int_points; ++i) {
	// calculating position, using curve 1
	// (we could also have used curve 2, which would give approximately
	// the same points).
	int temp;
	s1227(c1,         // we evaluate on the first curve
	      0,          // calculate no derivatives
	      intpar1[i], // parameter value on which to evaluate
	      &temp,      // not used for our purposes (gives parameter interval)
	      &point_coords_3D[3*i], // result written here
	      &jstat);
    
	if (jstat < 0) {
	    cout << "Error occured inside call to SISL routine s1227." << endl ;
	} else if (jstat > 0) {
	    cout << "WARNING: warning occured inside call to SISL routine s1227. \n" << endl;
	}
    }
  
    QList<QPoint3f *> intersections ;
    for(int i=0 ; i < num_int_points ; i++)
	intersections << new QPoint3f(point_coords_3D[3*i+0],point_coords_3D[3*i+1], point_coords_3D[3*i+2]) ;

    return intersections ;
}

QList<QPoint3f *> QSislPlugin::intersection(QBSplineCurve * curve, QCell * cell)
{
    QPoint3f * tlCorner = new QPoint3f(cell->xmin(), cell->ymax(), 0.0) ;
    QPoint3f * trCorner = new QPoint3f(cell->xmax(), cell->ymax(), 0.0) ;
    QPoint3f * blCorner = new QPoint3f(cell->xmin(), cell->ymin(), 0.0) ;
    QPoint3f * brCorner = new QPoint3f(cell->xmax(), cell->ymin(), 0.0) ;

    QList<QPoint3f *> l ;
    l << intersection(curve, qMakePair(tlCorner, trCorner)) ;
    l << intersection(curve, qMakePair(trCorner, brCorner)) ;
    l << intersection(curve, qMakePair(brCorner, blCorner)) ;
    l << intersection(curve, qMakePair(blCorner, tlCorner)) ;
    return l ;
}

QList<QPoint3f *> QSislPlugin::intersection(QBSplineCurve * curve, QPair<QPoint3f *, QPoint3f *> line)
{
    QPoint3f * a = line.first  ;
    QPoint3f * b = line.second ;

    SISLCurve * sislcurve = curves[curve] ;
    double   point[2] = { a->x(), a->y() } ;
    double  normal[2] = { -b->y()+a->y(), b->x()-a->x() } ;
    int       dim     = 2 ;
    double  epsco     = 1e-3 ; // Computational resoluation, not used by sisl
    double  epsge     = 1e-6 ; // Geometry resolution

    int             numintpts ; // number of single intersection points
    double *           intpar ; // values of single intersection points
    int              numintcu ; // number of intersection curves
    SISLIntcurve ** intcurves ; // values of intersection curves
    int                  stat ;

    s1850(sislcurve,
	  point,
	  normal,
	  dim,
	  epsco,
	  epsge,
	  &numintpts,
	  &intpar,
	  &numintcu,
	  &intcurves,
	  &stat) ;

    if (stat < 0) {
	cout << "Error occured inside call to SISL routine s1850." << endl ;
    } else if (stat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1850. \n" << endl;
    }

    // evaluating intersection points
    vector<double> point_coords_3D(3*numintpts) ;
    for (int i = 0; i < numintpts; ++i) {
       	int temp;
	s1227(sislcurve,
	      0,          // calculate no derivatives
	      intpar[i],  // parameter value on which to evaluate
	      &temp,      // not used for our purposes (gives parameter interval)
	      &point_coords_3D[3*i], // result written here
	      &stat) ;
    
	if (stat < 0) {
	    cout << "Error occured inside call to SISL routine s1227." << endl ;
	} else if (stat > 0) {
	    cout << "WARNING: warning occured inside call to SISL routine s1227. \n" << endl;
	}
    }
  
    QList<QPoint3f *> intersections ;
    for(int i=0 ; i < numintpts ; i++)
	intersections << new QPoint3f(point_coords_3D[3*i+0],point_coords_3D[3*i+1], point_coords_3D[3*i+2]) ;
    
    return intersections ;
}

QList<QPoint3f *> QSislPlugin::intersection(QBSplineCurve * curve, QPlane * plane)
{
    cout << __func__ << " WILL NOT WORK, FIND A POINT INCIDENT TO THE PLANE" << endl ;
    SISLCurve * sislcurve = curves[curve] ;
    double   point[3] = { 0.0, 0.0, 0.0 } ;
    double  normal[3] = { plane->a(), plane->b(), plane->c() } ;
    int       dim     = 3 ;
    double  epsco     = 1e-3 ; // Computational resoluation, not used by sisl
    double  epsge     = 1e-6 ; // Geometry resolution

    int             numintpts ; // number of single intersection points
    double *           intpar ; // values of single intersection points
    int              numintcu ; // number of intersection curves
    SISLIntcurve ** intcurves ; // values of intersection curves
    int                  stat ;

    s1850(sislcurve,
	  point,
	  normal,
	  dim,
	  epsco,
	  epsge,
	  &numintpts,
	  &intpar,
	  &numintcu,
	  &intcurves,
	  &stat) ;

    if (stat < 0) {
	cout << "Error occured inside call to SISL routine s1850." << endl ;
    } else if (stat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1850. \n" << endl;
    }

    // evaluating intersection points
    vector<double> point_coords_3D(3*numintpts) ;
    for (int i = 0; i < numintpts; ++i) {
       	int temp;
	s1227(sislcurve,
	      0,          // calculate no derivatives
	      intpar[i],  // parameter value on which to evaluate
	      &temp,      // not used for our purposes (gives parameter interval)
	      &point_coords_3D[3*i], // result written here
	      &stat) ;
    
	if (stat < 0) {
	    cout << "Error occured inside call to SISL routine s1227." << endl ;
	} else if (stat > 0) {
	    cout << "WARNING: warning occured inside call to SISL routine s1227. \n" << endl;
	}
    }
  
    QList<QPoint3f *> intersections ;
    for(int i=0 ; i < numintpts ; i++)
	intersections << new QPoint3f(point_coords_3D[3*i+0],point_coords_3D[3*i+1], point_coords_3D[3*i+2]) ;
    
    return intersections ;
}

QBoundingBox * QSislPlugin::bounding(QBSplineCurve * curve)
{
    SISLCurve * sislcurve = curves[curve] ;
    double    * emin      = NULL ; // Bottom-left corner of the box
    double    * emax      = NULL ; // Top-right   corner of the box
    int         jstat     = 0 ;

    s1988(sislcurve,
	  &emax,
	  &emin,
	  &jstat) ;

    if (jstat < 0) {
	cout << "Error occured inside call to SISL routine s1988: Computing the bounding box of a curve." << endl ;
    } else if (jstat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1988: Computing the bounding box of a curve." << endl;
    }
    
    return new QBoundingBox(emin[0], emax[0], emin[1], emax[1]) ;
}

QBoundingBox * QSislPlugin::bounding(QBSplineSurface * surface)
{
    SISLSurf * sislsurface = surfaces[surface] ;
    double   * emin        = NULL ; // Bottom-left corner of the box
    double   * emax        = NULL ; // Top-right   corner of the box
    int        jstat       = 0 ;

    s1989(sislsurface,
	  &emax,
	  &emin,
	  &jstat) ;

    if (jstat < 0) {
	cout << "Error occured inside call to SISL routine s1989: Computing the bounding box of a surface." << endl ;
    } else if (jstat > 0) {
	cout << "WARNING: warning occured inside call to SISL routine s1989: Computing the bounding box of a surface." << endl;
    }
    
    return new QBoundingBox(emin[0], emax[0], emin[1], emax[1], emin[2], emax[2]) ;
}

Q_EXPORT_PLUGIN(QSislPlugin)
