/*****************************************************************************
 * A x e l
 *****************************************************************************
 * IStreamGcml
 * 2006-07-03
 * Arnaud Fabre
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *               Copyright (C) 2006 LSIIT - UMR CNRS-ULP - Strasbourg
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <iostream>

# include <QFile>
# include <QLabel>
# include <QStringList>
# include <QProgressDialog>
# include <QApplication>
# include <QMessageBox>

# include <QDomDocument>
# include <QDomNodeList>
# include <QDomElement>

# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QConstraintDistance.h>
# include <QAxel/QConstraintIncidence.h>

# include <QGui/Viewer.h>
# include <QStream/IStream/IStreamGcml.h>

using namespace std ;
using namespace qglviewer ;

namespace QStream {

IStreamGcml::IStreamGcml(QFile* file, QString s, Viewer * v) 
{
    filename = s ;
    viewer = v ;
    
    domdocument = new QDomDocument("gcml") ;
    if( !domdocument->setContent(file)) {
	QMessageBox::information(0, "Axel", filename + "is not a valid Gcml file","&Ok",0) ;
	return ;
    }
    
    file->close() ;
}
    
void IStreamGcml::run(void)
{
    getUnknowns();
    getConstraints();
    getSketch();
    getValuation();
    loadObjects();
    loadConstraints();
}

void IStreamGcml::getUnknowns()
{
    QDomNodeList nodelist= domdocument->elementsByTagName("unknowns");
    for(unsigned int i=0 ; i<nodelist.length(); i++)
    {
	QDomElement element = nodelist.item(i).toElement();
	_unknowns = element.text();
    }
// 	std::cerr << _unknowns.toStdString() << std::endl;
}

void IStreamGcml::getConstraints()
{
    QDomNodeList nodelist= domdocument->elementsByTagName("constraints");
    for(unsigned int i=0 ; i<nodelist.length(); i++)
    {
	QDomElement element = nodelist.item(i).toElement();
	_constraints = element.text();
    }
// 	std::cerr << _constraints.toStdString() << std::endl;
}

void IStreamGcml::getSketch()
{
    QDomNodeList nodelist= domdocument->elementsByTagName("sketch");
    for(unsigned int i=0 ; i<nodelist.length(); i++)
    {
	QDomElement element = nodelist.item(i).toElement();
	_sketch = element.text();
    }
// 	std::cerr << _sketch.toStdString() << std::endl;
}

void IStreamGcml::getValuation()
{
    QDomNodeList nodelist= domdocument->elementsByTagName("valuation");
    for(unsigned int i=0 ; i<nodelist.length(); i++)
    {
	QDomElement element = nodelist.item(i).toElement();
	_valuation = element.text();
    }
// 	std::cerr << _valuation.toStdString() << std::endl;
}

void IStreamGcml::loadObjects()
{
    QRegExp regexp("\\s");
    QStringList list = _unknowns.split(regexp, QString::SkipEmptyParts);
    
    int objectType=0;
    for(QStringList::Iterator it = list.begin(); it != list.end(); it++ ) 
    {
// 		std::cerr << (*it).toStdString() << std::endl;
	if((*it)=="point")
	    objectType = 1;
	else
	    if((*it)=="plane")
		objectType = 2;
	    else
		if(objectType)
		{
		    switch(objectType)
		    {
		    case 1:
		    {
			int j=0, k;
			j = _sketch.indexOf((*it), j);
			k = _sketch.indexOf("\n", j);
			QString s = _sketch.mid(j,k-j);
// 					std::cerr << s.toStdString() << std::endl;
			
			j = s.indexOf("=", 0);
			QString name = s.mid(0,j);
			
			j = s.indexOf("(", 0);
			k = s.indexOf(",", j);
			QString xs = s.mid(j+1,k-j-1);
// 					std::cerr << xs.toStdString() << std::endl;
			
			j = s.indexOf(",", k+1);
			QString ys = s.mid(k+1,j-k-1);
// 					std::cerr << ys.toStdString() << std::endl;
			
			k = s.indexOf(")", j+1);
			QString zs = s.mid(j+1,k-j-1);
// 					std::cerr << zs.toStdString() << std::endl;
			
			float x = xs.toFloat();
			float y = ys.toFloat();
			float z = zs.toFloat();
			
// 					std::cerr << name.toStdString() << "added !" << std::endl;
			
			QPoint3f * p = new QPoint3f(x,y,z);
			p->setProperty("name",name);
			viewer->addObject(p);
			break;
		    }
		    case 2:
		    {
			int j=0, k;
			j = _sketch.indexOf((*it), j);
			k = _sketch.indexOf("\n", j);
			QString s = _sketch.mid(j,k-j);
// 					std::cerr << s.toStdString() << std::endl;
			
			j = s.indexOf("=", 0);
			QString name = s.mid(0,j);
					
			j = s.indexOf("(", 0);
			k = s.indexOf(",", j);
			QString as = s.mid(j+1,k-j-1);
					
			j = s.indexOf(",", k+1);
			QString bs = s.mid(k+1,j-k-1);
					
			k = s.indexOf(",", j+1);
			QString cs = s.mid(j+1,k-j-1);
					
			j = s.indexOf(")", k+1);
			QString ds = s.mid(k+1,j-k-1);
			// 					
			float a = as.toFloat();
			float b = bs.toFloat();
			float c = cs.toFloat();
			float d = ds.toFloat();
					
// 					std::cerr << a << " " << b << " " << c << " " << d << std::endl;
			QPlane * p = new QPlane(a,b,c,d);
			p->setProperty("name",name);
			viewer->addObject(p);
			break;
		    }
		    }
		}
    }
}

void IStreamGcml::loadConstraints()
{
    QRegExp regexp("\\s");
    QStringList list = _constraints.split(regexp, QString::SkipEmptyParts);
	
    for(QStringList::Iterator it = list.begin(); it != list.end(); it++ ) 
    {
	QString s=(*it);
	int i = s.indexOf("distpp", 0);
	if(i==0)
	{
// 			std::cerr << (*it).toStdString() << std::endl;
	    int j=0, k;

	    j = s.indexOf("(", 0);
	    k = s.indexOf(",", j);
	    QString p1 = s.mid(j+1,k-j-1);
// 			std::cerr << p1.toStdString() << std::endl;
			
	    j = s.indexOf(",", k+1);
	    QString p2 = s.mid(k+1,j-k-1);
// 			std::cerr << p2.toStdString() << std::endl;
			
	    k = s.indexOf(")", j+1);
	    QString name = s.mid(j+1,k-j-1);
// 			std::cerr << name.toStdString() << std::endl;
			
	    j = _valuation.indexOf(name, 0);
	    k = _valuation.indexOf("\n", j);
	    QString val = _valuation.mid(j,k);
	    j = val.indexOf("(",0);
	    k = val.indexOf(")", j);
	    QString d=val.mid(j+1,k-j-1);
	    float distance = d.toFloat();
			
	    QAxelObject *A=viewer->getObjectFromName(p1);
	    QAxelObject *B=viewer->getObjectFromName(p2);
	    if (A && B)
	    {
		QConstraintDistance * dc = new QConstraintDistance(A, B, distance);
		dc->setProperty("name",name);

		viewer->addObject(dc);
	    }
	    else
	    {
		std::cerr << "Not found :" << std::endl;
		std::cerr << A << std::endl;
		std::cerr << B << std::endl;
	    }
	}
	i = s.indexOf("onP", 0);
	if(i==0)
	{
// 			std::cerr << (*it).toStdString() << std::endl;
	    int j=0, k;

	    j = s.indexOf("(", 0);
	    k = s.indexOf(",", j);
	    QString p = s.mid(j+1,k-j-1);
// 			std::cerr << p.toStdString() << std::endl;
			
	    j = s.indexOf(")", k+1);
	    QString pl = s.mid(k+1,j-k-1);
// 			std::cerr << pl.toStdString() << std::endl;
						
	    QConstraintIncidence * ic = new QConstraintIncidence(viewer->getObjectFromName(p), viewer->getObjectFromName(pl));

	    viewer->addObject(ic);
	}
    }
}

} ;
