/*****************************************************************************
 * A x e l
 *****************************************************************************
 * IStreamVect
 * 2006-03-24
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/
                     
# include <iostream>
                 
# include <QFile>
# include <QStringList>                 
# include <QProgressDialog>
# include <QApplication>

# include <QAxel/QVertex.h>               
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

# include <QStream/IStream/IStreamVect.h>

using namespace std ;

namespace QStream {

IStreamVect::IStreamVect(QFile * f) : IStream(f) 
{
    nVertices = nEdges = nColors = 0 ;
    currentVertex = 0 ; currentEdge = 0 ; currentColor = 0 ;
}

QMesh * IStreamVect::run(void) 
{ 
    QMesh * m = new QMesh ;     
    m->setMeshType(QMesh::Curve) ;
    int step = 0 ;
    while(!atEnd()) {                                     
	QString line = readLine().simplified() ;
	if(line.startsWith("#")) {
	    continue ;
	} else if(line.isEmpty()) {
	    continue ;
	} else if(step==0) {
	    if (line.startsWith("VECT")) {
		step++ ;                                                 
	    }                    
	} else if(step==1) {  
	    QStringList numbers = line.split(QRegExp("\\s")) ;
	    nEdges = numbers.takeFirst().toInt() ;
	    nVertices = numbers.takeFirst().toInt() ;
	    if(numbers.size()>0) nColors = numbers.takeFirst().toInt() ;  
	    
	    m->setNumberOfVertices(nVertices) ;
	    m->setNumberOfEdges(nEdges) ;
	    
	    progress = new QProgressDialog("Importing Vect File Format", 0, 0, nEdges) ;
	    step++ ;
	} else if(step==2) {                 
	    step++ ;
	} else if(step==3) {              
	    step++ ;
	} else if(step == 4) {
	    readEdge(line, m) ;
	    currentEdge++ ;
	    if(currentEdge == nEdges) step++ ;
	} else if(step == 5) {
	    readColor(line, m) ;
	    if(currentColor == nColors) step++ ;
	}
    }           
    
    return m ;
}

void IStreamVect::readEdge(QString line, QMesh * m) 
{  
    QStringList numbers = line.split(QRegExp("\\s")) ;
    
    int index1 = currentVertex ;
    float x1 = numbers.takeFirst().toFloat() ;
    float y1 = numbers.takeFirst().toFloat() ;
    float z1 = numbers.takeFirst().toFloat() ;                                   
    QVertex * v1 = new QVertex(index1,x1,y1,z1,m) ;        
    currentVertex++ ;
    m->push_vertex(v1) ;
    
    int index2 = currentVertex ;
    float x2 = numbers.takeFirst().toFloat() ;
    float y2 = numbers.takeFirst().toFloat() ;
    float z2 = numbers.takeFirst().toFloat() ;                                   
    QVertex * v2 = new QVertex(index2,x2,y2,z2,m) ;        
    currentVertex++ ;
    m->push_vertex(v2) ;
    
    QEdge * e = new QEdge(index1, index2, m) ;
    m->push_edge(e) ;
}

void IStreamVect::readColor(QString line, QMesh * m) 
{  
    QStringList numbers = line.split(QRegExp("\\s")) ;
    
    int r = (int)(numbers.takeFirst().toFloat()*255) ;
    int g = (int)(numbers.takeFirst().toFloat()*255) ;
    int b = (int)(numbers.takeFirst().toFloat()*255) ;
    int a= (int)(numbers.takeFirst().toFloat()*255) ;
    
    m->edges_[currentColor]->setRGBA(r,g,b,a) ;
    
    currentColor++ ;
}

} ;
