/*****************************************************************************
 * A x e l
 *****************************************************************************
 * OStreamAxl
 * 2006-07-04
 * Ashish Reddy
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <iostream>

# include <QFile>
# include <QLabel>
# include <QStringList>
# include <QProgressDialog>
# include <QApplication>
# include <QMessageBox>
# include <QTextStream>
# include <QDomDocument>
# include <QDomNodeList>
# include <QDomElement>

# include <QAxel/QPoint3f.h>
# include <QAxel/QPoint4f.h>
# include <QAxel/QPointSet.h>
# include <QAxel/QPlane.h>
# include <QAxel/QCube.h>
# include <QAxel/QSphere.h>
# include <QAxel/QCylinder.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QMesh.h>
# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>

# include <QGui/Viewer.h>
# include <QGui/color.h>
# include <QGui/MainWindow.h>
# include <QGui/ScriptingDock.h>

# include <QStream/OStream/OStreamAxl.h>

using namespace std ;
using namespace qglviewer ;

namespace QStream {

OStreamAxl::OStreamAxl(QString s, MainWindow * mw) 
{
    filename = s ;
    mainwindow = mw ;
    viewer = mw->viewer() ;
    
    domdocument = new QDomDocument() ;
}

void OStreamAxl::run(void)
{
    QList<QAxelObject *> objects = viewer->listObjects() ;
    
    cout << objects.size() << " objects" << endl ;

    foreach(QAxelObject * o, objects) {
        
        if(QPoint3f * p = dynamic_cast<QPoint3f *>(o))
            createPoint3fElement(p) ;
        if(QPoint4f * p = dynamic_cast<QPoint4f *>(o))
            createPoint4fElement(p) ;
	if(QPlane * p = dynamic_cast<QPlane *>(o))
            createPlaneElement(p) ;
        if(QPointSet * p = dynamic_cast<QPointSet *>(o))
            createPointSetElement(p) ;
	if(QCube * p = dynamic_cast<QCube *>(o))
            createCubeElement(p) ;
        if(QSphere * p = dynamic_cast<QSphere *>(o))
            createSphereElement(p) ;
        if(QCylinder * p = dynamic_cast<QCylinder *>(o))
            createCylinderElement(p) ;
        if(QCurve * p = dynamic_cast<QCurve *>(o))
            createCurveElement(p) ;
        if(QSurface * p = dynamic_cast<QSurface *>(o))
            createSurfaceElement(p) ;
	if(QMesh * p = dynamic_cast<QMesh *>(o))
            createMeshElement(p) ;

	createScriptElement() ;
        createViewerElement() ;
    }
    
    cout << __func__ << "about to build dom tree" << endl ;

    buildDomTree();
    save();
}

void OStreamAxl::createPoint3fElement(QPoint3f* p)
{
    QDomElement point3f = domdocument->createElement("point");

    point3f.setAttribute("name",p->name());

    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " 
			<< QString::number(g) << " " 
			<< QString::number(b) ;
    point3f.setAttribute("color", color);
    
    float x=p->x(),y=p->y(),z=p->z();
    QString location ;
    QTextStream(&location) << QString::number(x) << " " 
			   << QString::number(y) << " " 
			   << QString::number(z) ;
   
    QDomText text = domdocument->createTextNode(location) ;
    point3f.appendChild(text) ;

    elements.append(point3f);
}


void OStreamAxl::createPoint4fElement(QPoint4f* p)
{
    Q_UNUSED(p) ;
}

void OStreamAxl::createPlaneElement(QPlane* p)
{
    QDomElement plane = domdocument->createElement("plane");
    plane.setAttribute("name",p->name());
    
    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " 
			<< QString::number(g) << " " 
			<< QString::number(b) ;
    plane.setAttribute("color", color);

    QDomElement size = domdocument->createElement("size");
    QString sizeStr ;
    QTextStream(&sizeStr) << QString::number(p->size()) ;
    QDomText sizeVal = domdocument->createTextNode(sizeStr) ;
    size.appendChild(sizeVal) ;

    plane.appendChild(size);

    QDomElement polynomial = domdocument->createElement("polynomial");
    polynomial.setAttribute("variables","x y z");
    QString polyStr = p->equation();
    
    cout << FG_YELLOW << polyStr.toAscii().constData() << NOCOLOR << endl ;
    
    QDomText polyVal = domdocument->createTextNode(polyStr);
    polynomial.appendChild(polyVal);
    
    plane.appendChild(polynomial) ;
    
   
    elements.append(plane);
    
    
}


void OStreamAxl::createPointSetElement(QPointSet * p)
{
    QDomElement pointSet = domdocument->createElement("pointset");
    
    pointSet.setAttribute("name",p->name());
    pointSet.setAttribute("size",p->size());
    pointSet.setAttribute("color","rgb");
    
    int count=0;
    QString pointListStr;
    
    QTextStream(&pointListStr) << "\n\t\t" ;

    while(count!=p->size()){

        float x = *(p->vertices() + 3*count + 0) , 
	      y = *(p->vertices() + 3*count + 1) , 
	      z = *(p->vertices() + 3*count + 2) ;
	
        int r = static_cast<int>(*(p->colors()+3*count+0) * 255) , 
	    g = static_cast<int>(*(p->colors()+3*count+1) * 255) ,
	    b = static_cast<int>(*(p->colors()+3*count+2) * 255);
	
        QTextStream(&pointListStr) << QString::number(x) << " " 
				   << QString::number(y) << " " 
				   << QString::number(z) << " "
				   << QString::number(r) << " " 
				   << QString::number(g) << " " 
				   << QString::number(b) << "\n\t\t" ;
	cout << FG_RED 
	     << x << " " << y << " " << z << " "
	     << r << " " << g << " " << b << "\n"
	     << NOCOLOR ;

	count++;


    }
    QDomText pointSetText = domdocument->createTextNode(pointListStr);
    pointSet.appendChild(pointSetText);

    elements.append(pointSet);
}

void OStreamAxl::createCubeElement(QCube* p)
{
    QDomElement cube = domdocument->createElement("cube");

    cube.setAttribute("name",p->name());

    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " 
			<< QString::number(g) << " " 
			<< QString::number(b) ;

    cube.setAttribute("color", color);

    QDomElement center = domdocument->createElement("center");
    
    float x=p->x(),y=p->y(),z=p->z();
    QString location ;
    QTextStream(&location) << QString::number(x) << " " 
			   << QString::number(y) << " " 
			   << QString::number(z) ;

    QDomText locationVal = domdocument->createTextNode(location) ;
    center.appendChild(locationVal) ;
    
    cube.appendChild(center);


    QDomElement size = domdocument->createElement("size");
    QString sizeStr ;
    QTextStream(&sizeStr) << QString::number(p->size()) ;
    QDomText sizeVal = domdocument->createTextNode(sizeStr) ;
    size.appendChild(sizeVal) ;

    cube.appendChild(size);

    elements.append(cube);
    
}

void OStreamAxl::createSphereElement(QSphere* p)
{
    QDomElement sphere = domdocument->createElement("sphere");

    sphere.setAttribute("name",p->name());

    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " 
			<< QString::number(g) << " " 
			<< QString::number(b) ;
    
    sphere.setAttribute("color", color);

    QDomElement center = domdocument->createElement("center");
    
    float x=p->x(),y=p->y(),z=p->z();
    QString location ;
    QTextStream(&location) << QString::number(x) << " " 
			   << QString::number(y) << " " 
			   << QString::number(z) ;
    
    QDomText locationVal = domdocument->createTextNode(location) ;
    center.appendChild(locationVal) ;
    
    sphere.appendChild(center);


    QDomElement radius = domdocument->createElement("radius");
    QString radiusStr ;
    QTextStream(&radiusStr) << QString::number(p->s()) ;
    QDomText radiusVal = domdocument->createTextNode(radiusStr) ;
    radius.appendChild(radiusVal) ;

    sphere.appendChild(radius);

    elements.append(sphere);

}

void OStreamAxl::createCylinderElement(QCylinder* p)
{

    QDomElement cylinder = domdocument->createElement("cylinder");

    cylinder.setAttribute("name",p->name());

    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " 
			<< QString::number(g) << " " 
			<< QString::number(b) ;

    cylinder.setAttribute("color", color);

    QDomElement from = domdocument->createElement("from");
    
    float x=p->from()->x(),y=p->from()->y(),z=p->from()->z();
    QString fromStr ;
    QTextStream(&fromStr) << QString::number(x) << " " 
			  << QString::number(y) << " " 
			  << QString::number(z) ;
    
    QDomText fromVal = domdocument->createTextNode(fromStr) ;
    from.appendChild(fromVal) ;
    
    cylinder.appendChild(from);
    
    QDomElement to = domdocument->createElement("to");
    
    x=p->to()->x(),y=p->to()->y(),z=p->to()->z();
    QString toStr ;
    QTextStream(&toStr) << QString::number(x) << " " 
			<< QString::number(y) << " " 
			<< QString::number(z) ;

    QDomText toVal = domdocument->createTextNode(toStr) ;
    to.appendChild(toVal) ;
    
    cylinder.appendChild(to);


    QDomElement radius = domdocument->createElement("radius");
    QString radiusStr ;
    QTextStream(&radiusStr) << QString::number(p->s()) ;
    QDomText radiusVal = domdocument->createTextNode(radiusStr) ;
    radius.appendChild(radiusVal) ;

    cylinder.appendChild(radius);

    elements.append(cylinder);

}

void OStreamAxl::createCurveElement(QCurve* p)
{
    QDomElement curve = domdocument->createElement("curve");
    
    curve.setAttribute("name",p->name());
    
    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " << QString::number(g) << " " << QString::number(b) ;
    curve.setAttribute("color", color);

    if(QImplicitCurve2D * impCurve2D = dynamic_cast<QImplicitCurve2D *>(p)){
        curve.setAttribute("type","implicit");
        
	QDomElement domain = domdocument->createElement("domain");
	QString domainStr;
	QTextStream(&domainStr) << QString::number(impCurve2D->boundingbox()->xmin()) << " " 
				<< QString::number(impCurve2D->boundingbox()->xmax()) << " " 
				<< QString::number(impCurve2D->boundingbox()->ymin()) << " " 
				<< QString::number(impCurve2D->boundingbox()->ymax()) ;

	QDomText domainVal = domdocument->createTextNode(domainStr);
	domain.appendChild(domainVal);

	curve.appendChild(domain);

	QDomElement polynomial = domdocument->createElement("polynomial");
	polynomial.setAttribute("variables","x y");
	QString polyStr = impCurve2D->equation();
	
	cout << FG_YELLOW << polyStr.toAscii().constData() << NOCOLOR << endl ;
	
	QDomText polyVal = domdocument->createTextNode(polyStr);
	polynomial.appendChild(polyVal);

	curve.appendChild(polynomial) ;
    }

    if(QImplicitCurve3D * impCurve3D = dynamic_cast<QImplicitCurve3D *>(p)){
	curve.setAttribute("type","implicit");
	
	QDomElement domain = domdocument->createElement("domain");
	QString domainStr;
	QTextStream(&domainStr) << QString::number(impCurve3D->boundingbox()->xmin()) << " " 
				<< QString::number(impCurve3D->boundingbox()->xmax()) << " " 
				<< QString::number(impCurve3D->boundingbox()->ymin()) << " " 
				<< QString::number(impCurve3D->boundingbox()->ymax()) << " "
				<< QString::number(impCurve3D->boundingbox()->zmin()) << " " 
				<< QString::number(impCurve3D->boundingbox()->zmax()) ; 
	QDomText domainVal = domdocument->createTextNode(domainStr);
	domain.appendChild(domainVal);
	
	curve.appendChild(domain);
	
	QDomElement polynomial1 = domdocument->createElement("polynomial");
	polynomial1.setAttribute("variables","x y z");
	QString polyStr1 = impCurve3D->equation1();
	QDomText polyVal1 = domdocument->createTextNode(polyStr1);
	polynomial1.appendChild(polyVal1);
	
	curve.appendChild(polynomial1);

	QDomElement polynomial2 = domdocument->createElement("polynomial");
	polynomial2.setAttribute("variables","x y z");
	QString polyStr2 = impCurve3D->equation2();
	QDomText polyVal2 = domdocument->createTextNode(polyStr2);
	polynomial2.appendChild(polyVal2);
	
	curve.appendChild(polynomial2);

	elements.append(curve);
    }

    if(QImplicitCurve4D * impCurve4D = dynamic_cast<QImplicitCurve4D *>(p)){

	Q_UNUSED(impCurve4D) ;
	
    }

    if(QRationalCurve * rationalCurve = dynamic_cast<QRationalCurve *>(p)){
    
	curve.setAttribute("type","rational");
	
	QDomElement domain = domdocument->createElement("domain");
	QString domainStr;
	QTextStream(&domainStr) << QString::number(rationalCurve->tmin()) << " " 
				<< QString::number(rationalCurve->tmax()); 
	QDomText domainVal = domdocument->createTextNode(domainStr);
	domain.appendChild(domainVal);

	curve.appendChild(domain);

	QDomElement polynomial1 = domdocument->createElement("polynomial");
	polynomial1.setAttribute("variables","t");
	QString polyStr1 = rationalCurve->px();
	QDomText polyVal1 = domdocument->createTextNode(polyStr1);
	polynomial1.appendChild(polyVal1);

	curve.appendChild(polynomial1);

	QDomElement polynomial2 = domdocument->createElement("polynomial");
	polynomial2.setAttribute("variables","t");
	QString polyStr2 = rationalCurve->py();
	QDomText polyVal2 = domdocument->createTextNode(polyStr2);
	polynomial2.appendChild(polyVal2);

	curve.appendChild(polynomial2);

	QDomElement polynomial3 = domdocument->createElement("polynomial");
	polynomial3.setAttribute("variables","t");
	QString polyStr3 = rationalCurve->pz();
	QDomText polyVal3 = domdocument->createTextNode(polyStr3);
	polynomial3.appendChild(polyVal3);

	curve.appendChild(polynomial3);

	QDomElement polynomial4 = domdocument->createElement("polynomial");
	polynomial4.setAttribute("variables","t");
	QString polyStr4 = rationalCurve->pw();
	QDomText polyVal4 = domdocument->createTextNode(polyStr4);
	polynomial4.appendChild(polyVal4);

	curve.appendChild(polynomial4);


    }

     if(QBSplineCurve * bspCurve = dynamic_cast<QBSplineCurve *>(p)){

	 curve.setAttribute("type","bspline");
	 
	 QDomElement dimension = domdocument->createElement("dimension");
	 QString dimensionStr;
	 QTextStream(&dimensionStr) << QString::number(bspCurve->getDimension());
	 QDomText dimensionVal = domdocument->createTextNode(dimensionStr);
	 dimension.appendChild(dimensionVal);
	 
	 curve.appendChild(dimension);

	 QDomElement number = domdocument->createElement("number");
	 QString numberStr;
	 QTextStream(&numberStr) << QString::number(bspCurve->getNumber()); 
	 QDomText numberVal = domdocument->createTextNode(numberStr);
	 number.appendChild(numberVal);
	 
	 curve.appendChild(number);

	 QDomElement order = domdocument->createElement("order");
	 QString orderStr;
	 QTextStream(&orderStr) << QString::number(bspCurve->getOrder());
	 QDomText orderVal = domdocument->createTextNode(orderStr);
	 order.appendChild(orderVal);
	 
	 curve.appendChild(order);

	 QDomElement knots = domdocument->createElement("knots");
	 QString knotsStr;
	 double * curveKnots = bspCurve->getKnots();
	 if(curveKnots != NULL)
	     cout << "curveKnots is not NULL" << endl;
	 int i=0;
	 while (i < bspCurve->getNumber() + bspCurve->getOrder()){
	     QTextStream(&knotsStr) << QString::number(*(curveKnots + i)) << " ";
	     i++;
	 }
	 QDomText knotsVal = domdocument->createTextNode(knotsStr);
	 knots.appendChild(knotsVal);
	 
	 curve.appendChild(knots);

	 QDomElement points = domdocument->createElement("points");
	 QString pointsStr;
	 QList<QPoint3f *> curvePoints = bspCurve->points ;
	 
	 for(QList<QPoint3f *>::iterator itl = curvePoints.begin();itl != curvePoints.end();itl++){
	     QTextStream(&pointsStr) << QString::number((*itl)->x()) << " " 
				     << QString::number((*itl)->y()) << " " 
				     << QString::number((*itl)->z()) 
				     << (itl + 1 == curvePoints.end() ? "" : "\n\t") ;
	 }

	 QDomText pointsVal = domdocument->createTextNode(pointsStr);
	 points.appendChild(pointsVal);
	 
	 curve.appendChild(points);
	 
     }
        
    
    elements.append(curve);

}

void OStreamAxl::createSurfaceElement(QSurface* p)
{
    QDomElement surface = domdocument->createElement("surface");
    
    surface.setAttribute("name",p->name());
    
    int r,g,b,a=0;
    p->color()->getRgb(&r,&g,&b,&a);
    QString color ;
    QTextStream(&color) << QString::number(r) << " " 
			<< QString::number(g) << " " 
			<< QString::number(b) ;
    surface.setAttribute("color", color);
    
    if(QImplicitSurface * impSurface = dynamic_cast<QImplicitSurface *>(p)){
	
	surface.setAttribute("type","implicit");
	
	QDomElement domain = domdocument->createElement("domain");
	QString domainStr;
	QTextStream(&domainStr) << QString::number(impSurface->boundingbox()->xmin()) << " " 
				<< QString::number(impSurface->boundingbox()->xmax()) << " " 
				<< QString::number(impSurface->boundingbox()->ymin()) << " " 
				<< QString::number(impSurface->boundingbox()->ymax()) << " "
				<< QString::number(impSurface->boundingbox()->zmin()) << " " 
				<< QString::number(impSurface->boundingbox()->zmax()) ; 
	QDomText domainVal = domdocument->createTextNode(domainStr);
	domain.appendChild(domainVal);
	
	surface.appendChild(domain);
	
	QDomElement polynomial = domdocument->createElement("polynomial");
	polynomial.setAttribute("variables","x y z");
	QString polyStr = impSurface->equation();
	QDomText polyVal = domdocument->createTextNode(polyStr);
	polynomial.appendChild(polyVal);

	surface.appendChild(polynomial);
	
     }

    if(QRationalSurface * rationalSurface = dynamic_cast<QRationalSurface *>(p)){
	
	surface.setAttribute("type","rational");
	
	QDomElement domain = domdocument->createElement("domain");
	QString domainStr;
	QTextStream(&domainStr) << QString::number(rationalSurface->umin()) << " " 
				<< QString::number(rationalSurface->umax()) << " "
				<< QString::number(rationalSurface->vmin()) << " " 
				<< QString::number(rationalSurface->vmax()) ; 
	QDomText domainVal = domdocument->createTextNode(domainStr);
	domain.appendChild(domainVal);

	surface.appendChild(domain);

	QDomElement grid = domdocument->createElement("grid");
	QString gridStr;
	QTextStream(&gridStr) << QString::number(rationalSurface->gridU()) << " " 
			      << QString::number(rationalSurface->gridV()) ;
	QDomText gridVal = domdocument->createTextNode(gridStr);
	grid.appendChild(gridVal);

	surface.appendChild(grid);


	QDomElement polynomial1 = domdocument->createElement("polynomial");
	polynomial1.setAttribute("variables","s t");
	QString polyStr1 = rationalSurface->px();
	QDomText polyVal1 = domdocument->createTextNode(polyStr1);
	polynomial1.appendChild(polyVal1);

	surface.appendChild(polynomial1);

	QDomElement polynomial2 = domdocument->createElement("polynomial");
	polynomial2.setAttribute("variables","s t");
	QString polyStr2 = rationalSurface->py();
	QDomText polyVal2 = domdocument->createTextNode(polyStr2);
	polynomial2.appendChild(polyVal2);

	surface.appendChild(polynomial2);

	QDomElement polynomial3 = domdocument->createElement("polynomial");
	polynomial3.setAttribute("variables","s t");
	QString polyStr3 = rationalSurface->pz();
	QDomText polyVal3 = domdocument->createTextNode(polyStr3);
	polynomial3.appendChild(polyVal3);

	surface.appendChild(polynomial3);

	QDomElement polynomial4 = domdocument->createElement("polynomial");
	polynomial4.setAttribute("variables","s t");
	QString polyStr4 = rationalSurface->pw();
	QDomText polyVal4 = domdocument->createTextNode(polyStr4);
	polynomial4.appendChild(polyVal4);

	surface.appendChild(polynomial4);

    }

    if(QBSplineSurface * bspSurface = dynamic_cast<QBSplineSurface *>(p)){
	
	surface.setAttribute("type","bspline");
	
	QDomElement dimension = domdocument->createElement("dimension");
	QString dimensionStr;
	QTextStream(&dimensionStr) << QString::number(bspSurface->idim);
	QDomText dimensionVal = domdocument->createTextNode(dimensionStr);
	dimension.appendChild(dimensionVal);
	
	surface.appendChild(dimension);

	QDomElement number = domdocument->createElement("number");
	QString numberStr;
	QTextStream(&numberStr) << QString::number(bspSurface->number1) << " " 
				<< QString::number(bspSurface->number2);  
	QDomText numberVal = domdocument->createTextNode(numberStr);
	number.appendChild(numberVal);
	
	surface.appendChild(number);
	
	QDomElement order = domdocument->createElement("order");
	QString orderStr;
	QTextStream(&orderStr) << QString::number(bspSurface->order1) << " " 
			       << QString::number(bspSurface->order2);
	QDomText orderVal = domdocument->createTextNode(orderStr);
	order.appendChild(orderVal);
	
	surface.appendChild(order);
	
	QDomElement knots1 = domdocument->createElement("knots");
	QString knots1Str;
	double * surfaceKnots1 = bspSurface->knots1;
	if(surfaceKnots1 != NULL)
	     cout << "surfaceKnots1 is not NULL" << endl;
	int i=0;
	while (i < bspSurface->number1 + bspSurface->order1){
	    QTextStream(&knots1Str) << QString::number(*(surfaceKnots1 + i)) << " ";
	    i++;
	}
	QDomText knots1Val = domdocument->createTextNode(knots1Str);
	knots1.appendChild(knots1Val);
	
	surface.appendChild(knots1);
	
	QDomElement knots2 = domdocument->createElement("knots");
	QString knots2Str;
	double * surfaceKnots2 = bspSurface->knots2;
	if(surfaceKnots2 != NULL)
	     cout << "surfaceKnots2 is not NULL" << endl;
	i=0;
	while (i < bspSurface->number2 + bspSurface->order2){
	    QTextStream(&knots2Str) << QString::number(*(surfaceKnots2 + i)) << " ";
	    i++;
	}
	QDomText knots2Val = domdocument->createTextNode(knots2Str);
	knots2.appendChild(knots2Val);
	
	surface.appendChild(knots2);


	QDomElement points = domdocument->createElement("points");
	QString pointsStr;
	QList<QPoint3f *> * surfacePoints = &bspSurface->points;
	
	for(QList<QPoint3f *>::iterator itl = surfacePoints->begin();itl != surfacePoints->end();itl++){
	    
	    QTextStream(&pointsStr) 
		<< QString::number((*itl)->x()) << " " 
		<< QString::number((*itl)->y()) << " " 
		<< QString::number((*itl)->z()) 
		<< (itl + 1 == surfacePoints->end() ? "" : "\n\t") ;
	}
	
	QDomText pointsVal = domdocument->createTextNode(pointsStr);
	points.appendChild(pointsVal);
	
	surface.appendChild(points);
	
    }


    elements.append(surface);
    

}

void OStreamAxl::createMeshElement(QMesh* p)
{
    if( p->meshType() == QMesh::Surface){
	
	QDomElement surface = domdocument->createElement("surface");
	
	surface.setAttribute("name",p->name());
	
	int r,g,b,a=0;
	p->color()->getRgb(&r,&g,&b,&a);
	QString color ;
	QTextStream(&color) << QString::number(r) << " " 
			    << QString::number(g) << " " 
			    << QString::number(b) ;
	surface.setAttribute("color", color);
	
	surface.setAttribute("type","mesh");
	
	QDomElement off = domdocument->createElement("off");
	QString offStr = "OFF\n";

	QTextStream(&offStr) << QString::number(p->vertices().size()) << " " 
			     << QString::number(p->faces().size()) << " " 
			     << QString::number(0) << "\n\n" ;

	for(QList<QVertex *>::iterator itl = p->vertices().begin();itl != p->vertices().end();itl++){
	    
	    QTextStream(&offStr) << QString::number((*itl)->x()) << " " 
				 << QString::number((*itl)->y()) << " " 
				 << QString::number((*itl)->z()) << "\n" ;
	}


	for(QList<QFace *>::iterator itl = p->faces().begin();itl != p->faces().end();itl++){
	    
	    QTextStream(&offStr) << "3" << " " 
				 << QString::number((*itl)->indexes_[0]) << " " 
				 << QString::number((*itl)->indexes_[1]) << " " 
				 << QString::number((*itl)->indexes_[2]) << "\n" ;
	}

	QDomText offVal = domdocument->createTextNode(offStr);
	off.appendChild(offVal);
	
	surface.appendChild(off);	
	
	elements.append(surface);
}


    if(p->meshType() == QMesh::Curve){
	
	QDomElement curve = domdocument->createElement("curve");
	
	curve.setAttribute("name",p->name());
	
	int r,g,b,a=0;
	p->color()->getRgb(&r,&g,&b,&a);
	QString color ;
	QTextStream(&color) << QString::number(r) << " " 
			    << QString::number(g) << " " 
			    << QString::number(b) ;
	curve.setAttribute("color", color);
	
	curve.setAttribute("type","mesh");
	
	QDomElement vect = domdocument->createElement("vect");
	QString vectStr = "VECT\n";

	QTextStream(&vectStr) << QString::number(p->edges().size()) << " " 
			      << QString::number(p->vertices().size()) << " " 
			      << QString::number(p->edges().size()) << "\n" ;

	QTextStream(&vectStr) << "Just something to satisfy the format\n"
			      << "Just something to satisfy the format\n" ;

	for(QList<QVertex *>::iterator itl = p->vertices().begin();itl != p->vertices().end();itl++){
	    
	    QTextStream(&vectStr) << QString::number((*itl)->x()) << " " 
				  << QString::number((*itl)->y()) << " " 
				  << QString::number((*itl)->z()) << " " ;

	    itl++ ;

	    QTextStream(&vectStr)  << QString::number((*itl)->x()) << " " 
				   << QString::number((*itl)->y()) << " " 
				   << QString::number((*itl)->z()) << "\n";
	}


	for(QList<QEdge *>::iterator itl = p->edges().begin();itl != p->edges().end();itl++){
	    
	    QTextStream(&vectStr) << QString::number((*itl)->color()->redF()) << " " 
				  << QString::number((*itl)->color()->greenF()) << " " 
				  << QString::number((*itl)->color()->blueF()) << " " 
				  << QString::number((*itl)->color()->alphaF()) << "\n" ;
	}

	QDomText vectVal = domdocument->createTextNode(vectStr);
	vect.appendChild(vectVal);
	
	curve.appendChild(vect);	
	

	elements.append(curve);

}

}

void OStreamAxl::createScriptElement(void)
{
    QDomElement script = domdocument->createElement("script");

    QString scriptStr = mainwindow->scriptingDock()->text();
    QDomText scriptVal = domdocument->createTextNode(scriptStr);
    script.appendChild(scriptVal);
    
    elements.append(script);
}

void OStreamAxl::createViewerElement(void)
{
     elements.append(viewer->camera()->domElement("camera", *domdocument)) ;
}

void OStreamAxl::buildDomTree(void)
{
    QDomElement axl = domdocument->createElement("axl");
    domdocument->appendChild(axl);
    
    cout << __func__ << FG_RED << "elements.size() : " << elements.size() << NOCOLOR << endl ;

    foreach(QDomElement e, elements)
	axl.appendChild(e) ;
}

void OStreamAxl::save()
{
    QFile file(filename);
    if( !file.open(QIODevice::WriteOnly | QIODevice::Text)) {
	QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
	return ;
    }
    
    QTextStream ts( &file );
    ts << domdocument->toString();
    file.close();
}

} ; // namespace QStream

//  LocalWords:  itl
