/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QSynapsInterface
 * 2006-05-13
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006-2007 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# ifndef QSYNAPSINTERFACE_H
# define QSYNAPSINTERFACE_H

# include <QObject>
# include <QList>
# include <QStringList>

# include <QAxel/QInitializablePluginInterface.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>

# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>

# include <QData/QBoundingBox4D.h>
# include <QData/QCell.h>
# include <QData/QCell4D.h>
# include <QData/QRegion3f.h>
# include <QData/QRegionVertex.h>
# include <QData/QRegionEdge.h>

using namespace QAxel ;
using namespace QData ;

/****************************************************************************/
/*!
  \class QSynapsInterface QSynapsInterface.h
  \brief The class QSynapsInterface provides a link between Axel and Synaps
  for algebraic computations.

  More ...
*/
/****************************************************************************/
class QSynapsInterface : public QAxel::QInitializablePluginInterface
{
public :
    virtual ~QSynapsInterface() {}
    
    virtual QMesh * Mesh(QRationalCurve * c) = 0 ;
    virtual QMesh * Mesh(QRationalSurface * s) = 0 ;
    virtual QMesh * Mesh(QImplicitCurve2D * c) = 0 ;
    virtual QMesh * Mesh(QImplicitCurve3D * c) = 0 ;
    virtual QMesh * Mesh(QImplicitSurface * s) = 0 ;
    
    virtual bool xmonotonic(QImplicitCurve2D * c, QCell * bb) = 0 ;
    virtual bool ymonotonic(QImplicitCurve2D * c, QCell * bb) = 0 ;
    
    virtual QList<QPoint3f *> 		xtremPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-3) = 0 ;
    virtual QList<QPoint3f *> 		ytremPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-3) = 0 ;
    
    virtual QList<QPoint3f *> 		xtremPoints(QImplicitSurface * s, QCell * bb, double epsilon = 1e-3) = 0 ;
    virtual QList<QPoint3f *> 		ytremPoints(QImplicitSurface * s, QCell * bb, double epsilon = 1e-3) = 0 ;
    virtual QList<QPoint3f *> 		ztremPoints(QImplicitSurface * s, QCell * bb, double epsilon = 1e-3) = 0 ;
    
    // virtual QList<QPoint3f *> 		xtremPoints(QImplicitCurve4D * c, double xmin, double xmax, double ymin, double ymax, double epsilon = 1e-3) = 0 ;
    // virtual QList<QPoint3f *> 		ytremPoints(QImplicitCurve4D * c, double xmin, double xmax, double ymin, double ymax, double epsilon = 1e-3) = 0 ;
    
    virtual QList<QPoint3f *> 		singularPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-3) = 0 ;
    
    virtual QList<QPoint3f *> 		intersectionPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-6) = 0 ;
    virtual QList<QPoint3f *> 		intersectionPoints(QImplicitCurve2D * c, QCell * bb, int axis, int order, double epsilon = 1e-6) = 0 ;
    virtual QList<QPoint4f *> 		intersectionPoints4D(QImplicitCurve4D * c, QBoundingBox4D * bb, int faceBorder[2], double epsilon = 1e-6) = 0 ;
    virtual QPoint3f * 				map2D3D(float s, float t, QRationalSurface * s1) = 0 ;
    virtual QList<QPoint3f *>		intersection(QImplicitCurve2D * c1, QImplicitCurve2D * c2, QBoundingBox * bb, double espilon = 1e-6) = 0 ;
    
    // virtual QList<QPoint3f *> 		intersectionPoints(QImplicitCurve4D * c, float xmin, float xmax, float ymin, float ymax, double epsilon = 1e-6) = 0 ;
    // virtual bool 					regularPos(QImplicitCurve4D * c, float xmin, float xmax, float ymin, float ymax, double epsilon = 1e-6) = 0 ;
    
    virtual int						numberOfBranches(QPoint3f * p, QImplicitCurve2D * c, QCell * bb) = 0 ;
    
    virtual QImplicitCurve2D *		computeFirstDerivativeCurve(QImplicitCurve2D * c, QString variable) = 0 ;
    virtual QImplicitCurve2D *		computeSecondDerivativeCurve(QImplicitCurve2D * c, QString variable) = 0 ;
    virtual QImplicitCurve2D *		computeJacobiCurve(QImplicitCurve2D * c1, QImplicitCurve2D * c2) = 0 ;
    virtual QImplicitCurve2D * 		computeHessianCurve(QImplicitCurve2D * c) = 0 ;
    
    virtual QImplicitCurve4D *		computeIntersectionEquation4D(QRationalSurface * s1, QRationalSurface * s2) = 0 ;
    virtual bool isLocallyInjective4D(QImplicitCurve4D * c, QBoundingBox4D * bb, int i) = 0 ;
    virtual bool isEdgeMonotonic4D(QString f, char * var, int i, QBoundingBox4D * bb) = 0 ;
    virtual bool isBorderMonotonic4D(QString f, QString g, char * var, int i, QBoundingBox4D * bb) = 0;
    
    virtual bool isOn(QPoint3f * p, QImplicitCurve2D * c) = 0 ;
    virtual bool isOn(QPoint3f * p, QImplicitCurve2D * c, double tolerance) = 0 ;
    
    virtual int degree(QImplicitCurve2D * c, int var) = 0 ;
    virtual int binomial(int n, int p) =  0;
    
    virtual void toBezierArray(QImplicitCurve2D * s, float * array, int * deg_x, int * deg_y) = 0 ;
    virtual void toBezierArray(QImplicitSurface * s, float * array, int * deg_x, int * deg_y) = 0 ;
    
    virtual QString toLinearMonomialEquation(float * l, float xmin, float xmax, float ymin, float ymax) = 0 ;
    virtual QString toQuadraticMonomialEquation(float * l, float xmin, float xmax, float ymin, float ymax) = 0 ;

    virtual QList<QPoint3f *> intersection(QRegionEdge * e1, QRegionEdge * e2, QBoundingBox * bb, double espilon = 1e-6) = 0 ;
    virtual QList<QPoint3f *> intersectionPoints(QRegionEdge * e, QCell * bb, double epsilon = 1e-6) = 0 ;

    virtual QList<QPoint3f *> intersectionX(QRegionEdge * e, float y, float xmin, float xmax) = 0 ;
    virtual QList<QPoint3f *> intersectionY(QRegionEdge * e, float x, float ymin, float ymax) = 0 ;
} ;

Q_DECLARE_INTERFACE(QSynapsInterface, "Axel::Interfaces::Synaps/0.11")

# endif
