/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QSynapsPlugin
 * 2006-05-13
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QSYNAPSPLUGIN_H
# define QSYNAPSPLUGIN_H

# include <QSynaps/QSynapsInterface.h>

using namespace QAxel ;

/****************************************************************************/
/*!
  \class QSynapsPlugin QSynapsPlugin.h
  \brief The class QSynapsPlugin implements the \link QSynapsInterface \endlink
  as a glue to the Synaps library.

  More ...
*/
/****************************************************************************/
class QSynapsPlugin : public QObject, public QSynapsInterface
{
    Q_OBJECT
    Q_INTERFACES(QSynapsInterface QAxel::QPluginInterface QAxel::QInitializablePluginInterface)

public:
    QString name() const ;
    QStringList features() const ;

    void init(MainWindow * window) ;
	
    QMesh * Mesh(QImplicitCurve2D * c) ;
    QMesh * Mesh(QImplicitCurve3D * c) ;
    QMesh * Mesh(QImplicitSurface * s) ;
    QMesh * Mesh(QRationalCurve * c) ;
    QMesh * Mesh(QRationalSurface * s) ;
	
    bool xmonotonic(QImplicitCurve2D * c, QCell * bb) ;
    bool ymonotonic(QImplicitCurve2D * c, QCell * bb) ;
	
    QList<QPoint3f *> xtremPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-3) ;
    QList<QPoint3f *> ytremPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-3) ;

    QList<QPoint3f *> xtremPoints(QImplicitSurface * s, QCell * bb, double epsilon = 1e-3) ;
    QList<QPoint3f *> ytremPoints(QImplicitSurface * s, QCell * bb, double epsilon = 1e-3) ;
    QList<QPoint3f *> ztremPoints(QImplicitSurface * s, QCell * bb, double epsilon = 1e-3) ;
	
    QList<QPoint3f *> singularPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-3) ;
    
    QList<QPoint3f *> intersectionPoints(QImplicitCurve2D * c, QCell * bb, double epsilon = 1e-6) ;
    QList<QPoint3f *> intersectionPoints(QImplicitCurve2D * c, QCell * bb, int axis, int order, double epsilon = 1e-6) ;
    QList<QPoint4f *> intersectionPoints4D(QImplicitCurve4D * c, QBoundingBox4D * bb, int faceBorder[2], double epsilon = 1e-6) ;
	QPoint3f * map2D3D(float s, float t, QRationalSurface * s1) ;
    QList<QPoint3f *> intersection(QImplicitCurve2D * c1, QImplicitCurve2D * c2, QBoundingBox * bb, double espilon = 1e-6) ;
    
    int numberOfBranches(QPoint3f * p, QImplicitCurve2D * c, QCell * c) ;
	
    QImplicitCurve2D * computeFirstDerivativeCurve(QImplicitCurve2D * c, QString variable) ;
    QImplicitCurve2D * computeSecondDerivativeCurve(QImplicitCurve2D * c, QString variable) ;
    QImplicitCurve2D * computeJacobiCurve(QImplicitCurve2D * c1, QImplicitCurve2D * c2) ;
    QImplicitCurve2D * computeHessianCurve(QImplicitCurve2D * c) ;
	
    bool isOn(QPoint3f * p, QImplicitCurve2D * c) ;
    bool isOn(QPoint3f * p, QImplicitCurve2D * c, double tolerance) ;

    int degree(QImplicitCurve2D * c, int var) ;
    int binomial(int n, int p);

    void toBezierArray(QImplicitCurve2D * s, float * array, int * deg_x, int * deg_y) ;
    void toBezierArray(QImplicitSurface * s, float * array, int * deg_x, int * deg_y) ;

    QString toLinearMonomialEquation(float * l, float xmin, float xmax, float ymin, float ymax) ;
    QString toQuadraticMonomialEquation(float * l, float xmin, float xmax, float ymin, float ymax) ;

    QImplicitCurve4D * computeIntersectionEquation4D(QRationalSurface * s1, QRationalSurface * s2) ;
    bool isLocallyInjective4D(QImplicitCurve4D * c, QBoundingBox4D * bb, int i) ;
    bool isEdgeMonotonic4D(QString f, char * var, int i, QBoundingBox4D * bb) ;
    bool isBorderMonotonic4D(QString f, QString g, char * var, int i, QBoundingBox4D * bb) ;

    QList<QPoint3f *> intersection(QRegionEdge * e1, QRegionEdge * e2, QBoundingBox * bb, double espilon = 1e-6) ;
    QList<QPoint3f *> intersectionPoints(QRegionEdge * e, QCell * bb, double epsilon = 1e-6) ;

    QList<QPoint3f *> intersectionX(QRegionEdge * e, float y, float xmin, float xmax) ;
    QList<QPoint3f *> intersectionY(QRegionEdge * e, float x, float ymin, float ymax) ;

    QList<QPoint3f *> intersectionX(QImplicitCurve2D * c, float y, float xmin, float xmax) ;
    QList<QPoint3f *> intersectionY(QImplicitCurve2D * c, float x, float ymin, float ymax) ;

private:
    bool lineIntersection(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4, float * x, float * y) ;
} ;

# endif
