########################################################
# A x e l MacOSX Makefile
# Julien Wintz
# Inria Sophia-Antipolis
########################################################

QTDIR=/usr/local/Trolltech/Qt-4.3.0
QSADIR=/usr/local/Trolltech/Qsa-1.2.2
GROWLDIR=/Library/Frameworks
SPARKLEDIR=/Library/Frameworks
AXEL_DIR=../

NAME=Axel
VERSION=0.3.4
FILES=Axel.app 
DISK_DIR=../disk
ifneq ($(BUILD),no)
DISK_FILES=$(foreach f,$(FILES), $(addprefix $(DISK_DIR)/,$(f)))
endif
MASTER_DMG=axel-modeler_$(VERSION)-1.i386.dmg

# Other variables
QT_FRAMEWORK_VERSION=4
QT_FRAMEWORKS=QtCore QtXml QtNetwork QtGui QtSql Qt3Support QtOpenGL
AXEL_PLUGINS=QSisl QCore QSynaps

# The rules
.PHONY: buildall
all: build-axel $(DISK_FILES)

.PHONY: dmg
dmg: build-axel $(DISK_FILES) $(MASTER_DMG)

# Cleanup
.PHONY: clean
clean: clean-dmg
	-rm -rf $(DISK_DIR)

# Help
.PHONY: help
help:
	@echo "Usage:"
	@echo "    make : Build a distributable binary in $(DISK_DIR)/"
	@echo "    make dmg : Build a DMG disk image"

# Building the main source tree
.PHONY: build-axel
ifneq ($(BUILD),no)
build-axel:
	@echo
	@echo --------------------- Building Source Tree --------------------
	$(MAKE) -C $(AXEL_DIR)/src
	@echo
else
build-axel:
endif


################################################################################
# Generation of the final files
################################################################################

# The binary
$(DISK_DIR)/Axel.app: $(AXEL_DIR)/src/Axel.app
	@echo
	@echo --------------------- Packaging Binary --------------------
	rm -rf "$@"
	ditto -rsrc "$<" "$@"
	mkdir -p "$@/Contents/Frameworks"
	ditto -rsrc "$(GROWLDIR)/Growl.framework" "$@/Contents/Frameworks/Growl.framework"
	ditto -rsrc "$(GROWLDIR)/Sparkle.framework" "$@/Contents/Frameworks/Sparkle.framework"
	@echo
	@echo Installing QSA ...
	cp -rf "$(QSADIR)/lib/libqsa.1.2.1.dylib" "$@/Contents/Frameworks/"
	ln -s "libqsa.1.2.1.dylib" "$@/Contents/Frameworks/libqsa.dylib" 
	ln -s "libqsa.1.2.1.dylib" "$@/Contents/Frameworks/libqsa.1.dylib"
	ln -s "libqsa.1.2.1.dylib" "$@/Contents/Frameworks/libqsa.1.2.dylib"
	install_name_tool -id "@executable_path/../Frameworks/libqsa.1.dylib" "$@/Contents/Frameworks/libqsa.1.dylib"
	install_name_tool -change "libqsa.1.dylib" "@executable_path/../Frameworks/libqsa.1.dylib" "$@/Contents/MacOS/Axel"
	install_name_tool -change "$(QSADIR)/lib/libqsa.1.dylib" "@executable_path/../Frameworks/libqsa.1.dylib" "$@/Contents/MacOS/Axel"
	@echo 
	@echo Installing Plugins
	mkdir -p "$@/Contents/Plugins"
	for e in $(AXEL_PLUGINS); do \
	cp -f "$(AXEL_DIR)/plugins/lib$$e.dylib" "$@/Contents/Plugins/"; \
	done
	@echo
	@echo Installing Qt Frameworks ...
	for f in $(QT_FRAMEWORKS); do \
		ditto -rsrc "$(QTDIR)/lib/$$f.framework" "$@/Contents/Frameworks/$$f.framework"; \
		rm -rf "$@/Contents/Frameworks/$$f.framework/Headers"; \
		rm -rf "$@/Contents/Frameworks/$$f.framework/$$f""_debug"; \
		rm -rf "$@/Contents/Frameworks/$$f.framework/$$f""_debug.prl"; \
		rm -rf "$@/Contents/Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/Headers"; \
		rm -rf "$@/Contents/Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f""_debug"; \
		rm -rf "$@/Contents/Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f""_debug.prl"; \
		install_name_tool -id "@executable_path/../Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "$@/Contents/Frameworks/$$f.framework/$$f"; \
		for g in $(QT_FRAMEWORKS); do \
			install_name_tool -change "$(QTDIR)/lib/$$g.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$g" "@executable_path/../Frameworks/$$g.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$g" "$@/Contents/Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f"; \
		done; \
		for h in $(AXEL_PLUGINS); do \
			install_name_tool -change "$(QTDIR)/lib/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "@executable_path/../Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "$@/Contents/Plugins/lib$$h.dylib"; \
		done; \
		install_name_tool -change "$(QTDIR)/lib/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "@executable_path/../Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "$@/Contents/Frameworks/libqsa.1.dylib"; \
		install_name_tool -change "$(QTDIR)/lib/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "@executable_path/../Frameworks/$$f.framework/Versions/$(QT_FRAMEWORK_VERSION)/$$f" "$@/Contents/MacOS/Axel"; \
	done
	@echo
	@echo Installing Resources ...
	cp -rf ../resources/Info.plist "$@/Contents/"
	cp -rf ../resources/axel_icon.icns "$@/Contents/Resources/"
	cp -rf ../resources/docs_icon.icns "$@/Contents/Resources/"

# The rest
$(DISK_DIR)/%: $(AXEL_DIR)/%
	ditto -rsrc "$<" $@


################################################################################
# Disk image creation
################################################################################

WC_DMG=wc.dmg
WC_DIR=wc
TEMPLATE_DMG=template.dmg

$(TEMPLATE_DMG): $(TEMPLATE_DMG).bz2
	bunzip2 -k $<

$(TEMPLATE_DMG).bz2: 
	@echo
	@echo --------------------- Generating empty template --------------------
	mkdir template
	hdiutil create -size 80m "$(TEMPLATE_DMG)" -srcfolder template -format UDRW -volname "$(NAME)" -quiet
	rmdir template
	bzip2 "$(TEMPLATE_DMG)"
	@echo

$(WC_DMG): $(TEMPLATE_DMG)
	cp $< $@

$(MASTER_DMG): $(WC_DMG) $(DISK_FILES)
	@echo
	@echo --------------------- Creating Disk Image --------------------
	mkdir -p $(WC_DIR)
	hdiutil attach "$(WC_DMG)" -noautoopen -quiet -mountpoint "$(WC_DIR)"
	for i in $(DISK_DIR)/*; do  \
		file=`basename $$i`; \
		rm -rf "$(WC_DIR)/$$file"; \
		ditto -rsrc "$(DISK_DIR)/$$file" "$(WC_DIR)/$$file"; \
	done
	#rm -f "$@"
	#hdiutil create -srcfolder "$(WC_DIR)" -format UDZO -imagekey zlib-level=9 "$@" -volname "$(NAME) $(VERSION)" -scrub -quiet
	WC_DEV=`hdiutil info | grep "$(WC_DIR)" | grep "Apple_HFS" | awk '{print $$1}'` && \
	hdiutil detach $$WC_DEV -quiet -force
	rm -f "$(MASTER_DMG)"
	hdiutil convert "$(WC_DMG)" -quiet -format UDZO -imagekey zlib-level=9 -o "$@"
	rm -rf $(WC_DIR)
	-hdiutil internet-enable -yes -quiet "$@"
	@if [ ! -e $(AXEL_DIR)/packages ]; then mkdir $(AXEL_DIR)/packages; fi
	@mv "$@" $(AXEL_DIR)/packages
	@echo

.PHONY: clean-dmg
clean-dmg:
	-rm -rf $(TEMPLATE_DMG) $(TEMPLATE_DMG).bz2 $(MASTER_DMG) $(WC_DMG) $(NAME)-*.dmg
